//=============================================================================
//	authors: Steve Parker
//
//  $Id: BooleanType.java,v 1.3 2000/10/25 20:50:06 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: BooleanType.java,v $
//  Revision 1.3  2000/10/25 20:50:06  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.2  2000/09/18 20:41:02  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//
//=============================================================================

package ROM;

import java.util.HashSet;

/** This represents the <B>BooleanType</B> object
 * @author <I>Steve Parker</I>
 * <a href="mailto:sparker@titan.com">sparker@titan.com</a>
 */
public class BooleanType extends Type {

    public static final Integer TRUE = new Integer(Integer.MAX_VALUE);
    public static final Integer FALSE = new Integer(Integer.MIN_VALUE);

	private static BunchValue elements;

	/** The primary Constructor
	 * @paramIn parentIn, parent of this object
	 * @return Not applicable
	 */
	public BooleanType(BaseObject parentIn) {
		super(parentIn);
		HashSet returnSet = new HashSet();
		returnSet.add(TRUE);
		returnSet.add(FALSE);
		elements = new BunchValue(null, this, returnSet);
	}

	/** To verify that some object has correct type
	 * @param valIn Object to be tested against type 
	 * @return boolean, true if object of correct type
	 */
	public static boolean isTypeOf(Object valIn) {
		return ((valIn instanceof Integer) && 
			((Integer)valIn == TRUE || (Integer)valIn == FALSE));
	}

	/** To get the set of all possible values of type
	 * Note: It may not always be possible to implement this set (infinite)
	 * @return BunchValue containing values
	 */
	public BunchValue getElements() {
		return elements;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		return "Boolean";
	}
	
}

