//=============================================================================
//	authors: Cindy Kong
//
//  $Id: ElementType.java,v 1.1 2000/10/25 20:50:06 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ElementType.java,v $
//  Revision 1.1  2000/10/25 20:50:06  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.util.HashSet;

/** This represents the <B>ElementType</B> object
 * @author <I>Cindy Kong</I>
 * <a href="mailto:ckong@ittc.ukans.edu">ckong@ittc.ukans.edu</a>
 */
public class ElementType extends Type {

	private static BunchValue elements;

	/** The primary Constructor
	 * @param parentIn The parent object
	 * @param elementsIn The bunch value obtained from all element types
	 * @return Not applicable
	 */
	public ElementType(BaseObject parentIn, BunchValue elementsIn) {
		super(parentIn);
		elements = elementsIn;
	}
	
	/** To verify that some object has correct type
	 * @param valIn Object to be tested against type 
	 * @return boolean, true if object of correct type
	 */
	public static boolean isTypeOf(Object valIn) {
		return (NumberType.isTypeOf(valIn) ||
				CharacterType.isTypeOf(valIn));
	}
	
	/** To add new elements to the Element type
	 * @param bunchIn The bunch to union
	 * @return void
	 */
	public void addBunch(BunchValue bunchIn) {
		elements.bunchConcat(bunchIn);
	}

	/** To get the set of all possible values of type
	 * Note: It may not always be possible to implement this set (infinite)
	 * @return BunchValue containing values
	 */
	public BunchValue getElements() {
		return elements;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		return "Element";
	}

}

