//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: FacetReference.java,v 1.7 2001/03/12 23:56:46 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: FacetReference.java,v $
//  Revision 1.7  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.6  2000/11/27 23:46:51  cvs
//  Changed the publishROM method in  all the classes and added some
//  predefined operators to the Universe. 
//  --srinivas
//
//  Revision 1.5  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.4  2000/10/25 20:50:06  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.3  2000/09/16 19:48:16  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.2  2000/08/10 00:49:48  cvs
//  Implemented the new hierarchy for ItemReference and added some
//  accessor methods.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:08  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 04:29:52  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 00:51:15  cvs
//  Skeletons for several added classes. -David
//
//
//=============================================================================
package ROM;

import java.io.*;
import java.util.*;
/** This represents the <B>FacetReference</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */

public class FacetReference 
extends ParametizedReference {

	/** The primary constructor
	 * @param parent The BaseObject parent
	 * @param itemIn The facet item being referred
	 * @return Not applicable
	 */
	public FacetReference(BaseObject parent, Item itemIn) {
		super(parent, itemIn);
	}

	public LabelledItem labelLookup(Label labelIn){
		return getParent().labelLookup(labelIn);
	}

	public void publishROM(PrintWriter pw){
		if (itemReferred instanceof LabelledItem){
			String str =
				((LabelledItem)itemReferred).getLabel().toString();
			pw.print(str);
			pw.print("(");
			ListIterator itemIter =
				getParameters().listIterator();
			if (itemIter != null){
				while (itemIter.hasNext()){
					((Expression)itemIter.next()).publishROM(pw);
					pw.print(",");
				}
			}
			pw.println(")");
		}
	}



}

		

