//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Formula.java,v 1.11 2001/03/12 23:56:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Formula.java,v $
//  Revision 1.11  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.10  2000/11/27 23:46:52  cvs
//  Changed the publishROM method in  all the classes and added some
//  predefined operators to the Universe. 
//  --srinivas
//
//  Revision 1.9  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.8  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.7  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.6  2000/10/25 20:50:06  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.5  2000/08/30 20:39:11  cvs
//  Changed Expression class to inherit from Value and amended all
//  other classes affected by this change. -David 
//
//  Revision 1.4  2000/08/17 20:48:22  cvs
//  Added some members to the literal classes.
//  Modified the lookup algorithm in Universe to remove an error.
//  -- Cindy
//
//  Revision 1.3  2000/08/11 15:42:50  cvs
//  Fleshed out a bit more of the Formula class: added members and
//  accessors. -David 
//
//  Revision 1.2  2000/08/10 01:04:12  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:08  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.2  2000/08/09 04:29:52  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 00:51:14  cvs
//  Skeletons for several added classes. -David
//
//
//=============================================================================
package ROM;
import java.io.*;
/** This represents the <B>Formula</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */

public class Formula 
extends Expression {

	private Expression leftOp;
	private Expression rightOp;
	private OperatorReference operator;

	//changed the constructor so that the param is an object of type BaseObject
	/** Primary constructor
	 * @param parent The BaseObject parent of an object of this class
	 * @return Not applicable
	 */
	public Formula(BaseObject parent, Expression leftOpIn, 
				   Expression rightOpIn, OperatorReference operatorIn) {
		super(parent);
		leftOp = leftOpIn;
		rightOp = rightOpIn;
		operator = operatorIn;
	}

	//Added by srinivas on 23rd Oct, 2000
	/**secondary constructor 
	 * @param parent The BaseObject parent of an object of this class
	 * @return Not applicable
	 */
	public Formula(BaseObject parent){
		super(parent);
	}

	
	/** To retrieve left operand
	 * @return Expression representing formula's left operand
	 */
	public Expression getLeftOp(){
	    return leftOp;
	}

	/** To retrieve right operand
	 * @return Expression representing formula's right operand
	 */
	public Expression getRightOp(){
	    return rightOp;
	}

	/** To retrieve formula's main operator
	 * @return FunctionReference representing main operator
	 */
	public OperatorReference getOperator(){
		return operator;
	}
    
	/**To set the left operand
	 * @param Expression
	 * @return void
	 */
	public void setLeftOp(Expression exp){
		leftOp = exp;
	}
	/**To set the right operand
	 * @param Expression
	 * @return void
	 */
	public void setRightOp(Expression exp){
		rightOp= exp;
	}

	/**To set the operator 
	 * @param OperatorReference
	 * @return void
	 */
	public void setOperator(OperatorReference funcRef){
		operator = funcRef;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		String toReturn = new String();
		// to be completed
		if (getLeftOp() != null) 
			toReturn = getLeftOp().toString();
		if (getOperator() != null)
			toReturn = toReturn + " " + getOperator().toString();
		if (getRightOp() != null)
			toReturn = toReturn + " " + getRightOp().toString();
		return toReturn;
	}

	//Added by Srinivas on Nov 12th, 2000
	/* To look up for the label.As there are no items declared here it
	 * looks for the label in its parent
	 * @param Label
	 * @return void
	 */
	public LabelledItem labelLookup(Label label2){
		return getParent().labelLookup(label2);
	}
	
	public void publishROM(PrintWriter pw){
		if (leftOp != null)
			leftOp.publishROM(pw);
		if (getOperator() != null){
			if (!((getOperator().getItemReferred().toString().equals("'"))||
				  (getOperator().getItemReferred().toString().equals("@"))))
				pw.print(" ");
			getOperator().publishROM(pw);
		}
		pw.print(" ");
		if (rightOp != null)
			rightOp.publishROM(pw);
	}

}

		


