//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: 
//
//  $Id: Function.java,v 1.7 2001/03/12 23:56:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Function.java,v $
//  Revision 1.7  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.io.*;
import java.util.*;

/**
 * <b>Function</b>:  Represents a Rosetta Function.
 *
 * @author <i>Steve Parker</i> 
 * (<a href="mailto:sparker@averstar.com">sparker@averstar.com</a>).
 */
public class Function extends Expression {

	private ItemList interfaceList;
	private Expression expression;

    /**
     * Primary constructor.
     *
     * @param parentIn Item representing the parent of this Function.
     * @return not applicable
     */
    public Function(BaseObject parentIn) {
		super(parentIn);
		interfaceList = new ItemList();
    }

	//added by srinivas on Nov 11th, 2000
	/** To add an interface
	 * @param interfaceItemIn The Item to be added
	 * @return void
	 */
	public void addInterfaceItem(Item interfaceItem) {
		interfaceList.addItem(interfaceItem);
	}
	
	/** To set the interface list
	 * @param listIn, ItemList to be set
	 * @return void
	 */
	public void setInterfaceItems(ItemList listIn) {
		interfaceList = listIn;
	}
	
	/** To get the interface list
	 * @return The ItemList containing the interfaces
	 */
	public ItemList getInterfaceItems() {
		return interfaceList;
	}

	/**To set the expression of the function
	 * @param Expression
	 * @return void
	 */
	public void setExpression(Expression expr){
		expression = expr;
	}
	
	/**To get the expression of the function
	 * @param void
	 * @param Expression
	 */
	public Expression getExpression(){
		return expression;
	}

   /** Look up for a label in the parent. Though there are items in
	* the item list, the scope of these items is only in this function 
    * @param label Label that is being searched for.
    * @return LabelledItem returned, corresponding to searched for Label.
    */
    public LabelledItem labelLookup(Label label) {
		ListIterator itemIter = getInterfaceItems().listIterator();
		if (itemIter != null){
			while (itemIter.hasNext()){
				LabelledItem item = (LabelledItem)itemIter.next();
				if (item.getLabel().equals(label))
					return item;
			}
		}
		return getParent().labelLookup(label);
    }

    /** To get the string representation of the object
     * @return String representing the string of this object   
     */
    public String toString() {
      //TBD:  Shouldn't it just get the itemString from its parent Item??
      return null;
    }
	public void publishROM(PrintWriter pw){
		if (toString() !=null)
			pw.print(toString());
	}

}
