//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: ItemReference.java,v 1.9 2001/03/12 23:56:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ItemReference.java,v $
//  Revision 1.9  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.8  2000/12/04 19:39:38  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  Class Item is now an abstract class.
//  -- Cindy
//
//  Revision 1.7  2000/11/27 23:46:52  cvs
//  Changed the publishROM method in  all the classes and added some
//  predefined operators to the Universe. 
//  --srinivas
//
//  Revision 1.6  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.5  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.4  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.3  2000/08/30 20:39:11  cvs
//  Changed Expression class to inherit from Value and amended all
//  other classes affected by this change. -David 
//
//  Revision 1.2  2000/08/10 00:49:48  cvs
//  Implemented the new hierarchy for ItemReference and added some
//  accessor methods.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:09  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.2  2000/08/09 04:29:53  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 00:51:15  cvs
//  Skeletons for several added classes. -David
//
//
//=============================================================================
package ROM;
import java.io.*;
/** This represents the <B>ItemReference</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */

public class ItemReference 
extends Expression {

	Item itemReferred;

	/** The primary constructor
	 * @param parent The base object parent
	 * @param itemIn The item being referred to 
	 * @return Not applicable
	 */
	public ItemReference(BaseObject parent, Item itemIn) {
		super(parent);
		itemReferred = itemIn;
	}

	/** Another constructor
	 * @param parent The base object parent
	 * @param itemIn The item being referred to 
	 * @return Not applicable
	 */
	public ItemReference() {
		this(null, null);
	}

	/** To get the item being referred
	 * @return Item being referred
	 */
	public Item getItemReferred() {
		return itemReferred;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		// to be completed
		return itemReferred.toString();
	}

	//Added by srinivas on Nov10th , 2000
	/* This function looks up for the label. If the Item referred here
	 * has the same label then the item is returned
	 * @param Label
	 * @return LabelledItem
	 */
	public LabelledItem labelLookup(Label labelIn){
		if (itemReferred instanceof LabelledItem){
			if (((LabelledItem)itemReferred).getLabel().equals(labelIn))
				return (LabelledItem)itemReferred;
		}
		return getParent().labelLookup(labelIn);
	}
	
	//Added by srinivas on Nov10th , 2000
	/* This function just prints the label of the item referred
	 * @param PrintWriter
	 * @return void
	 */
	public void publishROM(PrintWriter pw){
		if (itemReferred instanceof LabelledItem){
			String str =
				((LabelledItem)itemReferred).getLabel().toString();
			pw.print(str);
		}
	}

}

		

