//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Label.java,v 1.6 2001/03/12 23:56:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Label.java,v $
//  Revision 1.6  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.5  2000/11/27 23:46:52  cvs
//  Changed the publishROM method in  all the classes and added some
//  predefined operators to the Universe. 
//  --srinivas
//
//  Revision 1.4  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.3  2000/10/25 20:50:07  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.2  2000/08/22 18:40:17  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.1  2000/08/09 04:29:54  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.lang.String;

/** This represents the <B>Label</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class Label 
extends BaseObject {

	private String labelName;

	/** The primary Constructor
	 * @param parentIn The parent of this label.
	 * @param nameIn The string representing name of the label.
	 * @return Not applicable
	 */
	public Label(Item parentIn, String nameIn) {
		super(parentIn);
		labelName = nameIn;
	}
	
	
	/** The secondary Constructor
	 * @param nameIn The string representing name of the label.
	 * @return Not applicable
	 */
	public Label(String nameIn) {
		labelName = nameIn;
	}
	
	/** To check if this label equals another label
	 * @param labelIn The label to be compared to
	 * @return boolean, true if the strings of both labels are equal,
	 * false otherwise
	 */
	public boolean equals(Label labelIn) {
		return labelName.equalsIgnoreCase(labelIn.toString());
	}

    /** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		return labelName;
	}

	/* The labelLookup function returns null
	 */
	public LabelledItem labelLookup(Label label){
		return null;
	}
	
}
		


