//=============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy Kong
//
//  $Id: NaturalType.java,v 1.1 2000/10/25 20:50:07 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: NaturalType.java,v $
//  Revision 1.1  2000/10/25 20:50:07  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//
//
//=============================================================================

package ROM;

import java.util.HashSet;

/** This represents the <B>NaturalType</B> object
 * @author <I>Cindy Kong</I>
 * <a href="mailto:ckong@ittc.ukans.edu">ckong@ittc.ukans.edu</a>
 */
public class NaturalType extends Type {

	public static final Integer MAXINT = BooleanType.TRUE;
    public static final Integer ZERO = BitType.ZERO;

	private static BunchValue elements;

	/** The primary Constructor
	 * @param parentIn The parent object
	 * @return Not applicable
	 */
	public NaturalType(BaseObject parentIn) {
		super(parentIn);
		HashSet returnSet = new HashSet();
		returnSet.add(MAXINT);
		returnSet.add("NaturalTo");
		returnSet.add(ZERO);
		elements = new BunchValue(null, this, returnSet);
	}

	/** To verify that some object has correct type
	 * @param valIn Object to be tested against type 
	 * @return boolean, true if object of correct type
	 */
	public static boolean isTypeOf(Object valIn) {
		return ((valIn instanceof Integer) && 
			(((Integer)valIn).intValue() >= ZERO.intValue() 
			 || ((Integer)valIn).intValue() <= MAXINT.intValue()));
	}

	/** To get the set of all possible values in type
	 * Note: It may not always be possible to implement this set (infinite)
	 * @return BunchValue containing values
	 */
	public BunchValue getElements() {
		return elements;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		return "Natural";
	}

}

