//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: ParametizedReference.java,v 1.11 2001/03/12 23:56:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ParametizedReference.java,v $
//  Revision 1.11  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.10  2000/11/27 23:46:52  cvs
//  Changed the publishROM method in  all the classes and added some
//  predefined operators to the Universe. 
//  --srinivas
//
//  Revision 1.9  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.8  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.7  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.6  2000/10/25 20:50:07  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.5  2000/09/16 19:48:16  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.4  2000/08/17 20:48:22  cvs
//  Added some members to the literal classes.
//  Modified the lookup algorithm in Universe to remove an error.
//  -- Cindy
//
//  Revision 1.3  2000/08/10 16:42:56  cvs
//  Morphed member called parameters from ItemReferenceList to
//  ItemList; appropriately altered getParameters and addParameter
//  methods. -David 
//
//  Revision 1.2  2000/08/10 01:04:13  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/10 00:49:48  cvs
//  Implemented the new hierarchy for ItemReference and added some
//  accessor methods.
//  -- Cindy
//
//
//=============================================================================
package ROM;

import java.util.ListIterator;
import java.io.*;
/** This represents the <B>ParametizedRefence</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */

public abstract class ParametizedReference 
	extends ItemReference {
	
	private ExpressionList parameters;
	
	/** Primary constructor
	 * @param parent The BaseObject parent
	 * @param itemIn The Item being referred
	 * @return Not applicable
	 */
	public ParametizedReference(BaseObject parent, Item itemIn) {
		super(parent, itemIn);
		parameters = new ExpressionList(); 
	}

	/** To add a parameter
	 * @param paramIn The Expression representing parameter to be added
	 * @return void
	 */
	public void addParameter(Expression paramIn) {
		parameters.addItem(paramIn);
	}

	/** To get list of parameters
	 * @return The ExpressionList containing the parameters
	 */
	public ExpressionList getParameters() {
		return parameters;
	}
	
	/** To set list of parameters
	 * @param  The ExpressionList containing the parameters
	 */
	public void setParameters(ExpressionList itemsIn) {
		parameters = itemsIn;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		String toReturn = getItemReferred().toString();
		ListIterator paramIterator = parameters.listIterator();
		Expression paramReference;
		toReturn.concat("(");
		while (paramIterator.hasNext()) {
			if (paramIterator.hasPrevious())
				toReturn.concat(", ");
			paramReference = (Expression)paramIterator.next();
			toReturn.concat(paramReference.toString());
		}
		toReturn.concat(")");
		return toReturn;
	}
	
	public abstract void publishROM(PrintWriter pw);
	
	public abstract LabelledItem labelLookup(Label labelIn);

}

		





