//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: RanFunction.java,v 1.1 2000/12/07 01:26:38 cvs Exp $
//
//
//=============================================================================

package ROM;
import java.io.*;
import java.util.*;
/** This represents the <B>ArrayIntegerType</B> object
 * @author <I>Srinivas Akkipeddi</I>
 * <a href="mailto:akki@ittc.ukans.edu"></a>
 */

public class RanFunction extends Expression{
	
	private ItemList itemList;
	private Expression expr;

	/** The primary Constructor
	 * @param parentIn Item representing the parent of this Facet
	 * @return Not applicable
	 */
	public RanFunction(BaseObject parent){
		super(parent);
	}

	/** The function to set the ItemList 
	 * @param itemListIn The ItemList 
	 * @return void
	 */
	public void setItemList(ItemList itemListIn){
		itemList = itemListIn;
	}

	/**The function to get the ItemList
	 * @param void
	 * @return ItemList 
	 */
	public ItemList getItemList(){
		return itemList;
	}

	/** The function to set the ItemList 
	 * @param exprIn The Expression 
	 * @return void
	 */
	public void setExpression(Expression exprIn){
		expr = exprIn;
	}

	/**The function to get the ItemList
	 * @param void
	 * @return Expression 
	 */
	public Expression getExpression(){
		return expr;
	}

	/**The function to return the String
	 * @param void
	 * @return String
	 */
	public String toString(){
		String str = null;
		try{
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw,true);
			publishROM(pw);
			str = sw.toString();
			sw.close();
			pw.flush();
			pw.close();
		}catch(Exception e){
			System.out.println(e);
			return null;
		}
		return str;
	}

	/**The function to publish the Function
	 * @param PrintWriter 
	 * @return void
	 */
	public void publishROM(PrintWriter pw){
		pw.print("Ran ");
		pw.print("(");
		if (itemList != null){
			ListIterator itemIter = itemList.listIterator();
			if (itemIter != null){
				while (itemIter.hasNext()){
					pw.print(((Item)itemIter.next()).toString());
					pw.print(";");
				}
			}
		}
		if (expr != null){
			pw.print(" |");
			expr.publishROM(pw);
		}
		pw.print(")");
	}

	/** labelLookup
	 *
	 * @param labelIn - label to search for
	 * @return LabelledItem -   found in this scope
	 */
	public LabelledItem labelLookup(Label labelIn) {
		ListIterator itemIter = itemList.listIterator();
		if (itemIter != null){
			while (itemIter.hasNext()){
				LabelledItem item = (LabelledItem)itemIter.next();
				if
					(item.getLabel().toString().equals(labelIn.toString()))
					return item;
			}
		}
		return getParent().labelLookup(labelIn);
	}
}
