//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Universe.java,v 1.17 2001/03/21 00:07:11 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Universe.java,v $
//  Revision 1.17  2001/03/21 00:07:11  cvs
//  Added the "design" mode for parameters.
//  -- Cindy
//
//  Revision 1.16  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.15  2001/03/06 02:15:14  cvs
//  Added some accessor methods that are used in DVTG to the ROM classes
//
//  --srinivas
//
//  Revision 1.14  2001/02/01 22:37:33  cvs
//  Added some predefined operators to the Universe class
//  --srinivas
//
//  Revision 1.13  2001/02/01 01:14:05  cvs
//  Added some predefined operators to the Universe class.
//  --srinivas
//
//  Revision 1.12  2000/12/04 19:39:39  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  Class Item is now an abstract class.
//  -- Cindy
//
//  Revision 1.11  2000/12/03 23:59:47  cvs
//  Added the new types of array, set and sequence. The methods
//  getType  and alreadyDeclared are also added to get a type that has
//  been previously declared. 
//  --srinivas
//
//  Revision 1.10  2000/11/27 23:46:52  cvs
//  Changed the publishROM method in  all the classes and added some
//  predefined operators to the Universe. 
//  --srinivas
//
//  Revision 1.9  2000/11/17 23:14:14  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.8  2000/11/13 21:49:12  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.7  2000/11/06 19:10:55  cvs
//  Added initialization of list containing items.
//  -- Cindy
//
//  Revision 1.6  2000/11/06 17:50:58  cvs
//  Additional changes made as the loader is being built. A new type
//  (component type) is added to the Universe. The types are initialized
//  in universe.
//  -- Cindy
//
//  Revision 1.5  2000/10/25 20:50:08  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.4  2000/08/22 18:40:18  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.1  2000/08/09 04:29:55  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.util.ListIterator;
import java.util.HashSet;
import SOM.SOM_In;
import SOM.SOM_Out;
import SOM.SOM_InOut;
import SOM.SOM_Mode;
import SOM.SOM_Design;
import java.io.*;
import java.util.*;
/** This represents the <B>Universe</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class Universe extends BaseObject {

	private ItemList predefinedItems = new ItemList();
	private ItemList items = new ItemList();

	// Built-in types
	private static BooleanType bool = null;
	private static BitType bit = null;
	private static IntegerType integer = null;
	private static NaturalType natural = null;
	private static RationalType rational = null;
	private static RealType real = null;
	private static CharacterType character = null;
	private static NumberType number = null;
    private static ElementType element = null;
	private static UniversalType universal = null;

	// Skeleton of composite types
	public static PackageType pack = null;
	public static FacetType facet = null;
	public static ComponentType component = null;
	public static ArrayIntegerType arrayInt = null;
	public static ArrayRealType arrayReal = null;
	public static ArrayBitType arrayBit = null;
	public static ArrayBooleanType arrayBool = null;
	public static SetIntegerType setInt = null;
	public static SetRealType setReal = null;
	public static SetBitType setBit = null;
	public static SetBooleanType setBool = null;
	public static SequenceBooleanType sequenceBool = null;
	public static SequenceBitType sequenceBit = null;
	public static SequenceIntegerType sequenceInt = null;
	public static SequenceRealType sequenceReal = null;
	/** The primary Constructor
	 * @return Not applicable
	 */
	public Universe() {
		// Initializing some built-in types
		bool = new BooleanType(this);
		bit = new BitType(this);
		integer = new IntegerType(this);
		natural = new NaturalType(this);
		rational = new RationalType(this);
		real = new RealType(this);
		character = new CharacterType(this);
		BunchValue numberBunch = 
			BunchValue.bunchUnion(bool.getElements(), bit.getElements());
		numberBunch.bunchConcat(integer.getElements());
		numberBunch.bunchConcat(natural.getElements());
		numberBunch.bunchConcat(real.getElements());
		number = new NumberType(this, numberBunch);
		BunchValue elementBunch =
			BunchValue.bunchUnion(numberBunch, character.getElements());
	    element = new ElementType(this, elementBunch);
		elementBunch.setType(element);
		BunchValue universalBunch =
			new BunchValue(null, null, elementBunch.getElementSet().clone());
		universal = new UniversalType(this, universalBunch);
		universalBunch.setType(universal);
		BunchValue packageBunch = 
			new BunchValue(null, null, new HashSet());
		pack = new PackageType(this, packageBunch);
		packageBunch.setType(pack);
		BunchValue facetBunch =
			new BunchValue(null, null, new HashSet());
		facet = new FacetType(this, facetBunch);
		facetBunch.setType(facet);
		BunchValue componentBunch =
			new BunchValue(null, null, new HashSet());
		component = new ComponentType(this, componentBunch);
		componentBunch.setType(component);

		// Adding some more values to the Universal type
		universal.addBunch(packageBunch);
		universal.addBunch(facetBunch);
		universal.addBunch(componentBunch);

		BunchValue arrayIntBunch =
			new BunchValue(null, null, new HashSet());
		arrayInt = new ArrayIntegerType(this, arrayIntBunch);
		arrayIntBunch.setType(arrayInt);
		BunchValue arrayRealBunch =
			new BunchValue(null, null, new HashSet());
		arrayReal = new ArrayRealType(this, arrayRealBunch);
		arrayRealBunch.setType(arrayReal);
		BunchValue arrayBitBunch =
			new BunchValue(null, null, new HashSet());
		arrayBit = new ArrayBitType(this, arrayBitBunch);
		arrayBitBunch.setType(arrayBit);
		BunchValue arrayBoolBunch =
			new BunchValue(null, null, new HashSet());
		arrayBool = new ArrayBooleanType(this, arrayBoolBunch);
		arrayBoolBunch.setType(arrayBool);
		BunchValue setIntBunch =
			new BunchValue(null, null, new HashSet());
		setInt = new SetIntegerType(this, setIntBunch);
		setIntBunch.setType(setInt);
		BunchValue setRealBunch =
			new BunchValue(null, null, new HashSet());
		setReal = new SetRealType(this, setRealBunch);
		setRealBunch.setType(setReal);
		BunchValue setBitBunch =
			new BunchValue(null, null, new HashSet());
		setBit = new SetBitType(this, setBitBunch);
		setBitBunch.setType(setBit);
		BunchValue setBoolBunch =
			new BunchValue(null, null, new HashSet());
		setBool = new SetBooleanType(this, setBoolBunch);
		setBoolBunch.setType(setBool);
		BunchValue sequenceBoolBunch =
			new BunchValue(null, null, new HashSet());
		sequenceBool = new SequenceBooleanType(this, sequenceBoolBunch);
		sequenceBoolBunch.setType(sequenceBool);
		BunchValue sequenceBitBunch =
			new BunchValue(null, null, new HashSet());
		sequenceBit = new SequenceBitType(this, sequenceBitBunch);
		sequenceBitBunch.setType(sequenceBit);
		BunchValue sequenceIntBunch =
			new BunchValue(null, null, new HashSet());
		sequenceInt = new SequenceIntegerType(this, sequenceIntBunch);
		sequenceIntBunch.setType(sequenceInt);
		BunchValue sequenceRealBunch =
			new BunchValue(null, null, new HashSet());
		sequenceReal = new SequenceRealType(this, sequenceRealBunch);
		sequenceRealBunch.setType(sequenceReal);
		
		addPredefinedOperators();

	}
	
	//Added by srinivas on Nov17th, 2000
	/** To add all the predefined operators as ROM.LabelledItem to the
	 * list of predefined items in the universe.
	 * @param void
	 * @return void
	 */
	private void addPredefinedOperators() {
		
		//equal operator
		LabelledItem equalOp = new LabelledItem
			(this, new Label("="), null, "=");
		Function equalFunc = new Function (equalOp);
		equalOp.setValue(equalFunc);
		setPredefinedItem(equalOp);
		
		LabelledItem notequalOp = new LabelledItem
			(this, new Label("/="), null, "/=");
		Function notequalFunc = new Function (notequalOp);
		notequalOp.setValue(notequalFunc);
		setPredefinedItem(notequalOp);
		
		LabelledItem equalEqualOp = new LabelledItem(this,new
			Label("=="),null,"==");
		Function equalEqualFunc = new Function (equalEqualOp);
		equalEqualOp.setValue(equalEqualFunc);
		setPredefinedItem(equalEqualOp);
		
		LabelledItem plusOp = new LabelledItem(this,new
			Label("+"),null,"+");
		Function plusFunc = new Function (plusOp);
		plusOp.setValue(plusFunc);
		setPredefinedItem(plusOp);

		LabelledItem minusOp = new LabelledItem(this,new
			Label("-"),null,"-");
		Function minusFunc = new Function (minusOp);
		minusOp.setValue(minusFunc);
		setPredefinedItem(minusOp);

		LabelledItem impliesOp = new LabelledItem(this,new
			Label("=>"),null,"=>");
		Function impliesFunc = new Function (impliesOp);
		impliesOp.setValue(impliesFunc);
		setPredefinedItem(impliesOp);
		
		LabelledItem notOp = new LabelledItem(this,new
			Label("not"),null,"not");
		Function notFunc = new Function (notOp);
		notOp.setValue(notFunc);
		setPredefinedItem(notOp);

		LabelledItem tildeOp = new LabelledItem
			(this, new Label("~"), null, "~");
		Function tildeFunc = new Function (tildeOp);
		tildeOp.setValue(tildeFunc);
		setPredefinedItem(tildeOp);

		LabelledItem tickOp = new LabelledItem(this,new
			Label("'"),null,"'");
		Function tickFunc = new Function (tickOp);
		tickOp.setValue(tickFunc);
		setPredefinedItem(tickOp);

		LabelledItem atOp = new LabelledItem(this,new
			Label("@"), null, "@");
		Function atFunc = new Function (atOp);
		atOp.setValue(atFunc);
		setPredefinedItem(atOp);

		LabelledItem orOp = new LabelledItem(this,new
			Label("or"),null,"or");
		Function orFunc = new Function (orOp);
		orOp.setValue(orFunc);
		setPredefinedItem(orOp);

		LabelledItem norOp = new LabelledItem(this,new
			Label("nor"),null,"nor");
		Function norFunc = new Function (norOp);
		norOp.setValue(norFunc);
		setPredefinedItem(norOp);

		LabelledItem xorOp = new LabelledItem(this,new
			Label("xor"),null,"xor");
		Function xorFunc = new Function (xorOp);
		xorOp.setValue(xorFunc);
		setPredefinedItem(xorOp);

		LabelledItem xnorOp = new LabelledItem(this,new
			Label("xnor"),null,"xnor");
		Function xnorFunc = new Function (xnorOp);
		xnorOp.setValue(xnorFunc);
		setPredefinedItem(xnorOp);

		LabelledItem andOp = new LabelledItem(this,new
			Label("and"),null,"and");
		Function andFunc = new Function (andOp);
		andOp.setValue(andFunc);
		setPredefinedItem(andOp);

		LabelledItem nandOp = new LabelledItem(this,new
			Label("nand"),null,"nand");
		Function nandFunc = new Function (nandOp);
		nandOp.setValue(nandFunc);
		setPredefinedItem(nandOp);

		LabelledItem greaterOp = new LabelledItem(this,new
			Label(">"),null,">");
		Function greaterFunc = new Function (greaterOp);
		greaterOp.setValue(greaterFunc);
		setPredefinedItem(greaterOp);

		LabelledItem greaterEqualOp = new LabelledItem(this,new
			Label(">="),null,">=");
		Function greaterEqualFunc = new Function (greaterEqualOp);
		greaterEqualOp.setValue(greaterEqualFunc);
		setPredefinedItem(greaterEqualOp);
		
		LabelledItem lessOp = new LabelledItem(this,new
			Label("<"),null,"<");
		Function lessFunc = new Function (lessOp);
		lessOp.setValue(lessFunc);
		setPredefinedItem(lessOp);

		LabelledItem lessEqualOp = new LabelledItem(this,new
			Label("=<"),null,"=<");
		Function lessEqualFunc = new Function (lessEqualOp);
		lessEqualOp.setValue(lessEqualFunc);
		setPredefinedItem(lessEqualOp);

		LabelledItem plusPlusOp = new LabelledItem(this,new
			Label("++"),null,"++");
		Function plusPlusFunc = new Function (plusPlusOp);
		plusPlusOp.setValue(plusPlusFunc);
		setPredefinedItem(plusPlusOp);
		
		LabelledItem minusMinusOp = new LabelledItem(this,new
			Label("--"),null,"--");
		Function minusMinusFunc = new Function (minusMinusOp);
		minusMinusOp.setValue(minusMinusFunc);
		setPredefinedItem(minusMinusOp);

		LabelledItem multiplyOp = new LabelledItem(this,new
			Label("*"),null,"*");
		Function multiplyFunc = new Function (multiplyOp);
		multiplyOp.setValue(multiplyFunc);
		setPredefinedItem(multiplyOp);

		LabelledItem ifOp = new LabelledItem(this,new
			Label("if"),null,"if");
		Function ifFunc = new Function (ifOp);
		ifOp.setValue(ifFunc);
		setPredefinedItem(ifOp);

		LabelledItem percentOp = new LabelledItem
			(this, new Label("%"), null, "%");
		Function percentFunc = new Function(percentOp);
		percentOp.setValue(percentFunc);
		setPredefinedItem(percentOp);

		LabelledItem catenationOp = new LabelledItem
			(this, new Label(";;"), null, ";;");
		Function catenationFunc = new Function(catenationOp);
		catenationOp.setValue(catenationFunc);
		setPredefinedItem(catenationOp);

		LabelledItem nmaxOp = new LabelledItem
			(this, new Label("nmax"), null, "nmax");
		Function nmaxFunc = new Function(nmaxOp);
		nmaxOp.setValue(nmaxFunc);
		setPredefinedItem(nmaxOp);
		
		LabelledItem capOp = new LabelledItem
			(this, new Label("^"), null, "^");
		Function capFunc = new Function(capOp);
		capOp.setValue(capFunc);
		setPredefinedItem(capOp);
		
		LabelledItem minOp = new LabelledItem
			(this, new Label("min"), null, "min");
		Function minFunc = new Function(minOp);
		minOp.setValue(minFunc);
		setPredefinedItem(minOp);
		
		LabelledItem maxOp = new LabelledItem
			(this, new Label("max"), null, "max");
		Function maxFunc = new Function(maxOp);
		maxOp.setValue(maxFunc);
		setPredefinedItem(maxOp);
		
		LabelledItem nminOp = new LabelledItem
			(this, new Label("nmin"), null, "nmin");
		Function nminFunc = new Function(nminOp);
		nminOp.setValue(nminFunc);
		setPredefinedItem(nminOp);
		
		LabelledItem absOp = new LabelledItem
			(this, new Label("abs"), null, "abs");
		Function absFunc = new Function(absOp);
		absOp.setValue(absFunc);
		setPredefinedItem(absOp);
		
		LabelledItem starstarOp = new LabelledItem(this,new
											   Label("**"),null,"**");
		Function starstarFunc = new Function(starstarOp);
		starstarOp.setValue(starstarFunc);
		setPredefinedItem(starstarOp);

		LabelledItem divOp = new LabelledItem(this,new
											   Label("/"),null,"/");
		Function divFunc = new Function(divOp);
		divOp.setValue(divFunc);
		setPredefinedItem(divOp);

		LabelledItem hashOp = new LabelledItem(this,new
											  Label("#"),null,"#");
		Function hashFunc = new Function(hashOp);
		hashOp.setValue(hashFunc);
		setPredefinedItem(hashOp);
	
		LabelledItem pipeOp = new LabelledItem
			(this, new Label("|"), null, "|");
		Function pipeFunc = new Function(pipeOp);
		pipeOp.setValue(pipeFunc);
		setPredefinedItem(pipeOp);

		LabelledItem memberOp = new LabelledItem
			(this, new Label("::"), null, "::");
		Function memberFunc = new Function(memberOp);
		memberOp.setValue(memberFunc);
		setPredefinedItem(memberOp);
	
		LabelledItem inOp = new LabelledItem
			(this, new Label("in"), null, "in");
		Function inFunc = new Function(inOp);
		inOp.setValue(inFunc);
		setPredefinedItem(inOp);
	}
	
	/** To set a predefined item
	 * @param predefinedItem The item to be added to the predefined list
	 * @return void
	 */
	public void setPredefinedItem(Item predefinedItem) {
		predefinedItems.addItem(predefinedItem);
	}

	/** To add a user defined item
	 * @param itemIn The item to be added to the items list
	 * @return void
	 */
	public void addItem(Item itemIn) {
		items.addItem(itemIn);
	}

	/** To get the list of user defined items
	 * @return ListItem, list containing user defined items
	 */
	public ItemList getItems() {
		return items;
	}


	/**To check if the String corresponds to a predefined Item
	 * @param String str
	 * @return boolean
	 */
	public boolean isPredefinedItem(String str){
		ListIterator preIter = predefinedItems.listIterator();
		boolean result = false;
		if (preIter != null){
			while (preIter.hasNext()){
				LabelledItem item = (LabelledItem)preIter.next();
				if (item.getLabel().toString().equals(str)){
					result = true;
					break;
				}
			}
		}
		return result;
	}
					
	/** To check if an item exists
	 * @param itemLabel The Label corresponding to a type
	 * @return true if type exists, false otherwise
	 */
	public boolean itemExists(Label itemLabel) {
		Object obj;
		LabelledItem labelledItem;
		ListIterator predefIterator = predefinedItems.listIterator();
		while (predefIterator.hasNext()) {
			obj = predefIterator.next();  
			if (obj instanceof LabelledItem) {
				labelledItem = (LabelledItem)obj; 
				if (labelledItem.labelLookup(itemLabel) != null)
					return true;
			}
		}
		ListIterator iterator = items.listIterator();
		while (iterator.hasNext()) {
			obj = iterator.next();
			if (obj instanceof LabelledItem) {
				labelledItem = (LabelledItem)obj;
				if (labelledItem.labelLookup(itemLabel) != null)
					return true;
			}
		}
		return false;
	}

	//Added by srinivas on Nov 10th , 2000
	/** To look up for a label.If the item is there in the list of
	 * predefined items or user defined items then the item is returned
	 * @param itemLabel The Label for the item being looked for
	 * @return the LabelledItem desired
	 */
	public LabelledItem labelLookup(Label itemLabel) {
		Object item;
		LabelledItem labelledItem;
		ListIterator predefIterator = predefinedItems.listIterator();
		while (predefIterator.hasNext()) {
			item = predefIterator.next();  
			if (item instanceof LabelledItem) {
				labelledItem = (LabelledItem)item;
				if (labelledItem.getLabel().equals(itemLabel))
					return labelledItem;
			}
		}
		ListIterator iterator = items.listIterator();
		while (iterator.hasNext()) {
			item = iterator.next();
			if (item instanceof LabelledItem) {
				labelledItem = (LabelledItem)item;
				if (labelledItem.getLabel().equals(itemLabel))
					return labelledItem;
			}
		}
		return null; // may rather want to throw an exception here
	}

	/** To look up for a label given a String
	 * @param stringIn The String representing the label name
	 * @return the Item desired
	 */
	public Item labelLookup(String labelName) {
		Label labelSearched = new Label(null, labelName);
		return labelLookup(labelSearched);
	}

	/** To get the type corresponding to a name (String)
	 * @param nameIn String corresponding to name of type
	 * @return Type, type corresponding to name
	 */
	public Type getType(String nameIn) {
		if (nameIn.equalsIgnoreCase("boolean"))
			return bool;
		if (nameIn.equalsIgnoreCase("bit"))
			return bit;
		if (nameIn.equalsIgnoreCase("integer"))
			return integer;
		if (nameIn.equalsIgnoreCase("natural"))
			return natural;
		if (nameIn.equalsIgnoreCase("rational"))
			return rational;
		if (nameIn.equalsIgnoreCase("real"))
			return real;
		if (nameIn.equalsIgnoreCase("character"))
			return character;
		if (nameIn.equalsIgnoreCase("element"))
			return element;
		if (nameIn.equalsIgnoreCase("universal"))
			return universal;
		if (nameIn.equalsIgnoreCase("univ"))
			return universal;
		if (nameIn.equalsIgnoreCase("package"))
			return pack;
		if (nameIn.equalsIgnoreCase("facet"))
			return facet;
		if (nameIn.equalsIgnoreCase("component"))
			return component;
		if (nameIn.equalsIgnoreCase("arrayInteger"))
			return arrayInt;
		if (nameIn.equalsIgnoreCase("arrayBit"))
			return arrayBit;
		if (nameIn.equalsIgnoreCase("arrayBoolean"))
			return arrayBool;
		if (nameIn.equalsIgnoreCase("arrayReal"))
			return arrayReal;
		if (nameIn.equalsIgnoreCase("setInteger"))
			return setInt;
		if (nameIn.equalsIgnoreCase("setReal"))
			return setReal;
		if (nameIn.equalsIgnoreCase("setBit"))
			return setBit;
		if (nameIn.equalsIgnoreCase("setBoolean"))
			return setBool;
		if (nameIn.equalsIgnoreCase("sequenceInteger"))
			return sequenceInt;
		if (nameIn.equalsIgnoreCase("sequenceBit"))
			return sequenceBit;
		if (nameIn.equalsIgnoreCase("sequenceBoolean"))
			return sequenceBool;
		if (nameIn.equalsIgnoreCase("sequenceReal"))
			return sequenceReal;
		if (nameIn.equalsIgnoreCase("bitvector"))
			return arrayBit;
		//check out use clause
		return null;
	}

	/** To get the mode value corresponding to a mode object
	 * @param modeIn The SOM_Mode 
	 * @return int The value corresponding to the mode
	 */
	public int getMode(SOM_Mode modeIn) {
		if (modeIn instanceof SOM_In)
			return ParameterItem.IN;
		if (modeIn instanceof SOM_Out)
			return ParameterItem.OUT;
		if (modeIn instanceof SOM_InOut)
			return ParameterItem.INOUT;
		if (modeIn instanceof SOM_Design)
			return ParameterItem.DESIGN;
		return ParameterItem.NOMODE;
	}

	// To be completed
	/** To choose combine/choose a type given two types
	 * @param typeOne First type to be compared
	 * @param typeTwo Second type
	 * @return Type The result of combining the types compared
	 */
	public static Type chooseType(Type typeOne, Type typeTwo) {
		return null;
	}

    /** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		ListIterator itemIter = items.listIterator();
		Item nextItem;
		while (itemIter.hasNext()) {
			nextItem = (Item)itemIter.next();
			return nextItem.toString();
		}
		return null;
	}

	
	/**To check if the array or set or sequence of the type already
	 * exsists
	 * @param type To know if it is a array, set or sequence
	 * @param elementType To search for the element type
	 * @return Type
	 */
	public Type alreadyDeclared(String type, Type elementType){
		//check out the use clause
		return null;
	}
	
	public void publishROM(PrintWriter pw){
		ListIterator itemIter = items.listIterator();
		Item nextItem;
		while (itemIter.hasNext()) {
			nextItem = (Item)itemIter.next();
			
			Expression expr = nextItem.getValue();
			if (expr instanceof Package){
				pw.println("\nTHE BASIC PACKAGE IS");
				pw.println(nextItem.toString());
				((Package)expr).publishROM(pw);
			}
		}
	}
}
		









