//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//
//  $Id: SOM_ArrayDeclaration.java,v 1.1 2000/07/24 23:41:01 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ArrayDeclaration.java,v $
//  Revision 1.1  2000/07/24 23:41:01  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//
//=============================================================================
package SOM;

import java.util.ArrayList;
import util.ListEnumeration;

/** This represents the <B>ArrayDeclaration</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 */
public class SOM_ArrayDeclaration
extends SOM_Declaration {

	protected SOM_Identifier arrayName;
	protected ArrayList arrayList = new ArrayList();
	// if size is not declared, size is infinite (big value = 100)
	protected int size = 100;

	/** The primary constructor
	 * @param arrayNameIn The SOM_Identifier corresponding to the name
	 * of the array
	 * @param sizeIn The size of the array
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * type of the array
	 * @return Not applicable
	 */
	public SOM_ArrayDeclaration(SOM_Identifier arrayNameIn,
								int sizeIn,
								SOM_TypeDeclaration myType) {
		super();
		arrayName = arrayNameIn;
		setKind(SOM_Declaration.ARRAY);
		size = sizeIn;
		setType(myType);
	}

	/** A constructor
	 * @param arrayNameIn The SOM_Identifier corresponding to the name
	 * of the array
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * type of the array
	 * @return Not applicable
	 */
	public SOM_ArrayDeclaration(SOM_Identifier arrayNameIn,
								SOM_TypeDeclaration myType) {
		super();
		arrayName = arrayNameIn;
		setKind(SOM_Declaration.ARRAY);
		setType(myType);
	}

	/** To add to the list of elements in array at a given index 
	 * @param exprIn The SOM_Expression to be added
	 * @param indexIn The index of element to be added
	 * @return true, if addition successful, otherwise false
	 */
	public boolean addToArrayList(SOM_Expression exprIn, int indexIn) {
		if (arrayList.size() < size) {
			arrayList.add(indexIn, exprIn);
			return true;
		} else
			return false;
	}

	/** To set the list of elements in array
	 * @param listIn The ArrayList to be set
	 * @return void
	 */
	public void setArrayList(ArrayList listIn) {
		arrayList = listIn;
	}

	/** To get the array of elements
	 * @return The Object[] containing the elements
	 */
	public Object[] getArrayElements() {
		return arrayList.toArray();
	}

	/** To set the size of array
	 * @param sizeIn The size of the array to set
	 * @return void
	 */
	public void setSize(int sizeIn) {
		size = sizeIn;
	}

	/** To get the size of array
	 * @return int, number of elements
	 */
	public int getSize() {
		return arrayList.size();
	}

	/** To get the string representation of this object. Overrides the
	 * method in SOM_Declaration
	 * @return String representation of this object - name of the set
	 */
	public String toString() {
		return arrayName.toString();
	}

	/** To implement my own version of isDistinguishable for array
	 * declarations
	 * @param declOne The first SOM_ArrayDeclaration
	 * @param declTwo The next SOM_ArrayDeclaration to be distinguished
	 * against declOne
	 * @return true if they are distinguishable, false otherwise
	 */
	public static boolean isDistinguishable(SOM_ArrayDeclaration declOne,
											SOM_ArrayDeclaration declTwo) { 
		SOM_ArrayType typeOne = (SOM_ArrayType)declOne.getType();
		SOM_ArrayType typeTwo = (SOM_ArrayType)declTwo.getType();
		if (!declOne.toString().equalsIgnoreCase(declTwo.toString()))
			return true;
	 
		if (declOne.getSize() != declTwo.getSize())
			return true;
			
		return(SOM_ArrayType.isDistinguishable(typeOne, typeTwo));
	}

	/** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
	 * @param declLHS The ArrayDecl corresponding to the lhs object
	 * @param declRHS The ArrayDecl corresponding to the rhs object
	 * @return true, if the rhs is assignable to the lhs, false otherwise
	 */
	public static boolean isLeftCompatibleWithRight
		(SOM_ArrayDeclaration declLHS, SOM_ArrayDeclaration declRHS) { 
		SOM_ArrayType typeLHS = (SOM_ArrayType)declLHS.getType();
		SOM_ArrayType typeRHS = (SOM_ArrayType)declRHS.getType();
		if (declLHS.getSize() < declRHS.getSize())
			return false;

		return SOM_ArrayType.isLeftTypeCompatibleWithRightType
			(typeLHS, typeRHS);
	}
}
