//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//
//  $Id: SOM_ArrayType.java,v 1.1 2000/07/24 23:41:01 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ArrayType.java,v $
//  Revision 1.1  2000/07/24 23:41:01  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

/** This represents the <B>ArrayType</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 */
public class SOM_ArrayType
extends SOM_TypeDeclaration {

	// This will be the type of the members of the array
	protected SOM_TypeDeclaration memberType;

	/** The primary constructor
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * "bunch MetaType"
	 * @param myId The Identifier containing the name of the type
	 * @return Not applicable
	 */
	public SOM_ArrayType(SOM_Identifier myId, SOM_TypeDeclaration myType) {
		super(myId, myType);
	}

	/** Another constructor
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * "bunch MetaType"
	 * @return Not applicable
	 */
	public SOM_ArrayType(SOM_TypeDeclaration myType) {
		// The default name is setType
		this(new SOM_Identifier("arrayType"), myType);
	}

	/** To set the type of members 
	 * @param memberTypeIn The SOM_TypeDeclaration corresponding to members
	 * @return void
	 */
	public void setMemberType(SOM_TypeDeclaration memberTypeIn) {
		memberType = memberTypeIn;
	}

	/** To get the member type
	 * @return The SOM_TypeDeclaration of member
	 */
	public SOM_TypeDeclaration getMemberType() {
		return memberType;
	}

	/** To implement a version of isDistinguishable for array types
	 * @param typeOne The first SOM_ArrayType
	 * @param typeTwo The next SOM_ArrayType to be distinguished
	 * against typeOne
	 * @return true if they are distinguishable, false otherwise
	 */
	public static boolean isDistinguishable(SOM_ArrayType typeOne,
											SOM_ArrayType typeTwo) { 
		if (!typeOne.toString().equalsIgnoreCase(typeTwo.toString()))
			return true;
		else 
			return SOM_TypeDeclaration.isDistinguishable
				(typeOne.getMemberType(), typeTwo.getMemberType());
	}

	/** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
	 * @param typeLHS The ArrayType corresponding to the lhs object
	 * @param typeRHS The ArrayType corresponding to the rhs object
	 * @return true, if the rhs is assignable to the lhs, false otherwise
	 */
	public static boolean isLeftTypeCompatibleWithRightType
		(SOM_ArrayType typeLHS, SOM_ArrayType typeRHS) { 
		return typeRHS.getMemberType().isPredicateSubtypeOf
			(typeLHS.getMemberType());
	}
}
