//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//           Cindy ckong@ittc.ukans.edu
//           Krishna krishna@ittc.ukans.edu
//
//  $Id: SOM_DeclarationStatement.java,v 1.32 2001/03/26 19:04:57 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_DeclarationStatement.java,v $
//  Revision 1.32  2001/03/26 19:04:57  cvs
//  Completed the implementation for the change in export clauses. Also
//  added code in SOM_PackageDeclaration to take care of use clauses (case
//  was missing for when these clauses are found in declaration part of
//  packages).
//  -- Cindy
//
//  Revision 1.31  2001/03/12 23:54:37  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.30  2001/03/06 05:02:27  cvs
//  The M__Value method has been implemented for SelectedName and some
//  errors in the Meta functions have been removed in the other files 
//  --srinivas
//
//  Revision 1.29  2000/12/05 03:05:02  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.28  2000/12/03 23:55:25  cvs
//  Added the M__Value method to the Literal classes.Previously they
//  were returning null.The M__Type method for all the Declaration
//  statement classes has been completed 
//  --srinivas
//
//  Revision 1.27  2000/11/29 00:20:02  cvs
//  Can now process list of identifiers declared together (for the ROM).
//  -- Cindy
//
//  Revision 1.26  2000/11/27 23:55:30  cvs
//  Changed the M__Value so that the Function declared is built correctly
//  --srinivas
//
//  Revision 1.25  2000/11/18 00:18:27  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.24  2000/11/13 23:36:39  cvs
//  Changed the way the Items are created.Now they are created through
//  the factory class 
//  --srinivas
//
//  Revision 1.23  2000/11/13 21:50:35  cvs
//  Changed the method M__Value in these classes so that they return
//  the complete Expression. 
//  --srinivas
//
//  Revision 1.22  2000/11/06 18:12:00  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.21  2000/08/01 22:48:46  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.20  2000/07/24 23:41:01  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//  Revision 1.19  2000/07/21 06:35:26  cvs
//  Made some changes to the M__Type function in both the
//  declarationStatement and the facetDeclaration 
//  --srinivas
//
//  Revision 1.18  2000/07/14 23:01:21  cvs
//  Made changes to the meta functions and added some meta functions
//  to the identifier object. 
//  --srinivas
//
//  Revision 1.17  2000/07/11 00:11:08  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are keywords. Typechecking for set is done. Added SetType
//  and FunctionType. 
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.16  2000/06/22 17:22:40  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.15  2000/05/19 00:03:59  cvs
//  restoring the files after the file system had crashed
//  --srinivas
//
//  Revision 1.15  2000/05/06 00:14:52  cvs
//  The Meta Functions for the term items have been changed to the
//  class SOM_LabelledExpression .The M__Value and M__String have
//  been implemented for the AssociationListing.The meta functions
//  for the declaration items are also implemented in the
//  SOM_DeclarationStatement 
//  --srinivas
//
//  Revision 1.14  2000/04/28 19:34:05  cvs
//  Added some more of typechecking. Added parsing of the $ operator
//  and constant literal. 
//  -- Cindy
//
//  Revision 1.13  2000/04/11 03:17:51  cvs
//  The dummy M__Label and M__Value functions have been removed .
//  --srinivas
//
//  Revision 1.12  2000/04/08 02:21:58  cvs
//  The Meta Functions have been implemented and these are the files
//  that had to be modified.Addded some accessor methods to classes
//  like SOM_DeclarationStatement, SOM_Expression,
//  SOM_RecordTupleDeclarationStatement. The Meta functions for facet
//  terms and the term items are implemented  
//  --srinivas
//
//  Revision 1.11  2000/04/06 19:13:18  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy 
//
//  Revision 1.10  2000/03/23 00:22:45  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.9  2000/03/07 22:55:52  cvs
//  Modified the class due the change in syntax for function
//  declarations and facet declarations 
//  --srinivas
//
//  Revision 1.8  2000/03/04 02:26:30  cvs
//  Was not calling the method publishSOM in facet declaration
//  statement so done the encessary modifications 
//  -- srinivas
//
//  Revision 1.7  2000/02/29 21:11:23  cvs
//  manually added the log messages corresponding to the previous commit
//  -- srinivas
//
//  Revision 1.6  2000/02/29 18:06:48  cvs
//  modified this class to include the facet declaration statement production
//  -- srinivas
//
//  Revision 1.5  2000/02/13 01:15:53  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. 
//  Modified the grammar to incorporate these changes. Also populated
//  the object model with these new objects and completed the
//  pretty-printing for these new constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//  Revision 1.4  2000/02/10 03:56:28  cvs
//  Modified the grammar for incorporating records, tuples and "let"
//  expressions. Also populated the object model with these above
//  constructs, and completed the pretty printing for them. 
//  - krishna
//
//  Revision 1.3  2000/02/10 00:37:12  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  2000/02/08 18:42:21  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.1  2000/02/02 20:16:48  cvs
//  I am committing all the new class files I have created for the new
//  operators and new rules Krishna has added to the grammar. 
//  -- Cindy
//
//
//
//=============================================================================
package SOM;

import java.io.*;
import util.*;
import ROM.*;
//import java.lang.UnsupportedOperationException; //only from JDK1.2 :-(

/** This represents the <B>DeclarationStatement</B> object
 * NOTE Even though this represents a declaration, it is NOT to be
 * stored in the symbol table. Rather, this statement should be
 * converted to multiple SOM_VariableDeclarations and then introduced
 * in the symbol table.
 * @author <I>Roshan Kamath, Cindy Kong and Krishna Ranganathan</I> 
 * (rkamath@ececs.uc.edu, ckong@ittc.ukans.edu, krishna@ittc.ukans.edu)
 */
public class SOM_DeclarationStatement
extends SOM_Declaration
implements SOM_FacetDeclarativeItem, SOM_ComponentDeclarativeItem, 
		   SOM_PackageDeclarativeItem {

	public static final int STATEMENTZ = 0;
	public static final int FACET = 1;
	public static final int TYPE = 2;
	public static final int SET = 3;
	public static final int SEQUENCE = 4;
	public static final int ARRAY = 5;

	private SOM_Identifier id;
	private SOM_InterfaceListing interList = null;
	private SOM_IdentifierListing idList = null;
	// One of the following will not be null
	private	SOM_DeclarationStatementZ declStmtZ = null;
	private SOM_FacetDeclarationStatement facetDeclStmt = null;
	private SOM_TypeDeclarationStatement typeDeclStmt = null;
	private SOM_SetDeclarationStatement setDeclStmt = null;
	private SOM_SequenceDeclarationStatement sequenceDeclStmt = null;
	private SOM_ArrayDeclarationStatement arrayDeclStmt = null;

	private int declType = STATEMENTZ;
	// varFncDecl = false => variable; varFncDecl = true => function
	private boolean varFncDecl = false;

	/** The primary constructor
	 * @return Not applicable
	 */
	public SOM_DeclarationStatement() {
		super();
		// Let the kind be undefined.
	}
	
	/** To get the declaration type
	 * @return int
	 */
	public int getDeclType() {
		return declType;
	}

	/** To get whether variables or functions are declared
	 * @return boolean
	 */
	public boolean getVarFncDecl() {
		return varFncDecl;
	}

	/** To set the identifier
	 * @param idIn The SOM_Identifier to be set
	 * @return void
	 */
	public void setIdentifier(SOM_Identifier idIn) {
		id = idIn;
	}

	/** To get the identifier
	 * @return The SOM_Identifier  object for this designUnit
	 */
    public  SOM_Identifier getIdentifier(){
		return id;
    }
	 
	/** To set the interfaceListing
	 * @param interfaceIn The SOM_InterfaceListing object to be set
	 * @return void
	 */
	public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
		interList = interfaceIn;
		varFncDecl = true;
	}

	/** To get the interfaceListing
	 * @return The SOM_InterfaceListing object for this designUnit
	 */
	public SOM_InterfaceListing getInterfaceListing() {
		return interList;
	}
	
	/** To set the identifierListing
	 * @param listIn The SOM_IdentifierListing to be set
	 * @return void
	 */
	public void setIdentifierListing(SOM_IdentifierListing listIn) {
		idList = listIn;
		varFncDecl = false;
	}

	/** To get the identifierListing
	 * @return The SOM_IdentifierListing that was set
	 */
	public SOM_IdentifierListing getIdentifierListing() {
		return idList;
	}

	/** To set the DeclarationStatementZ
	 * @param declStmtZIn The SOM_DeclarationStatementZ to be set
	 * @return void
	 */
	public void setDeclarationStatementZ(SOM_DeclarationStatementZ 
										 declStmtZIn) {
		declStmtZ = declStmtZIn;
	}

	/** To get the DeclarationStatementZ
	 * @return The SOM_DeclarationStatementZ that was set
	 */
	public SOM_DeclarationStatementZ getDeclarationStatementZ() {
		return declStmtZ;
	}

	/** To set the FacetDeclarationStatement
	 * @param facetDeclStmtIn  The SOM_FacetDeclarationStatement to be set
	 * @return void
	 */
	public void setFacetDeclarationStatement(SOM_FacetDeclarationStatement 
											 facetDeclStmtIn){
		facetDeclStmt =facetDeclStmtIn;
		declType = FACET;
	}

	/** To get the FacetDeclarationStatement
	 * @return The SOM_FacetDeclarationStatement  that was set
	 */
	public SOM_FacetDeclarationStatement getFacetDeclarationStatement(){
		return facetDeclStmt;
	}

	/** To set the TypeDeclarationStatement
	 * @param typeDeclStmtIn  The SOM_TypeDeclarationStatement to be set
	 * @return void
	 */
	public void setTypeDeclarationStatement(SOM_TypeDeclarationStatement 
											 typeDeclStmtIn){
		typeDeclStmt = typeDeclStmtIn;
		declType = TYPE;
	}

	/** To get the TypeDeclarationStatement
	 * @return The SOM_TypeDeclarationStatement  that was set
	 */
	public SOM_TypeDeclarationStatement getTypeDeclarationStatement(){
		return typeDeclStmt;
	}

	/** To set the SetDeclarationStatement
	 * @param setDeclStmtIn  The SOM_SetDeclarationStatement to be set
	 * @return void
	 */
	public void setSetDeclarationStatement(SOM_SetDeclarationStatement 
											 setDeclStmtIn){
		setDeclStmt = setDeclStmtIn;
		declType = SET;
	}

	/** To get the SetDeclarationStatement
	 * @return The SOM_SetDeclarationStatement  that was set
	 */
	public SOM_SetDeclarationStatement getSetDeclarationStatement(){
		return setDeclStmt;
	}

	/** To set the SequenceDeclarationStatement
	 * @param sequenceDeclStmtIn  The SOM_SequenceDeclarationStatement
	 * to be set
	 * @return void
	 */
	public void setSequenceDeclarationStatement
		(SOM_SequenceDeclarationStatement sequenceDeclStmtIn){
		sequenceDeclStmt = sequenceDeclStmtIn;
		declType = SEQUENCE;
	}

	/** To get the SequenceDeclarationStatement
	 * @return The SOM_SequenceDeclarationStatement  that was set
	 */
	public SOM_SequenceDeclarationStatement getSequenceDeclarationStatement(){
		return sequenceDeclStmt;
	}

    /** To set the ArrayDeclarationStatement
	 * @param arrayDeclStmtIn  The SOM_ArrayDeclarationStatement
	 * to be set
	 * @return void
	 */
	public void setArrayDeclarationStatement
		(SOM_ArrayDeclarationStatement arrayDeclStmtIn){
		arrayDeclStmt = arrayDeclStmtIn;
		declType = ARRAY;
	}

	/** To get the ArrayDeclarationStatement
	 * @return The SOM_ArrayDeclarationStatement  that was set
	 */
	public SOM_ArrayDeclarationStatement getArrayDeclarationStatement(){
		return arrayDeclStmt;
	}

    // added by srinivas on april 12th 2000
    /* Meta Function to return the value
	 * @param 
     * @param void
     * @ return Object
	 */
  	public Expression M__Value(Factory romFactory, BaseObject parentIn,
							   Universe root){		
		SOM_Expression expr = null;
		if (getDeclarationStatementZ() != null){
			expr = getDeclarationStatementZ().getValueExpression();
		}else if (getTypeDeclarationStatement() != null){
			expr = getTypeDeclarationStatement().getValue();
		}else if (getSetDeclarationStatement() != null){
			expr = getSetDeclarationStatement().getValueExpr();
		}else if (getSequenceDeclarationStatement() != null){
			expr = getSequenceDeclarationStatement().getValue();
		}else if (getArrayDeclarationStatement() !=null){
			expr = getArrayDeclarationStatement().getValue();
		}else if (getFacetDeclarationStatement() != null){
			expr = getFacetDeclarationStatement().getFacetExpression();
		}
		if (getInterfaceListing() != null){
			Function function= romFactory.newFunction(parentIn);
			ListEnumeration interList = getInterfaceListing().interfaces();
			while (interList.hasMoreElements()){
				SOM_InterfaceItem interItem = (SOM_InterfaceItem)
					interList.nextElement();
				function.addInterfaceItem(interItem.buildROM
										  (romFactory, root, function));
			}
			if (expr != null)
				function.setExpression
					(expr.M__Value(romFactory, function, root));
			return (Expression)function;
		}else if (expr != null){
			return expr.M__Value(romFactory, parentIn, root);
		}else {
			return null;
		}
	}

    //added by srinivas on april 10th 2000
    // MetaFunction M__Label for a type item
    // return SOM_Label
    /** M__Label routine to get the label of the type item
     * @param void
     * @return SOM_Label
     */
    public ListEnumeration M__Label(){
		
		util.List labels = new util.List();
		SOM_IdentifierListing idenListing = getIdentifierListing();
		labels.addElement(getIdentifier());
		if (idenListing !=null){
		    ListEnumeration identifiers = idenListing.identifiers();
		    while (identifiers.hasMoreElements())
			labels.addElement(identifiers.nextElement());
		}
		if (labels.isEmpty())
		    return (ListEnumeration)null;
		else 
		    return labels.elements();
    }

    // added by srinivas on april 10th 2000
    // the Meta Function to get the type of the type item
    /** M__Type routine to get the type of the type item
     *  @param void
     * @ return Object
     */
	public String M__Type(){
		if (getDeclarationStatementZ() != null){
			SOM_Expression expr =
				getDeclarationStatementZ().getTypeExpression();
			if (expr.getResolvedType().toString() != null)
				return expr.getResolvedType().toString();
			else
				return null;
		}
		return null;
	}

	//Added by srinivas on Nov 29th, 2000
	/*The function gets the ROM.Type of the item declared
	 * @param parent The parent of the declaration statement. this is
	 * used to search for user defined types
	 * @param newParent The parent of the new user defined type should
	 * be the declaration statement item that is created
	 * @param root To search for predefined types
	 * @param identifier if a new type is declared than the name of
	 * the new type is the identifier
	 * @param value To add the value to the BunchValue of the type.The
	 * Expression is added as an element to the bunch.
	 * NOTE: The Expression is not a evaluated.
	 * @return Type
	 */
    public Type M__Type(BaseObject parent,BaseObject
						newParent,Universe root, String identifier,
						Expression value){
		Type type = null;
		if (getDeclarationStatementZ() != null){
			type = getDeclarationStatementZ().M__Type(parent,root,value);
		}else if (getArrayDeclarationStatement() != null){
			type = getArrayDeclarationStatement().M__Type(parent,root,value);
		}else if (getSetDeclarationStatement() != null){
			type = getSetDeclarationStatement().M__Type(parent,root,value);
		}else if (getSequenceDeclarationStatement() != null){
			type =
				getSequenceDeclarationStatement().M__Type(parent,root,value);
		}else if (getFacetDeclarationStatement() != null){
			type = null;
		}else if (getTypeDeclarationStatement() != null){
			type =
				getTypeDeclarationStatement().M__Type(parent,newParent,
													  root,identifier,value);

		}
		return type;
    }
	
	// added by srinivas on april6th,2000
    /** This is the MetaFunction to retrieve the String
     * @return the String corresponding to the type item or the
     * variable item ,in general the declaration statements.
     */
    public String M__String() {
		try{
			SOM_DeclarationStatementZ declStmtZ = getDeclarationStatementZ();
			String typeString = new String();
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw,true);
			if (declStmtZ !=null){
				publishSOM(pw);
				pw.flush();
				pw.close();
				typeString = sw.toString();
				sw.close();
				return typeString;
			}
			else
				return (String)null;
		} catch (Exception e) {
			System.out.println(e);
			return (String)null;			
		}
	}

	public String M__String(SOM_Identifier iden) {
		try {
			String typeString = new String();
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw, true);
			iden.publishSOM(pw);
			pw.print(" :: ");
			if(declStmtZ != null) {
				declStmtZ.publishSOM(pw);
			}
			if (facetDeclStmt != null) {
				facetDeclStmt.publishSOM(pw);
			}
			if (typeDeclStmt != null) {
				typeDeclStmt.publishSOM(pw);
			}
			if(setDeclStmt != null) {
				setDeclStmt.publishSOM(pw);
			}
			if (sequenceDeclStmt != null) {
				sequenceDeclStmt.publishSOM(pw);
			}
			if (arrayDeclStmt != null) {
				arrayDeclStmt.publishSOM(pw);
			}
			pw.println("; ");
			pw.flush();
			pw.close();
			typeString = sw.toString();
			sw.close();
			return typeString;
		} catch (Exception e) {
			System.out.println(e);
			return (String)null;			
		}
	}

	/** The buildROM method for this item
	 * @param romFactory, Factory object used to create objects in ROM
	 * @param root, Universe object 
	 * @param parentIn, BaseObject parent of item
	 * @return LabelledItem, corresponding to this declaration
	 */
	public LabelledItem buildROM(Factory romFactory, Universe root, 
								 BaseObject parentIn) {
		// Declaration of a list of variables grouped together
		if (idList != null) {
			ListEnumeration idEnum = idList.identifiers();
			SOM_Identifier iden;
			LabelledItem declItemElem;
			Label declLabelElem;
			Expression declValueElem;
			Type declTypeElem;
			while (idEnum.hasMoreElements()) {
				iden = (SOM_Identifier)idEnum.nextElement();
			    declItemElem = romFactory.newLabelledItem
					(parentIn, null, null, null, M__String(iden));
				declLabelElem = romFactory.newLabel
					(declItemElem, iden.toString());
				declItemElem.setLabel(declLabelElem);
				declValueElem = M__Value(romFactory, declItemElem, root);
				declItemElem.setValue(declValueElem);
				declTypeElem = M__Type(parentIn, declItemElem, root,
							iden.toString(), declValueElem);
				declItemElem.setType(declTypeElem);
				((ROM.Facet)parentIn).addItem(declItemElem);
			}
		}
		LabelledItem declItem;
		if (varFncDecl) {
			declItem = romFactory.newLabelledItem
				(parentIn, null, null, null, M__String());
		}
		else
			declItem = romFactory.newLabelledItem
				(parentIn, null, null, null, M__String(id));
		Label declLabel = romFactory.newLabel(declItem, id.toString());
		declItem.setLabel(declLabel);
		Expression declValue = M__Value(romFactory, declItem, root);
		declItem.setValue(declValue);
		Type type = M__Type(parentIn, declItem, root, id.toString(),
							declValue); 
		declItem.setType(type);
		return declItem;
	}

	/** publishSOM routine to print object module
	 * @param PrintWriter
	 * @return void - mak
	 */
    public void publishSOM(PrintWriter pw) {
		id.publishSOM(pw);
		if (interList != null) {
			interList.publishSOM(pw);
		} else
			if (idList != null) {
				pw.print( ",");
				idList.publishSOM(pw);
			}
		pw.print(" :: ");
		if(declStmtZ != null) {
				declStmtZ.publishSOM(pw);
		}
		if (facetDeclStmt != null) {
				facetDeclStmt.publishSOM(pw);
		}
		if (typeDeclStmt != null) {
				typeDeclStmt.publishSOM(pw);
		}
		if(setDeclStmt != null) {
				setDeclStmt.publishSOM(pw);
		}
		if (sequenceDeclStmt != null) {
				sequenceDeclStmt.publishSOM(pw);
		}
		if (arrayDeclStmt != null) {
				arrayDeclStmt.publishSOM(pw);
		}
		pw.println("; ");
    }

	/** publishPVS routine needed for SOCCER
	 * @param PrintWriter
	 * @return void - mak
	 */
    public void publishPVS(PrintWriter pw) {
		id.publishSOM(pw);
		if (interList != null) {
			interList.publishSOM(pw);
		} else
			if (idList != null) {
				pw.print( ",");
				idList.publishSOM(pw);
			}
		pw.print(":");
		if(declStmtZ != null) {
			declStmtZ.publishSOM(pw);
		}
		if (facetDeclStmt != null) {
			facetDeclStmt.publishSOM(pw);
		}
		if (typeDeclStmt != null) {
			typeDeclStmt.publishSOM(pw);
		}
		if(setDeclStmt != null) {
			setDeclStmt.publishSOM(pw);
		}
		if (sequenceDeclStmt != null) {
			sequenceDeclStmt.publishSOM(pw);
		}
		if (arrayDeclStmt != null) {
			arrayDeclStmt.publishSOM(pw);
		}
		pw.println("");
    }

	// the next three productions are not actually needed per se
	// they are added only to instantiate the methods for the abstract
	// class SOM_PackageDeclarativeItem - krishna on Feb 12, 2000
	/** To set the domainList - implements the function from
	 * SOM_PackageDeclarativeItem
	 * @param domainListIn The SOM_DeclarationList object to be set
	 * @return void
	 */
	public void setDomainList(SOM_DeclarationList domainListIn) {
	}

	/** To get the domainList - implements the function from
	 * SOM_PackageDeclarativeItem
	 * @return The SOM_DeclarationList object for this designUnit
	 */
	public SOM_DeclarationList getDomainList() {
		return null;
	}

	/** To get the exports
	 * @return The ListEnumeration corresponding to the export clauses
	 */
	public ListEnumeration exports() {
		return null;
	}
}












