//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_ExpTerm.java,v 1.14 2001/03/12 23:54:38 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ExpTerm.java,v $
//  Revision 1.14  2001/03/12 23:54:38  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.13  2000/11/18 00:18:27  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.12  2000/11/02 23:11:54  cvs
//  Some changes to the M__Value method so that it returns just an
//  Expression and not a Formula only 
//  --srinivas
//
//  Revision 1.11  2000/10/30 22:13:23  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas 
//
//  Revision 1.10  2000/08/01 22:48:46  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.9  2000/06/22 17:22:41  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.8  2000/04/06 19:13:19  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.7  2000/03/23 00:22:46  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.6  2000/03/22 03:21:27  cvs
//  Modified the grammar to remove thew production exptermtail and the
//  classes are also modified 
//  --srinivas
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;

/** This represents the <B>ExpTerm</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */

public class SOM_ExpTerm
extends SOM_ExpressionWithDyadicOperator {
	// This represents the <expTerm> object for which the grammar is
	// <expTerm> ::== <factor> (<EXPONENT> <expTerm>)?
	
	private SOM_DyadicOperator exponentOp = null;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_ExpTerm() {
		super();
	}

	/** To set the factor
	 * @param factorIn The SOM_Factor to be set
	 * @return void
	 */
	public void setFactor(SOM_Factor factorIn) {
		setStrongerExpression((SOM_AbstractExpression)factorIn);
	}

	/** To get the factor
	 * @return The SOM_Factor for this expression
	 */
	public SOM_Factor getFactor() {
		return (SOM_Factor)getExpression();
	}

	/** To set the operator
	 * @param opIn The SOM_DyadicOperator to be set
	 * @return void
	 */
	public void setOperator(SOM_DyadicOperator opIn) {
		exponentOp = opIn;
	}

	/** To get the operator
	 * @return The SOM_DyadicOperator 
	 */
	public SOM_DyadicOperator getOperator() {
		return exponentOp;
	}

	/** To set the expTerm
	 * @param expTerm The SOM_ExpTerm to be set
	 * @return void
	 */
	public void setExpTerm(SOM_ExpTerm expTermIn) {
		// First create the ExpTermTail object
		SOM_ExpTermTail exprTail = new SOM_ExpTermTail(exponentOp,expTermIn);

		addToAbstractExpressionTailList(exprTail);
	}

	public ListEnumeration expTerms() {
		return getExpressionTailList().elements();
	}

	/** To get the expTerm
	 * @return The SOM_ExpTerm
	 * changed the implementation - krishna on Mar 19, 2000
	 */
	public SOM_ExpTerm getExpTerm() {
		ListEnumeration expTermTailEnum = expTerms();
		while (expTermTailEnum.hasMoreElements()) {
			SOM_ExpTermTail expTermTail = 
				(SOM_ExpTermTail)expTermTailEnum.nextElement();
			return (SOM_ExpTerm)expTermTail.getExpression();
		}
		return (SOM_ExpTerm)null;
	}


	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		Expression leftNode ;
		if (getOperator() != null){
			Formula retExpr = new Formula(parent);
			leftNode =  getFactor().M__Value(romFactory,retExpr,root);
			Expression rightNode = getExpTerm().
				M__Value(romFactory,retExpr,root);
			retExpr.setLeftOp(leftNode);
			String str=getOperator().toString();
			OperatorReference operRef = new
				OperatorReference(retExpr, root.labelLookup(str), null);
			retExpr.setOperator(operRef);
			retExpr.setRightOp(rightNode);
			leftNode = new Formula(retExpr);
			return retExpr;
		}else {
			leftNode =  getFactor().M__Value(romFactory,parent,root);
			return leftNode;
		}
	}
	
	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		ListEnumeration expTermTailEnum = expTerms();
		SOM_ExpTermTail expTermTail;
		
		getFactor().publishSOM(pw);
		if (exponentOp != null) {
			getOperator().publishSOM(pw);
			while (expTermTailEnum.hasMoreElements()) {
				expTermTail = (SOM_ExpTermTail)expTermTailEnum.nextElement();
				expTermTail.publishSOM(pw);
			}
		}
	}

		/** The publishPVS routine for this class
		 * @return void
		 */
		public void publishPVS(PrintWriter pw) {
				ListEnumeration expTermTailEnum = expTerms();
				SOM_ExpTermTail expTermTail;
		
				getFactor().publishPVS(pw);
				if (exponentOp != null) {
						getOperator().publishPVS(pw);
						while (expTermTailEnum.hasMoreElements()) {
								expTermTail = (SOM_ExpTermTail)expTermTailEnum.nextElement();
								expTermTail.publishPVS(pw);
						}
				}
		}
}	



