//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_ExportBlock.java,v 1.8 2001/03/21 00:05:05 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ExportBlock.java,v $
//  Revision 1.8  2001/03/21 00:05:05  cvs
//  Added the "design" mode for parameters (SOM_Design.java). Changed the
//  semantics for export clauses. Removed the expression member from
//  SOM_FacetDeclaration.java and SOM_PackageDeclaration.java. Added code
//  to catch error when type-checking fails. Removed the redundant methods
//  getExportList() from SOM_FacetDeclaration.java,
//  SOM_PackageDeclaration.java and SOM_ComponentDeclaration.java.
//  -- Cindy
//
//  Revision 1.7  2001/03/12 23:54:37  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.6  2000/02/10 00:37:15  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.5  2000/02/08 18:42:24  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.4  1999/11/16 04:05:23  cvs
//  Added support for export clause and visiblity
//
//  Revision 1.3  1999/11/11 00:04:20  cvs
//  Fresh changes as part of the streamlining process with new
//  object model and grammar
//
//  Revision 1.2  1999/11/05 23:43:08  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:33  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:46  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.2  1999/06/25 15:10:23  rkamath
//  Modified the Makefile and changed the javadoc comments in the java files
//
//  Revision 1.1  1999/06/23 18:35:00  rkamath
//  A fresh new addition and modification of new files
//
//
//=============================================================================
package SOM;

import util.List;
import util.ListEnumeration;
import java.io.PrintWriter;

/** This represents the <B>ExportBlock</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 * @see SOM.SOM_BaseClass
 */
public class SOM_ExportBlock
extends SOM_BaseClass {

	private SOM_ExportClauseList exportClauseList = new SOM_ExportClauseList();
	private boolean exportAll = false;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_ExportBlock() {
		super();
	}

	/** To add an exportClause to the exportClauseList
	 * @param exportIn The SOM_ExportClause to be added
	 * @return void
	 */
	public void addToExportClauseList(SOM_ExportClause exportIn) {
		exportClauseList.addElement(exportIn);
	}

	/** to retrieve an enumeration of the exports in this export block
	 * @return ListEnumeration of the exports
	 */
	public ListEnumeration exports() {
		ListEnumeration exportClauseEnum = exportClauseList.elements();
		List finalList = new List();
		SOM_ExportClause exportClause;
		while (exportClauseEnum.hasMoreElements()) {
			exportClause = (SOM_ExportClause)exportClauseEnum.nextElement();
			List.concat(finalList, exportClause.getIdentifierList());
		}
		return finalList.elements();
		// Note than an empty enumeration is returned for the case when the
		// export clause is not present (or when the exportAll clause
		// was present) 
	}

	/** To set the exportAll variable true
	 * @return void
	 */
	public void setExportAll() {
		exportAll = true;
	}

	/** To retrieve the listEnumeration of the exportClauseList
	 * @return The ListEnumeration corresponding to the exportClauseList
	 */
	public ListEnumeration exportClauses() {
		return exportClauseList.elements();
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		ListEnumeration exportClauseEnum = this.exportClauses();
		SOM_ExportClause exportClause;

		if (exportAll == true) {
			pw.print("EXPORT ALL;\n");
		} else {
		  while (exportClauseEnum.hasMoreElements()) {
			exportClause = (SOM_ExportClause)exportClauseEnum.nextElement();
			exportClause.publishSOM(pw);
		  }
		}
	}
			
}

