//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_Expression.java,v 1.26 2001/03/12 23:54:37 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_Expression.java,v $
//  Revision 1.26  2001/03/12 23:54:37  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.25  2000/12/07 01:23:18  cvs
//  Adding the M__Value to all the Quantifiers. Making minor changes
//  to the other classes 
//  --srinivas
//
//  Revision 1.24  2000/12/06 01:18:16  cvs
//  Modified the loader for expressions to allow expressions such as:
//  var = var1 op var2 op var3 (without the use of parentheses to group
//  items)
//  -- Cindy
//
//  Revision 1.23  2000/12/05 03:05:02  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.22  2000/11/27 23:49:55  cvs
//  Previously was not taking the tick and at operators into
//  consideration while building the ROM so made changes to include
//  these operators. 
//  --srinivas
//
//  Revision 1.21  2000/11/18 00:18:27  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.20  2000/11/02 23:11:54  cvs
//  Some changes to the M__Value method so that it returns just an
//  Expression and not a Formula only 
//  --srinivas
//
//  Revision 1.19  2000/10/30 22:13:23  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas
//
//  Revision 1.18  2000/08/30 20:12:17  cvs
//  Changed the function isTickExpression in the class SOM_Expression
//  --srinivas
//
//  Revision 1.17  2000/08/01 22:48:47  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.16  2000/07/14 23:01:22  cvs
//  Made changes to the meta functions and added some meta functions
//  to the identifier object. 
//  --srinivas
//
//  Revision 1.15  2000/06/22 17:22:41  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.14  2000/05/18 18:03:42  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I 
//  am committing the files again.
//  -- Cindy
//
//  Revision 1.14  2000/05/05 19:28:30  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.13  2000/04/28 19:34:06  cvs
//  Added some more of typechecking. Added parsing of the $ operator
//  and constant literal. 
//  -- Cindy
//
//  Revision 1.12  2000/04/21 19:54:11  cvs
//  Made some changes to the meta function M__Label so that it returns
//  a SOM_Label 
//  --srinivas
//
//  Revision 1.11  2000/04/21 19:05:10  cvs
//  Made some changes to the meta function M__Label so that the return
//  value is SOM_Identifier 
//  --srinivas
//
//  Revision 1.10  2000/04/10 17:44:37  cvs
//
//
//   Made a slight modification to the Meta Function M__String
//  --srinivas
//
//  Revision 1.9  2000/04/09 01:50:03  cvs
//   The identifier had a space printed in the publishSOM which was
//   not required for the tick and the at operators so changing the
//   classes and the grammar had the tick and single quote
//   interchanged so modifing the grammar also 
//  --srinivas
//
//  Revision 1.8  2000/04/08 02:21:58  cvs
//  The Meta Functions have been implemented and these are the files
//  that had to be modified.Addded some accessor methods to classes
//  like SOM_DeclarationStatement, SOM_Expression,
//  SOM_RecordTupleDeclarationStatement.The Meta functions for facet
//  terms and the term items are implemented  
//  --srinivas
//
//  Revision 1.7  2000/04/06 23:16:58  cvs
//  Added a method to determine if an expression is pre or post condition
//  - Krishna and Makarand
//
//  Revision 1.6  2000/04/06 19:13:19  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.5  2000/03/23 00:22:46  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.4  2000/02/10 00:37:15  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.3  2000/02/08 18:42:24  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.2  1999/11/05 23:43:09  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:33  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:51  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.9  1999/10/01 15:23:25  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.8  1999/08/01 20:54:29  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.7  1999/07/28 01:59:37  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.6  1999/07/22 22:53:17  rkamath
//  Lot of changes especially in terms of adding accessors to the OM
//  This does not compile completely, the toString() of SOM_FormalIdentifier
//  is to be completed. This commit is purely for records
//
//  Revision 1.5  1999/07/14 23:23:37  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.4  1999/06/30 22:19:54  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//  Revision 1.3  1999/06/30 02:18:59  rkamath
//  Added a lot more files dealing with the expression OM.
//  Also modified some previous classes
//  Now the ROOT in the hierarchy is the SOM_Unifier interface.
//
//  Revision 1.2  1999/06/25 15:10:25  rkamath
//  Modified the Makefile and changed the javadoc comments in the java files
//
//  Revision 1.1  1999/06/23 18:35:03  rkamath
//  A fresh new addition and modification of new files
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

import java.io.PrintWriter;
import java.io.StringWriter;
import ROM.*;
/** This represents the <B>Expression</B> object (it is more correctly an
 * <B>IffExpression</B> object).
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_Expression
extends SOM_ExpressionWithDyadicOperator {
	// This represents the <expression> object, for which the grammar is
	// <expression> ::== <termExpression> (<expressionTail>)*
	// <expressionTail> ::== == <termExpression>

	 /** The primary Constructor
		* @return Not applicable
		*/
	 public SOM_Expression() {
		 super();
	 }

	 /** To set the termExpression
		* @param expressionIn The SOM_TermExpression object to be set
		* @return void
		*/
	 public void setTermExpression(SOM_TermExpression expressionIn) {
		 setStrongerExpression(expressionIn);
	 }

	 /** To get the termExpression
		* @return The SOM_TermExpression object for this expression
		*/
	 public SOM_TermExpression getTermExpression() {
		 return (SOM_TermExpression)getExpression();
	 }

	 /** To add to the expressionTailList
		* @param eTail The SOM_ExpressionTail object to be added
		* @return void
		*/
	 public void addToExpressionTailList(SOM_ExpressionTail eTail) {
		 addToAbstractExpressionTailList(eTail);
	}

	/** To retrieve the listEnumeration of the expressionTailList
	 * @return The ListEnumeration corresponding to the expressionTailList
	 */
	public ListEnumeration expressions() {
		return getExpressionTailList().elements();
	}

	// added by krishna and makarand on Apr 6, 2000
	/** To determine if expression is pre or post condition
	 * @return The boolean value, true if it is a post condition and
	 * false otherwise 
	 */
	public boolean isTickExpression() {
		try {
			String ret_str = new String();
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw, true);
			this.publishSOM(pw);
			pw.flush();
			pw.close();

			ret_str = sw.toString();
			sw.close();
			
			if((ret_str.indexOf('\'')>0) ||(ret_str.indexOf('@')> 0))
			    return true;
			else
			    return false;
		}
		catch (Exception e) {
		    System.out.println(e);
		    return false;
		}
	}
    
    
    // added by srinivas  on Mar 28, 2000
    /** To retrieve the SOM_Factor
     * @return The SOM_Factor corresponding to this expression
     */
	public SOM_Factor getFactor() {
		SOM_TermExpression termExpr = getTermExpression();
		SOM_ImpliesExpression impliesExpr = termExpr.getImpliesExpression();
		SOM_AndExpression andExpr = impliesExpr.getAndExpression();
		SOM_RelationalTermExpression relTermExpr =
			andExpr.getRelationalTermExpression(); 
		SOM_SimpleExpression simpleExpr = relTermExpr.getSimpleExpression();
		SOM_Term term = simpleExpr.getTerm();
		SOM_ExpTerm expTerm = term.getExpTerm();
		
		return expTerm.getFactor();
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
     *@param Universe root
	 * @return Expression
     */
	public Expression M__Value(Factory romFactory, BaseObject parent,
							   Universe root){
		Expression leftNode;
		ListEnumeration expressionTailEnum = expressions();
		if (expressionTailEnum != null){
			if(expressionTailEnum.hasMoreElements()){
				Formula intermExpr = new Formula(parent);
				leftNode =getTermExpression().
					M__Value(romFactory, intermExpr, root);
				Formula retExpr = intermExpr;
				while (expressionTailEnum.hasMoreElements()) {
					SOM_ExpressionTail exprTail = 
					(SOM_ExpressionTail)expressionTailEnum.nextElement();
					SOM_DyadicOperator oper =exprTail.getEqualEqualOperator();
					Expression rightNode = exprTail.getTermExpression().
						M__Value(romFactory, intermExpr, root);
					intermExpr.setLeftOp(leftNode);
					String str = oper.toString();
					Type operType = 
						root.getType(oper.getResolvedType().toString());
					OperatorReference operRef = new
						OperatorReference(intermExpr,
										  root.labelLookup(str), operType);
					intermExpr.setOperator(operRef);
					intermExpr.setRightOp(rightNode);
					retExpr = intermExpr;
					leftNode = intermExpr;
					intermExpr = new Formula(parent);
				}
				return retExpr;
			}else {
				leftNode = getTermExpression().
					M__Value(romFactory,parent,root);
				return leftNode;
			}
		}else {
			leftNode = getTermExpression().
				M__Value(romFactory,parent,root);
			return leftNode;
		}
	}

		
		
	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		ListEnumeration expressionTailEnum = this.expressions();
		SOM_ExpressionTail exprTail;

		getExpression().publishSOM(pw);

		while (expressionTailEnum.hasMoreElements()) {
			exprTail = (SOM_ExpressionTail)expressionTailEnum.nextElement();
			exprTail.publishSOM(pw);
		}
	}

	/** The publishPVS routine for this class
	 * @return void
	 * Even though this method is exactly same as publishSOM
	 * above it is needed to traverse the PTOM (parse tree object model)
	 * and access individual publishPVS routines which may be different from
	 * corresponding publishSOM routines
	 */
	public void publishPVS(PrintWriter pw) {
		ListEnumeration expressionTailEnum = this.expressions();
		SOM_ExpressionTail exprTail;
		
		getExpression().publishPVS(pw);
		
		while (expressionTailEnum.hasMoreElements()) {
			exprTail = (SOM_ExpressionTail)expressionTailEnum.nextElement();
			exprTail.publishPVS(pw);
		}
	}
		
}





