//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_FacetDeclaration.java,v 1.35 2001/03/21 00:05:05 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_FacetDeclaration.java,v $
//  Revision 1.35  2001/03/21 00:05:05  cvs
//  Added the "design" mode for parameters (SOM_Design.java). Changed the
//  semantics for export clauses. Removed the expression member from
//  SOM_FacetDeclaration.java and SOM_PackageDeclaration.java. Added code
//  to catch error when type-checking fails. Removed the redundant methods
//  getExportList() from SOM_FacetDeclaration.java,
//  SOM_PackageDeclaration.java and SOM_ComponentDeclaration.java.
//  -- Cindy
//
//  Revision 1.34  2001/03/12 23:54:38  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.33  2001/02/07 20:51:08  cvs
//  There was an error causing an infinite loop when doing label lookup in
//  use clauses. Modifications were made in M__Value and buildROM methods
//  to remove error.
//  -- Cindy
//
//  Revision 1.32  2000/12/07 01:23:18  cvs
//  Adding the M__Value to all the Quantifiers. Making minor changes
//  to the other classes 
//  --srinivas
//
//  Revision 1.31  2000/12/05 03:05:03  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.30  2000/12/03 23:55:26  cvs
//  Added the M__Value method to the Literal classes.Previously they
//  were returning null.The M__Type method for all the Declaration
//  statement classes has been completed 
//  --srinivas
//
//  Revision 1.29  2000/11/13 21:50:35  cvs
//  Changed the method M__Value in these classes so that they return
//  the complete Expression. 
//  --srinivas
//
//  Revision 1.28  2000/11/06 18:12:01  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.27  2000/10/22 18:42:02  cvs
//  Meta functions are added to the SOM_InterfaceItem class and the
//  meta function M__Domain is changed so that it returns a string and
//  not SOM_FacetExpression 
//  --srinivas
//
//  Revision 1.26  2000/10/04 03:05:25  cvs
//  Changed some of the meta functions and the facetDeclaration and
//  componentDeclration no longer implement the design unit.This is
//  because all the files should be a package now 
//  --srinivas
//
//  Revision 1.24  2000/09/26 04:15:43  cvs
//  Added some new meta functions that could be used in the loader
//  --srinivas
//
//  Revision 1.23  2000/07/21 06:35:27  cvs
//  Made some changes to the M__Type function in both the
//  declarationStatement and the facetDeclaration 
//  --srinivas
//
//  Revision 1.22  2000/07/14 23:01:22  cvs
//  Made changes to the meta functions and added some meta functions
//  to the identifier object. 
//  --srinivas
//
//  Revision 1.21  2000/06/22 17:22:41  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.20  2000/05/18 18:03:43  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I am committing the files again.
//  -- Cindy
//
//  Revision 1.20  2000/05/05 19:28:31  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.19  2000/04/28 19:34:07  cvs
//  Added some more of typechecking. Added parsing of the $ operator
//  and constant literal. 
//  -- Cindy
//
//  Revision 1.18  2000/04/21 19:54:11  cvs
//  Made some changes to the meta function M__Label so that it returns
//  a SOM_Label 
//  --srinivas
//
//  Revision 1.17  2000/04/21 02:09:38  cvs
//  Made changes to the publishSOM routine so that the ObjectModel
//  file is correct 
//  --srinivas
//
//  Revision 1.16  2000/04/12 18:41:05  cvs
//  Implemented some type checking for "use" clause.
//  -- Cindy
//
//  Revision 1.15  2000/04/11 03:17:52  cvs
//  The dummy M__Label and M__Value functions have been removed .
//  --srinivas
//
//  Revision 1.14  2000/04/09 01:50:03  cvs
//   The identifier had a space printed in the publishSOM which was
//   not required for the tick and the at operators so changing the
//   classes and the grammar had the tick and single quote
//   interchanged so modifing the grammar also 
//  --srinivas
//
//  Revision 1.13  2000/04/08 02:21:57  cvs
//  The Meta Functions have been implemented and these are the files
//  that had to be modified.Addded some accessor methods to classes
//  like SOM_DeclarationStatement, SOM_Expression,
//  SOM_RecordTupleDeclarationStatement. The Meta functions for facet 
//  terms and the term items are implemented   
//  --srinivas
//
//  Revision 1.12  2000/03/23 00:22:46  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.11  2000/03/03 18:38:28  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.10  2000/02/17 01:27:04  cvs
//  Implemented the facet expression grammar to incorporate facet composition.
//  Also updated the "use" clause with the above changes.
//  - krishna.
//
//  Revision 1.9  2000/02/13 01:15:53  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. 
//  Modified the grammar to incorporate these changes. Also populated
//  the object model with these new objects and completed the
//  pretty-printing for these new constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//  Revision 1.8  2000/02/10 00:37:17  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.7  2000/02/08 18:42:26  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.6  1999/11/16 04:05:23  cvs
//  Added support for export clause and visiblity
//
//  Revision 1.5  1999/11/15 22:59:15  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.4  1999/11/13 20:01:26  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.3  1999/11/11 00:04:20  cvs
//  Fresh changes as part of the streamlining process with new
//  object model and grammar
//
//  Revision 1.2  1999/11/05 23:43:09  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:34  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:59  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.3  1999/10/20 20:40:06  rkamath
//  Commit before the archive moves over to Kansas
//
//  Revision 1.2  1999/07/13 14:27:16  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.1  1999/07/09 22:21:11  rkamath
//  A lot of changes to incorporate the intent of type checking
//
//
//=============================================================================
package SOM;
import util.List;
import util.Link;
import util.ListEnumeration;

import java.io.*;
import java.lang.NullPointerException;
import java.io.Serializable;
import java.lang.String;

import ROM.*;

/** This represents the <B>FacetDeclaration</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
    // made serializable by makarand

public class SOM_FacetDeclaration
extends SOM_Declaration
implements SOM_FacetDeclarativeItem, SOM_PackageDeclarativeItem, Serializable {
    
    /** The name of this Facet - and other dataMembers */
    private SOM_Identifier id;
    private SOM_InterfaceListing interfaceList = null;
    private SOM_FacetDefinition facetDefinition = null;
    
    private transient SOM_DeclarationList domainList;
    
    /** The primary constructor
     * @param facetTypeIn The typeDeclaration corresponding to the "facet type"
     * @return Not applicable
     */
    public SOM_FacetDeclaration(SOM_TypeDeclaration facetTypeIn) {
		super();
		setKind(FACET);
		setType(facetTypeIn);
    }
    
    /** To set the identifier
     * @param idIn The SOM_Identifier to be set
     * @return void
     */
    public void setIdentifier(SOM_Identifier idIn) {
		id = idIn;
    }			 
    
    /** To get the identifier
     * @return The SOM_Identifier for this designUnit
     */
    public SOM_Identifier getIdentifier() {
		return id;
    }			 
    
    /** To set the interfaceListing - implements the function from
     * SOM_DesignUnit
     * @param interfaceIn The SOM_InterfaceListing object to be set
     * @return void
     */
    public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
		interfaceList = interfaceIn;
    }
    
    /** To get the interfaceListing - implements the function from
     * SOM_DesignUnit
     * @return The SOM_InterfaceListing object for this designUnit
     */
    public SOM_InterfaceListing getInterfaceListing() {
		return interfaceList;
    }
    
    /** To set the domainList - implements the function from
     * SOM_DesignUnit
     * @param domainListIn The SOM_DeclarationList object to be set
     * @return void
     */
    public void setDomainList(SOM_DeclarationList domainListIn) {
		domainList = domainListIn;
    }
    
    /** To get the domainList - implements the function from
     * SOM_DesignUnit
     * @return The SOM_DeclarationList object for this designUnit
     */
    public SOM_DeclarationList getDomainList() {
		return domainList;
    }
    
    /** To set the facetDefinition
     * @param facetDefIn The SOM_FacetDefinition object to be set
     * @return void
     */
    public void setFacetDefinition(SOM_FacetDefinition facetDefIn){
		facetDefinition = facetDefIn;
    }
    
    /** To get the facetDefinition
     * @return The SOM_FacetDefinition object to get
     */
    public SOM_FacetDefinition getFacetDefinition(){
		return facetDefinition;
    }
    
    /** added by krishna on Mar 5, 2000
     * To get the desired object by traversal of SOM
     */
    public ListEnumeration M__Terms() {
		List terms = new List();
		SOM_FacetBody facetBody = getFacetDefinition().getFacetBody();
		if(facetBody != null){
			ListEnumeration items = facetBody.items();
			if (items !=null){
				while (items.hasMoreElements()){
					SOM_FacetBodyItem facetItem =
						(SOM_FacetBodyItem)items.nextElement();
					if (facetItem !=null){
						terms.addElement
							(((Object)facetItem.getLabelledExpression()));
					}
				}
			}
		}
		if (terms.isEmpty())
			return null;
		else
			return terms.elements();
    }
    
    /** Added by srinivas on march 28 2000
     *To get all the parameters that are passed to the facet.It
	 * returns objects of type SOM_InterfaceItem
	 * @param void
	 * @return ListEnumeration
	 */
    public ListEnumeration M__Params(){
		List parameters = new List();
		if (getInterfaceListing() !=null){
	    ListEnumeration  interListEnum =
			getInterfaceListing().interfaces();
	    
	    SOM_InterfaceItem interItem;
	    if (interListEnum != null){
			while (interListEnum.hasMoreElements()){
				interItem = (SOM_InterfaceItem)
					interListEnum.nextElement();
		    if(interItem !=null)
				parameters.addElement((Object)interItem);
			}
	    }
		}
		if (parameters.isEmpty()){
			return null;
		}
		else{
			return parameters.elements();
		}
    }
    
    /**Added by srinivas on march 28 2000
     *To get all the physical variables that are defined in
     * the facet
	 * param void
	 * return ListEnumeration
	 */
    public ListEnumeration M__Pvars(){
		ListEnumeration facetDeclItemEnum;
		List pVars = new List();
		ListEnumeration idListEnum;
		SOM_FacetDefinition facetDef = getFacetDefinition();
		if (facetDef.getFacetDeclarativeBlock() != null){
			facetDeclItemEnum =
				facetDef.getFacetDeclarativeBlock().items();
			if (facetDeclItemEnum !=null){
				while (facetDeclItemEnum.hasMoreElements()){
					SOM_FacetDeclarativeItem facetDeclItem =
						((SOM_FacetDeclarativeItem)facetDeclItemEnum.
						 nextElement());
					
					if (facetDeclItem instanceof
						SOM_FacetDeclaration){
						if (((SOM_FacetDeclaration)facetDeclItem)
							!=null) 
							pVars.addElement(((Object)
											  ((SOM_FacetDeclaration)
											   facetDeclItem)
											  .getIdentifier()));
						
					} else if (facetDeclItem instanceof
							   SOM_DeclarationStatement){
						pVars.addElement(((Object)
										  ((SOM_DeclarationStatement)
										   facetDeclItem)
										  .getIdentifier()));
						if (((SOM_DeclarationStatement)facetDeclItem)
							.getIdentifierListing() !=null){
							idListEnum = ((SOM_DeclarationStatement)
										  facetDeclItem).
								getIdentifierListing().identifiers();
							while(idListEnum.hasMoreElements()){
								pVars.addElement((Object)idListEnum.
												 nextElement());
							}
						}
					}
				}
			}
		}
		if (pVars.isEmpty()){
			return null;
		} else
			return pVars.elements();
    }
	
    /**Added by srinivas on march 28 2000
	 *To get all the types that are defined in the entire facet
	 * @param void
	 * return ListEnumeration
	 */
    public ListEnumeration M__Types(){   
        List types = new List();
		ListEnumeration declStmts = getDeclarationStatements();
		if (declStmts !=null){
			while (declStmts.hasMoreElements()){
				Object type =
					((SOM_DeclarationStatement)declStmts.nextElement()).
					M__Type();
				if (type !=null)
					types.addElement(type);
			}
		}
		if (getInterfaceListing() !=null){
			ListEnumeration  interListEnum =
				getInterfaceListing().interfaces();
			if (interListEnum != null){
				while (interListEnum.hasMoreElements()){
					String nametype =
						((SOM_TypeDeclaration)
						 ((SOM_InterfaceItem)interListEnum.nextElement()).
						 getType()).toString();
					if (nametype !=null)
						types.addElement(nametype);
				}
			}
		}
		if (types.isEmpty())
			return null;
		else
			return types.elements();
    }
    
    
    /**Added by srinivas on march 28 2000
     *To get all the domain of the facet 
	 * @param void
	 * @return String
     */
    public String  M__Domain(){
        SOM_FacetDefinition facetDefn = getFacetDefinition();
		try{
			if (facetDefn !=null){
				String str = new String();
				StringWriter sw = new StringWriter();
				PrintWriter pw = new PrintWriter(sw,true);
				if (facetDefn.getDomainExpression() != null)
					facetDefn.getDomainExpression().publishSOM(pw);
				pw.flush();
				pw.close();
				str = sw.toString();
				sw.close(); 
				return str;
			}else return null;
		}catch (Exception e) {
			System.out.println(e);
			return (String)null;
		}
    }	
    
	/**Added by srinivas on march 28 2000
	 *To get all the visible labels that are defined in the
	 * entire facet
	 * @param void
	 * return ListEnumeration
	 */
    public ListEnumeration M__Visible(){
		List visible = new List();
		ListEnumeration pvars = M__Pvars();
		ListEnumeration params = M__Params();
		if (pvars !=null){
		    while (pvars.hasMoreElements())
				visible.addElement(((Object)pvars.nextElement()));
		}
		if (params != null){
		    while (params.hasMoreElements())
				visible.addElement(((Object)params.nextElement()));
		}
		SOM_FacetBody facetBody = getFacetDefinition().getFacetBody();
		if (facetBody !=null){
		    ListEnumeration facetBodyItemEnum = facetBody.items();
		    while (facetBodyItemEnum.hasMoreElements()){
				SOM_FacetBodyItem facetBodyItem =
					((SOM_FacetBodyItem)facetBodyItemEnum.nextElement());
				SOM_LabelledExpression labelledExpr = 
					facetBodyItem.getLabelledExpression();
				SOM_Label label= labelledExpr.getLabel();
				if (label !=null) {
					visible.addElement((Object)label.getIdentifier());    
				}
		    }
		}
		
		if (visible.isEmpty())
		    return null;
		else
		    return visible.elements();
    }
    
    /** Added by srinivas on march 28 2000
     * To get all the items that are defined in the facet body
	 * @param void
	 * @return ListEnumeration
     */
    public ListEnumeration M__Items(){
		List items = new List();
		ListEnumeration params = M__Params();
		ListEnumeration pvars = M__Pvars();
		ListEnumeration terms = M__Terms();
		ListEnumeration types = M__Types();
	    String domain = M__Domain();
		ListEnumeration visibles = M__Visible();
		
		if (pvars!=null){
			while (pvars.hasMoreElements())
				items.addElement(pvars.nextElement());
		}
		if (params !=null){
			while (params.hasMoreElements())
				items.addElement(params.nextElement());
		}
		if (terms !=null){
			while (terms.hasMoreElements())
				items.addElement(terms.nextElement());
		}
		if (types !=null){
			while (types.hasMoreElements())
				items.addElement(types.nextElement());
		}
		if (domain !=null){
			items.addElement(domain);
		}
		if (visibles!=null){
			while (visibles.hasMoreElements())
				items.addElement(visibles.nextElement());
		}
		if (items.isEmpty())
		    return null;
		else
		    return items.elements();
    }
    
    /** Added by srinivas on march 28 2000
     * To get all the declaration statements that are declared
     * in the facet
	 * @param void
	 * @return ListEnumeration
     */
    public ListEnumeration getDeclarationStatements(){
		SOM_FacetDeclarativeBlock declBlock =
			facetDefinition.getFacetDeclarativeBlock();
		List declStmts = new List();
		if (declBlock != null){
			ListEnumeration facetDeclStmts = declBlock.items();
			if (facetDeclStmts !=null){
				while(facetDeclStmts.hasMoreElements()){
					SOM_FacetDeclarativeItem declItem =
						(SOM_FacetDeclarativeItem)facetDeclStmts.nextElement();
					if (declItem instanceof SOM_DeclarationStatement){
						SOM_DeclarationStatement declStmt =
							((SOM_DeclarationStatement)declItem);
						declStmts.addElement(declStmt);
					}
				}
			}
		}
		if (declStmts.isEmpty())
			return null;
		else
			return declStmts.elements();
    }

    /** To get the exports as a ListEnumeration 
     * @return ListEnumeration corresponding to the export clauses
     */
    public ListEnumeration exports() {
		SOM_ExportBlock exportBlock = facetDefinition.getExportBlock();
		if (exportBlock != null) 
			return exportBlock.exports();
		else
			return null;
    }
        
    /** To get an enumeration of the parents' Declarations for this facet
     * @return ListEnumeration representing the list of the parents' 
     * declarations for this facet
     */
    public ListEnumeration parents() {
		try {
			SOM_DeclarationList parentList =
				facetDefinition.getParentDeclarationList(); 
			return parentList.elements();
		} catch (NullPointerException except) {
			return new ListEnumeration(null);
		}
    }
    
    /** Added by srinivas on june 26 2000
     * To get the label of a facet item
     * in the facet
     */  
    public SOM_Identifier M__Label(){
		return getIdentifier();
    }
    /** Added by srinivas on june 26 2000
     * To get the type of a facet item
     * in the facet
     */  
    public String M__Type(){
		return getType().toString();
    }
    /** Added by srinivas on june 26 2000
     * To get the String of a facet item
     * in the facet
     */  
    public String M__String(){
		try{
			String str = new String();
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw,true);
			publishSOM(pw);
			pw.flush();
			pw.close();
			str = sw.toString();
			sw.close(); 
			return str;
		}
		catch (Exception e) {
			System.out.println(e);
			return (String)null;
		}
    }
    /** Added by srinivas on june 26 2000
     * To get the value of a facet item
     * in the facet
     */  
    public SOM_FacetDeclaration M__Value(){
		return this;
    }
	
    
    /** To get the string representation of this object. Overriding the
     * method in SOM_Declaration 
     * @return String The "name" of the facet corresponding to this
     * declaration 
     */
    public String toString() {
		return id.toString();
    }
    
	// Added by srinivas on Oct 27th, 2000. This function returns the
	// ROM.Expression 
	/** The function that creates the ROM.Expression for the facet 
	 * item
	 * @param Factory,BaseObject,Universe
	 * @return ROM.Expression(ROM.Facet)
	 */
	public Expression M__Value(Factory romFactory, BaseObject
							   parent, Universe root){

		Facet facetValue = (Facet)parent;
				
		// ***** Populating the facet value ***** //
		
		// Populating the interface items
		ListEnumeration paramList = M__Params();
		SOM_InterfaceItem interItem;
		if (paramList != null) {
			while (paramList.hasMoreElements()) {
				interItem = (SOM_InterfaceItem)paramList.nextElement();
				facetValue.addInterfaceItem
					(interItem.buildROM(romFactory, root, facetValue));
			}
		}

		// Populating the declared items
		ListEnumeration declList = getDeclarationStatements();
		SOM_FacetDeclarativeItem facetDeclItem;
		// It is being assumed that a facet will always have some
		// declared items
		if (declList != null){
			while (declList.hasMoreElements()) {
				facetDeclItem = 
					(SOM_FacetDeclarativeItem)declList.nextElement();
				if (facetDeclItem instanceof SOM_FacetDeclaration)
					facetValue.addItem
						(((SOM_FacetDeclaration)facetDeclItem).
						 buildROM(romFactory, root, facetValue));
				

				if (facetDeclItem instanceof SOM_DeclarationStatement) {
					facetValue.addItem
						(((SOM_DeclarationStatement)facetDeclItem).
						 buildROM(romFactory, root, facetValue));
				}
			}

			// To be completed
			//if (facetDeclItem instanceof SOM_UseClause)
		}

		// Populating the domain
		Label domainLabel = romFactory.newLabel
			((Item)facetValue.getParent(), M__Domain()); 
		facetValue.setDomain(domainLabel);

		// Populating the export list
		ListEnumeration exportList = exports();
	    SOM_Identifier exportId;
		if (exportList != null) {
			if (!exportList.hasMoreElements()) {
				facetValue.addExportLabel
					(romFactory.newLabel((Item)facetValue.getParent(),
										 "ALL"));
			} else {										  
				while (exportList.hasMoreElements()) {
					exportId = (SOM_Identifier)exportList.nextElement();
					facetValue.addExportLabel
						(romFactory.newLabel
						 ((Item)facetValue.getParent(), exportId.toString()));
				}
			}
		}
		ListEnumeration terms = M__Terms();
		if (terms != null){
			while(terms.hasMoreElements()){
				SOM_LabelledExpression labelExpr =
					((SOM_LabelledExpression)terms.nextElement());
				if (labelExpr != null)
					facetValue.addTerm
						(labelExpr.buildROM(romFactory, root, facetValue));
			}
		}
		// To be completed
		// Populating the use clause list

		return (Expression)facetValue;
	}

   
	// Added by srinivas on Oct22nd, 2000
	/** The buildROM method for this high-level item
	 * @param ROMFactory, Factory object used to create objects in ROM
	 * @return LabelledItem, corresponding to this facet  
	 */
	public LabelledItem buildROM(Factory romFactory, 
								 Universe root, BaseObject parentIn) {
		LabelledItem facetItem = romFactory.newLabelledItem
			(parentIn, null, null, null, M__String());
		Label facetLabel = romFactory.newLabel(facetItem, toString());
		facetItem.setLabel(facetLabel);
		// Creating the value first
		ROM.Facet facetValue = romFactory.newFacet(facetItem);
		((FacetType)root.getType("facet")).addOneElement(facetValue);
		facetItem.setValue(facetValue);
		// The object facetValue is being modified directly but not created
		M__Value(romFactory,(BaseObject)facetValue, root);
		Type facetType = root.getType("facet");
		facetItem.setType(facetType);

		return facetItem;
	}


    /** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
		pw.print("FACET ");
		id.publishSOM(pw);
		if (interfaceList != null) {
			interfaceList.publishSOM(pw);
		}
		pw.println(" IS ");
		facetDefinition.publishSOM(pw);
		id.publishSOM(pw);
		pw.print(";\n");
		pw.println();
		
    }
}




