//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_FormalName.java,v 1.18 2001/03/12 23:54:38 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_FormalName.java,v $
//  Revision 1.18  2001/03/12 23:54:38  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.17  2001/03/06 05:02:27  cvs
//  The M__Value method has been implemented for SelectedName and some
//  errors in the Meta functions have been removed in the other files 
//  --srinivas
//
//  Revision 1.16  2000/12/07 01:23:19  cvs
//  Adding the M__Value to all the Quantifiers. Making minor changes
//  to the other classes 
//  --srinivas
//
//  Revision 1.15  2000/12/05 03:05:03  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.14  2000/11/27 23:49:56  cvs
//  Previously was not taking the tick and at operators into
//  consideration while building the ROM so made changes to include
//  these operators. 
//  --srinivas
//
//  Revision 1.13  2000/11/18 00:18:27  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.12  2000/11/13 21:50:35  cvs
//  Changed the method M__Value in these classes so that they return
//  the complete Expression. 
//  --srinivas
//
//  Revision 1.11  2000/10/30 22:13:23  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas
//
//  Revision 1.10  2000/08/10 00:38:29  cvs
//  added publishPVS routine in Formal name class and modified the
//  exisiting publishPVS routines in AttickExpression and Identifier
//  to remove an error during conversion of tick to underscore - makarand
//
//  Revision 1.9  2000/08/01 22:48:48  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.8  2000/07/11 00:11:10  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are 
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.7  2000/06/22 17:22:42  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.6  2000/04/06 19:13:21  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.5  2000/02/10 00:37:19  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:28  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.3  1999/11/17 18:31:54  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.2  1999/11/13 20:01:26  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/11/11 00:04:23  cvs
//  Fresh changes as part of the streamlining process with new
//  object model and grammar
//
//
//=============================================================================
package SOM;

import SYM.SYM_UnresolvableException;
import SYM.SYM_StaticFunctions;

import SOM.SOM_DeclarationList;
import ROM.*;
import util.ListEnumeration;
import java.io.*;
import java.io.PrintWriter;

/** This represents the <B>FormalName</B> object
 * <formal_name> ::== <name> <association_list>
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_FormalName
extends SOM_Name {

  /** The name in this formalName */
  private SOM_Name name;
  /** The asociationListing for this formalName */
  private SOM_AssociationListing assocListing;

	/** The flag indicating that this formalName is being used to
	 * declare a type. 
	 * It is necessary because the process of resolving the type is
	 * different in this case. 
	 */
	private boolean isTypeDeclaration = false;

  /** The primary Constructor
   * @return Not applicable
   */
  public SOM_FormalName() {
		super();
  }

	/** To set the name
	 * @param nameIn The SOM_Name to be set
	 * @return void
	 */
	public void setName(SOM_Name nameIn) {
		name = nameIn;
	}

	/** To get the name - overrides the getName in SOM_Name
	 * @return SOM_Name that is the name in this formalName
	 */
	public SOM_Name getName() {
		return name;
	}
	 
	/** To set the associationListing
	 * @param listingIn The SOM_AssociationListing to be set
	 * @return void
	 */
	public void setAssociationListing(SOM_AssociationListing listingIn) {
		assocListing = listingIn;
	}

	/** To get the associationListing
	 * @return The SOM_AssociationListing stored here
	 */
	public SOM_AssociationListing getAssociationListing() {
		return assocListing;
	}

	/** Overrides the setDeclarations from SOM_Name - this was dicated by
	 * resolveFinalType since it needed the lower children of this
	 * name for each type in the typeList. Hence the typeList is now
	 * being set in the lookUp of the symboltable and shouldn't be set
	 * here!!! This was a HACK 
	 * @param declsIn The SOM_DeclarationList to be set
	 * @return void
	 */
	public void setDeclarations(SOM_DeclarationList declsIn) {
		declList = declsIn;
	}

	/** Makes public the protected setTypeList of
	 * SOM_AbstractExpression. Needed since the setDeclarations is not
	 * to set the typeList 
	 * @param listIn The SOM_TreeNodeList to be set as the typeList
	 * @return void
	 */
	public void setTypeList(SOM_TreeNodeList listIn) {
		super.setTypeList(listIn);
	}
	
	// Put here in SOM_FormalName for the moment - Cindy
	/** Sets the typeList for a declaration
	 * @param declList The declList for the name of the function from
	 * Symboltable 
	 * @return void
	 */
	public void resolveDeclarationType(SOM_DeclarationList declList,
									   String fncName) { 
		ListEnumeration assocEnum = assocListing.associations();
		ListEnumeration declEnum = declList.elements();

		SOM_Expression currItem;
		SOM_ArgumentList assocAsArgs = new SOM_ArgumentList();
		SOM_FunctionDeclaration currDecl;
		SOM_DeclarationList domainList;
		while (assocEnum.hasMoreElements()) {
			currItem = (SOM_Expression)assocEnum.nextElement();			
			assocAsArgs.addElement(currItem.getTypeList());
		}
		while (declEnum.hasMoreElements()) {
			currDecl = (SOM_FunctionDeclaration)declEnum.nextElement();
			domainList = 
				((SOM_FunctionType)currDecl.getType()).getDomainList();
			if (domainList.countElements() == 1 &&
				assocAsArgs.countElements() == 1) 
				{
					if (fncName.equalsIgnoreCase("type") || 
						fncName.equalsIgnoreCase("bunch")) {
						setTypeList((SOM_TreeNodeList)assocAsArgs.first());
						isTypeDeclaration = true;
						break;
					}
					SOM_TreeNodeList treeList =
						SYM_StaticFunctions.buildTree(declList, assocAsArgs);
					setTypeList(treeList);

					//if (fncName.equalsIgnoreCase("array")) {
					//	SOM_TreeNodeList resultTrees = new SOM_TreeNodeList();
					//	SOM_Expression expr = (SOM_Expression)
					//		assocListing.associations().nextElement();
					//	expr.resolveFinalType();
					//	SOM_TypeDeclaration elementType =
					//		expr.getResolvedType(); 
					//	SOM_ArrayDeclaration decl = 
					//		new SOM_ArrayDeclaration(elementType,
					//								 currDecl.getType()); 
					//	SOM_TreeNode tempNode = new SOM_TreeNode(decl);
					//	resultTrees.addElement(tempNode);
					//	
					//	setTypeList(resultTrees);
					//	}
				}
		}

	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		// The set should have resulted in a single type.
		SOM_TreeNodeList typeList = getTypeList();
		//if (typeList == null)
		//	System.out.println("typeList is null in SOM_FormalName");
		if (typeList.countElements() != 1)
			throw  new SYM_UnresolvableException
				("Cannot resolve final type for expression (FormalName)");
		
		// else the final type is the sole element of the typeList.
		SOM_TreeNode onlyNode = (SOM_TreeNode)typeList.first();
		if (!isTypeDeclaration) {			
			resolveFinalType(onlyNode.getResultType());
		} else {
			setResolvedType(onlyNode.getResultType());
			isTypeDeclaration = false;
		}
	}


	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		// typeIn should be one of the types in typeList

		ListEnumeration typeEnum = getTypeList().elements();
		SOM_TreeNode currNode;
		SOM_TypeDeclaration currType;
		SOM_DeclarationList currDomainList;

		ListEnumeration assocEnum;
		ListEnumeration domainEnum;
		SOM_Expression currExpr;
		SOM_TypeDeclaration currDomain;

		//System.out.println("Fresh TCC Generation");

		while (typeEnum.hasMoreElements()) {
			//System.out.println("New Cycle");

			currNode = (SOM_TreeNode)typeEnum.nextElement();
			currType = currNode.getResultType();
			currDomainList = currNode.getDomainList();
			
			try {
				if (SOM_TypeDeclaration.resolveFinalType(currType, typeIn)) {
					
					// now handle each assocation element
					assocEnum = assocListing.associations();
					domainEnum = currDomainList.elements();
					// if we have reached here, then assocEnum and
					// domainEnum should have equal count of elements.
					while (domainEnum.hasMoreElements()) {
						currExpr = (SOM_Expression)assocEnum.nextElement();
						currDomain = 
							(SOM_TypeDeclaration)domainEnum.nextElement();
						currExpr.resolveFinalType(currDomain);
					}
					// All the associations were successfully resolved

					setResolvedType(currType);
					//System.out.println("End Fresh TCC Generation");
					return;
				}
			} catch (SYM_UnresolvableException except) {
				// means that some assocation failed to resolve
				// go to the next currType in typeEnum
				//System.out.println("Flush previous cycle");
				//System.out.println("New Cycle");
			}
		}

		//System.out.println("End Fresh TCC Generation");

		// typeIn was not found in the overload resolved types.
		throw  new SYM_UnresolvableException
			("Cannot resolve final type for expression in FormalName");
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root) {
		Item item = null;
		if (name instanceof SOM_Identifier) {
			String str = ((SOM_Identifier)name).toString();
			Label label =new Label(str);
			item = parent.labelLookup(label);
		}else {
			Expression itemExpr =
				name.M__Value(romFactory,parent,root);
			if (itemExpr instanceof ItemReference){
				item = ((ItemReference)itemExpr).getItemReferred();
			}
		}

		if (item !=null) {
			if (item.getValue() instanceof Facet) {
				FacetReference facetRef = new FacetReference(parent,item);
				if (assocListing != null){
					ListEnumeration exprEnum =
						assocListing.associations();
					if (exprEnum !=null){
						while (exprEnum.hasMoreElements()){
							Object obj = exprEnum.nextElement();
							if (obj instanceof SOM_Expression){
								SOM_Expression expr =
									(SOM_Expression)obj;
								facetRef.addParameter
									(expr.M__Value(romFactory,facetRef,root));
							}
						}
					}
				}
			return (Expression)facetRef;

			} else if (item.getValue() instanceof Function) {
				FunctionReference funcRef = new
					FunctionReference(parent,item);
				if (assocListing != null){
					ListEnumeration exprEnum =
						assocListing.associations();
					if (exprEnum !=null){
						while (exprEnum.hasMoreElements()){
							Object obj = exprEnum.nextElement();
							if (obj instanceof SOM_Expression){
								SOM_Expression expr = (SOM_Expression)obj;
								funcRef.addParameter
									(expr.M__Value(romFactory,funcRef,root));
							}
						}
					}
				}
				return (Expression)funcRef;
			}
		}
		
		return null;
	}
	
	/** publishSOM routine 
	 * @return void
	 */
	public void publishSOM(PrintWriter pw){
		name.publishSOM(pw);
		assocListing.publishSOM(pw);
	}

	/** The publishPVS routine for this class
	 * @return void
     * To be implemented as needed */
	public void publishPVS(PrintWriter pw) {
		name.publishPVS(pw);
		assocListing.publishPVS(pw);
	}	
}
