//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_FunctionDeclaration.java,v 1.7 2001/04/04 22:23:20 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_FunctionDeclaration.java,v $
//  Revision 1.7  2001/04/04 22:23:20  cvs
//  Removed an error in typechecking of record. Type checking now works
//  for recursive functions.
//  -- Cindy
//
//  Revision 1.6  2000/07/11 00:11:10  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are 
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.5  2000/06/22 17:22:42  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.4  2000/04/06 19:13:21  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.3  2000/02/10 00:37:19  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  1999/11/13 20:01:26  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:36  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:11  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.4  1999/10/01 20:00:45  rkamath
//  fixed a bug with init
//
//  Revision 1.3  1999/08/01 20:54:30  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.2  1999/07/16 21:50:30  rkamath
//  Changed everyexpression to point to the type declaration instead of having
//  a private variable which would indicate the resulting type
//
//  Revision 1.1  1999/07/13 19:29:07  rkamath
//  Moved the List and ListEnumeration code to a new util directory
//  Mode the operators as children of AbstractExpression
//  Introduced the FunctionDeclaration and made OperatorDeclaration its child
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

/** This represents the <B>FunctionDeclaration</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_FunctionDeclaration
extends SOM_Declaration {

	protected SOM_Identifier functionName;
	protected boolean isOp = false;
	protected SOM_Expression expr = null;

	/** The primary constructor
	 * @param functionNameIn The SOM_Identifier corresponding to the name
	 * of the function
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * type of the function (Ds -> Rs)
	 * @return Not applicable
	 */
	public SOM_FunctionDeclaration(SOM_Identifier functionNameIn,
								   SOM_TypeDeclaration myType) {
		super();
		functionName = functionNameIn;
		setKind(SOM_Declaration.FUNCTION);
		setType(myType);
	}

	/** A second constructor
	 * @return Not applicable
	 */
	public SOM_FunctionDeclaration() {
		super();
		setKind(SOM_Declaration.FUNCTION);
	}

	/** To set the name of the function declaration
	 * @param functionNameIn The SOM_Identifier corresponding to the
	 * name of the function
	 */
	public void setFunctionName(SOM_Identifier functionNameIn) {
		functionName = functionNameIn;
	}

	/** To set the expression body of the function 
	 * @param exprIn The SOM_Expression of the body of the function
	 * @return void
	 */
	public void setExpression(SOM_Expression exprIn) {
		expr = exprIn;
	}

	/** To get the string representation of this object. Overrides the
	 * method in SOM_Declaration
	 * @return String representation of this object - name of the function
	 */
	public String toString() {
		return functionName.toString();
	}

	/** To ascertain if this is an operator
	 * @return boolean if this is an operator
	 */
	public boolean isOperator() {
		return isOp;
	}

	/** To implement my own version of isDistinguishable for function
	 * declarations
	 * @param declOne The first SOM_FunctionDeclaration
	 * @param declTwo The next SOM_FunctionDeclaration to be distinguished
	 * against declOne
	 * @return true if they are distinguishable, false otherwise
	 */
	public static boolean isDistinguishable(SOM_FunctionDeclaration declOne,
											SOM_FunctionDeclaration	declTwo) { 
		SOM_FunctionType typeOne = (SOM_FunctionType)declOne.getType();
		SOM_FunctionType typeTwo = (SOM_FunctionType)declTwo.getType();
		
		if (typeOne == null) {
			System.out.println
				("WARNING: Function declaration is not complete");
			return false;
		}
		else if(typeTwo == null) {
			System.out.println
				("WARNING: Function declaration is not complete");
			return false;
		}
		else if (!declOne.toString().equalsIgnoreCase(declTwo.toString()))
			return true;
		else
			return(SOM_FunctionType.isDistinguishable(typeOne, typeTwo));
	}

	/** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
	 * @param declLHS The functionDecl corresponding to the lhs object
	 * @param declRHS The functionDecl corresponding to the rhs object
	 * @return true, if the rhs is assignable to the lhs, false otherwise
	 */
	public static boolean
		isLeftCompatibleWithRight(SOM_FunctionDeclaration declLHS,
								  SOM_FunctionDeclaration declRHS) { 
		SOM_FunctionType typeLHS = (SOM_FunctionType)declLHS.getType();
		SOM_FunctionType typeRHS = (SOM_FunctionType)declRHS.getType();

		return SOM_FunctionType.isLeftCompatibleWithRight(typeLHS, typeRHS);
	}
}
