//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_IfExpression.java,v 1.11 2000/11/18 00:18:27 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_IfExpression.java,v $
//  Revision 1.11  2000/11/18 00:18:27  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.10  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.9  2000/10/09 21:09:55  cvs
//  Removed error in the typechecking of an if expression. Test files are
//  now parsing without crashing.
//  -- Cindy
//
//  Revision 1.8  2000/08/01 22:48:49  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.7  2000/07/11 00:11:12  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and sequence are
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.6  2000/06/22 17:22:43  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.5  2000/04/06 19:13:21  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.4  2000/03/23 00:22:47  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.3  2000/03/21 03:13:18  cvs
//  Made changes to the grammar since the if expression production
//  should have an else part compulsory.The expterm had to be modified 
//  --srinivas
//
//  Revision 1.2  2000/03/10 22:36:38  cvs
//  Modified the grammar to include the if expression
//  --srinivas
//
//  Revision 1.1  2000/03/10 21:42:49  cvs
//  Adding the new class if expression
//  --srinivas
//
//  Revision 1.5  2000/02/10 00:37:20  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:30  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.3  1999/11/15 22:59:15  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.2  1999/11/15 03:16:28  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.1  1999/11/05 23:43:05  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//
//=============================================================================
package SOM;

import SYM.SYM_UnresolvableException;

import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;

/** This represents the <B>IfOperator</B> object.
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_IfExpression
extends SOM_AbstractExpression
	implements SOM_Factor {

	private SOM_Expression condExpression;
	private SOM_Expression thenExpression;
	private SOM_Expression elseExpression;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_IfExpression() {
		super();
	}

	/** To set the cond expression
	 * @param expressionIn the SOM_Expression to be set
	 * @return void
	 */
	public void setConditionalExpression(SOM_Expression expressionIn) {
		condExpression = expressionIn;
	}

	/** To get the cond expression
	 * @return The SOM_Expression for the 'condition' part
	 */
	public SOM_Expression getConditionalExpression() {
		return condExpression;
	}

	/** To set the then expression
	 * @param expressionIn the SOM_Expression to be set
	 * @return void
	 */
	public void setThenExpression(SOM_Expression expressionIn) {
		thenExpression = expressionIn;
	}

	/** To get the then expression
	 * @return The SOM_Expression for the 'then' part
	 */
	public SOM_Expression getThenExpression() {
		return thenExpression;
	}

	/** To set the Else expression
	 * @param expressionIn the SOM_Expression to be set
	 * @return void
	 */
	public void setElseExpression(SOM_Expression expressionIn) {
		elseExpression = expressionIn;
	}

	/** To get the Else expression
	 * @return The SOM_Expression for the 'else' part
	 */
	public SOM_Expression getElseExpression() {
		return elseExpression;
	}

	/** To do the semCheck. Overrides the one in SOM_AbstractExpression
	 * The typeList associated with the ifExpression contains the nodes
	 * that have same types in the then and else parts
	* @return void
	*/
	public void semCheck() {
		// The intersection of the types of the thenExpression and
		// elseExpression should be found first
		SOM_TreeNodeList intersectList = new SOM_TreeNodeList();
		ListEnumeration thenEnum = thenExpression.getTypeList().elements();
		SOM_TreeNodeList elseTypeList = elseExpression.getTypeList();
	
		// Now find the intersection
		SOM_TreeNode currThenNode, currElseNode;
		SOM_TypeDeclaration currThenType, currElseType;		
		ListEnumeration elseEnum;

		while (thenEnum.hasMoreElements()) {
			currThenNode = (SOM_TreeNode) thenEnum.nextElement();
			currThenType = currThenNode.getResultType();
			System.out.println("If expression, type of then part: " +
							   currThenType.toString());
			
			// Now iterate through the elseExpression's types
			// I know this is a dirty way - but I'm too tired to optimize this
			// rightnow
			elseEnum = elseTypeList.elements();
			while (elseEnum.hasMoreElements()) {
				currElseNode = (SOM_TreeNode) elseEnum.nextElement();
				currElseType = currElseNode.getResultType();
				System.out.println("If expression, type of else part: " +
								   currElseType.toString());
				if (currThenType.isPredicateSubtypeOf(currElseType))
					intersectList.addElement(currElseNode);
                else if (currElseType.isPredicateSubtypeOf(currThenType))
					intersectList.addElement(currThenNode);
			} // end of iterating through the elseExpression's types
		}
		setTypeList(intersectList);
		
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		
		SOM_TreeNodeList intersectList = getTypeList();

		// Now the intersection must yield exactly one type else throw
		// exception 
		if (intersectList.countElements() != 1)
			throw  new SYM_UnresolvableException
				("Cannot resolve type for expression in IfExpression");

		// Else the only node is the resulting type
		SOM_TreeNode onlyNode = (SOM_TreeNode)intersectList.first();
		SOM_TypeDeclaration onlyType = onlyNode.getResultType();

		thenExpression.resolveFinalType(onlyType);
		elseExpression.resolveFinalType(onlyType);
		setResolvedType(onlyType);
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		thenExpression.resolveFinalType(typeIn);
		elseExpression.resolveFinalType(typeIn);
		// fortunately isDistinguishable is based on equality of types :)
		// else I would have a problem as to which finaltype to select, the
		// thenExpression's or the elseExpression's!
		setResolvedType(typeIn);
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		SOM_DeclarationList declList = new SOM_DeclarationList();
		SOM_DeclarationList thenList = thenExpression.getNames();
		SOM_DeclarationList elseList = elseExpression.getNames();
		ListEnumeration thenEnum = thenList.elements();
		ListEnumeration elseEnum;
		SOM_Declaration thenDecl, elseDecl;
		while (thenEnum.hasMoreElements()) {
			thenDecl = (SOM_Declaration)thenEnum.nextElement();
			elseEnum = elseList.elements();
			while (elseEnum.hasMoreElements()) {
				elseDecl = (SOM_Declaration)elseEnum.nextElement();
				if (!SOM_Declaration.isDistinguishable(thenDecl, elseDecl))
					declList.addElement(thenDecl);
			}
		}
		return declList;
	}

	/** dummy function to instantiate the method in SOM_Factor
	 */
	public SOM_Label getLabel() {
			return (SOM_Label)null;
	}

	/** dummy function to instantiate the method in SOM_Factor
	 */
	public SOM_Expression getLabelExpression() {
			return (SOM_Expression)null;
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */		
	public Expression M__Value(Factory romFactory,BaseObject parent, 
							   Universe root){
		Item ifItem = root.labelLookup("if");
		FunctionReference ifFunc = new FunctionReference(parent,ifItem); 
		Expression condExpr = condExpression.M__Value(romFactory,ifFunc,root);
		Expression thenExpr = thenExpression.M__Value(romFactory,ifFunc,root);
		Expression elseExpr = elseExpression.M__Value(romFactory,ifFunc,root);
		ifFunc.addParameter(condExpr);
		ifFunc.addParameter(thenExpr);
		ifFunc.addParameter(elseExpr);
		return (Expression)ifFunc;
	 }
	
	/** The publishSOM routine
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		pw.print("if ");
		condExpression.publishSOM(pw);
		pw.print(" then ");
		thenExpression.publishSOM(pw);
		pw.print(" else ");
		elseExpression.publishSOM(pw);
		pw.print (" endif ");
	}

		/** The publishPVS routine
		 * @return void
		 * To be implemented as needed
		 */
		public void publishPVS(PrintWriter pw) {
				pw.print("if ");
				condExpression.publishPVS(pw);
				pw.print(" then ");
				thenExpression.publishPVS(pw);
				pw.print(" else ");
				elseExpression.publishPVS(pw);
				pw.print (" endif ");
		}
}
