//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_IntegerLiteral.java,v 1.9 2001/03/12 23:54:39 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_IntegerLiteral.java,v $
//  Revision 1.9  2001/03/12 23:54:39  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.8  2000/12/03 23:55:26  cvs
//  Added the M__Value method to the Literal classes.Previously they
//  were returning null.The M__Type method for all the Declaration
//  statement classes has been completed 
//  --srinivas
//
//  Revision 1.7  2000/11/18 00:18:27  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.6  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas
//
//  Revision 1.5  2000/08/01 22:48:51  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.4  2000/02/10 00:37:22  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.3  2000/02/08 18:42:33  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.2  1999/11/13 20:01:27  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:39  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:30  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.2  1999/07/13 14:27:24  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.1  1999/07/07 21:22:19  rkamath
//  Added some literal support
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import java.lang.Integer;
import ROM.*;

/** This represents the <B>IntegerLiteral</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_IntegerLiteral
extends SOM_NumericLiteral 
implements SOM_Terminal {

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_IntegerLiteral() {
		super();
	}

	/** The second constructor
	 * @param literalIn The String representation of the integer literal
	 * @return Not applicable
	 */
	public SOM_IntegerLiteral(String literalIn) {
		super(literalIn);
	}

	/** To get the value associated with the float literal
	 * @return The float value associated with the float literal
	 */
	public Object getValue() {
		return new Integer(literal);
	}

	/** The publish Rosetta routine for this terminal
	 * @return void
	 */
	public void publishRosettaTerminal(PrintWriter pw) {
		pw.print(literal + " ");
	}
	
	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		String str = getLiteral();
		Integer value = new Integer(str);
		DataLiteral literal = new DataLiteral(parent,value);
		return (Expression)literal;
	}

	/** The publishSOM routine for this terminal
	 * @return void
	 */
	public void publishSOM  (PrintWriter pw) {
		pw.print(literal + " ");
	}

		/** The publishPVS routine for this terminal
		 * @return void
		 */
		public void publishPVS  (PrintWriter pw) {
				pw.print(literal + " ");
		}
		
}
