//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_LetDeclarationStatementListing.java,v 1.1 2000/02/10 03:56:29 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_LetDeclarationStatementListing.java,v $
//  Revision 1.1  2000/02/10 03:56:29  cvs
//  Modified the grammar for incorporating records, tuples and "let" expressions. Also populated the object model with these above constructs, and completed the pretty printing for them.
//  - krishna
//
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

import java.io.PrintWriter;

/** This represents the <B>LetDeclarationStatementListing</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */

public class SOM_LetDeclarationStatementListing
extends SOM_BaseClass {

	private SOM_LetDeclarationStatementList letDeclStmtList = new SOM_LetDeclarationStatementList();

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_LetDeclarationStatementListing() {
		super();
	}

	/** To add a LetDeclaration
	 * @param letDeclStmtIn The SOM_LetDeclarationStatement to be added
	 * @return void
	 */
	public void addToLetDeclarationStatementList(SOM_LetDeclarationStatement letDeclStmtIn) {
		letDeclStmtList.addElement(letDeclStmtIn);
	}

	/** to retreive the LetDeclarationStatement list
	 * @return SOM_LetDeclarationStatementList
	 */
	public SOM_LetDeclarationStatementList getLetDeclarationStatementList() {
		return letDeclStmtList;
	}

	/** To retreive an enumeration of the LetDeclarationStatements
	 * @return A ListEnumeration corresponding to the LetDeclarationStatements
	 * stored in the LetDeclarationStatementList
	 */
	public ListEnumeration letDeclarationStatements() {
		return letDeclStmtList.elements();
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
	    ListEnumeration letDeclStmtEnum = letDeclarationStatements();
	    SOM_LetDeclarationStatement letDeclStmt;

			letDeclStmt = (SOM_LetDeclarationStatement)letDeclStmtEnum.nextElement();
	    letDeclStmt.publishSOM(pw);
	    while (letDeclStmtEnum.hasMoreElements()) {
					pw.print(", ");
					letDeclStmt=(SOM_LetDeclarationStatement)letDeclStmtEnum.nextElement();
					letDeclStmt.publishSOM(pw);
	    }
	}
}
