//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//
//  $Id: SOM_MinFunction.java,v 1.2 2000/12/07 01:23:19 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_MinFunction.java,v $
//  Revision 1.2  2000/12/07 01:23:19  cvs
//  Adding the M__Value to all the Quantifiers. Making minor changes to the other classes
//  --srinivas
//
//  Revision 1.1  2000/05/18 18:03:45  cvs
//  Due to /projects directory failing and the latest version available is old, I
//  am committing the files again.
//  -- Cindy
//
//  Revision 1.1  2000/05/05 19:28:33  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//
//=============================================================================
package SOM;

import util.List;
import util.Link;
import util.ListEnumeration;

import java.io.*;
import java.lang.NullPointerException;
import java.lang.String;

import ROM.*;
/** This represents the <B>min function</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 */
public class SOM_MinFunction
extends SOM_KeyWord
implements SOM_Quantifier {

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_MinFunction() {
		super();
	}

	/** The publishRosetta routine for this terminal
	 * @return void
	 */
	public void publishRosettaTerminal(PrintWriter pw) {
		pw.print("min ");
	}

	/** The publishSOM routine for this terminal
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		pw.print("min ");
	}

	/** Providing a body for the method in the interface
	 * @returns int corresponding to SOM.SOM_Quantifier.EXISTS
	 */

	public int getQuantifier() {
		return MIN;
	}
	
	/** The function to get the Value of the Quantified
	 * Expression
	 * @param romFactory factory through which the items are built
	 * @param root The Universe to search for predefined functions
	 * @param expr The Expression object 
	 * @param interfaceItem ListEnumeration
	 * @return ROM.Expression
	 */
	public Expression M__Value(Factory romFactory,Universe root,
							   BaseObject parent,SOM_Expression expr,
							   ListEnumeration paramList){
		MinFunction min = new MinFunction(parent);
		ItemList itemList = new ItemList();
		SOM_InterfaceItem interItem;
		if (paramList != null) {
			while (paramList.hasMoreElements()) {
				interItem = (SOM_InterfaceItem)paramList.nextElement();
				itemList.addItem
					(interItem.buildROM(romFactory, root, min));
			}
		}
		min.setItemList(itemList);
		Expression expression = expr.M__Value(romFactory,min,root);
		min.setExpression(expression);
		return min;
	}

}
