//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_Name.java,v 1.16 2001/03/12 23:54:39 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_Name.java,v $
//  Revision 1.16  2001/03/12 23:54:39  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.15  2000/12/05 03:05:03  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.14  2000/11/18 00:18:28  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.13  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas
//
//  Revision 1.12  2000/08/09 23:37:55  cvs
//  added publishPVS routines for these classes - Makarand
//
//  Revision 1.11  2000/06/22 17:22:45  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.10  2000/04/06 19:13:23  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.9  2000/03/23 00:22:48  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.8  2000/02/10 00:37:27  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.7  2000/02/08 18:42:38  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.6  1999/11/17 18:31:55  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.5  1999/11/15 22:59:16  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.4  1999/11/15 03:16:29  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.3  1999/11/13 20:01:28  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.2  1999/11/11 00:04:24  cvs
//  Fresh changes as part of the streamlining process with new
//  object model and grammar
//
//  Revision 1.1  1999/10/27 20:43:44  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:51:03  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.8  1999/10/20 20:40:14  rkamath
//  Commit before the archive moves over to Kansas
//
//  Revision 1.7  1999/10/01 15:23:33  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.6  1999/08/01 20:54:34  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.5  1999/07/28 01:59:44  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.4  1999/07/16 21:50:33  rkamath
//  Changed everyexpression to point to the type declaration instead of having
//  a private variable which would indicate the resulting type
//
//  Revision 1.3  1999/07/13 14:27:34  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.2  1999/07/06 21:01:44  rkamath
//  Having added the type expression syntax the corresponding OM...etc
//  Also now, only subclasses of RuntimeException are being thrown
//
//  Revision 1.1  1999/06/27 17:33:37  rkamath
//  Added some of the object model for the Expression grammar and a
//  few changes to the other files..
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;

//import java.lang.Error;
//import java.util.NoSuchElementException;
import java.util.StringTokenizer;

/** This represents the <B>Name</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public abstract class SOM_Name
extends SOM_AbstractExpression 
implements SOM_Factor{

	// This is saved purely for information purposes - it is also used in
	// getNames function
	protected SOM_DeclarationList declList;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_Name() {
		super();
	}

	/** Since most SOM_Name objects (except SOM_Identifier) have an internal
	 * name object embedded, it is better to have a generic function which will
	 * be overridden by all children
	 * This will save some code in the symbol table lookUp
	 * @return The embedded SOM_Name object
	 */
	public abstract SOM_Name getName();

	/** To set the declarations. This also populates the TypeList
	 * setTypeList to point to the possible types 
	 * I wish we had multiple inheritance - I wouldn't have to duplicate
	 *  methods here and in SOM_Literal
	 * @param declsIn The SOM_DeclarationList to be set
	 * @return void
	 */
	public void setDeclarations(SOM_DeclarationList declsIn) {
		if (declList == null) {
			declList = declsIn;
			
			SOM_TreeNodeList toReturn = new SOM_TreeNodeList();
			ListEnumeration enum = declsIn.elements();
			SOM_TreeNode node;
			SOM_Declaration currDecl;
			while (enum.hasMoreElements()) {
				currDecl = (SOM_Declaration)enum.nextElement();
				node = new SOM_TreeNode(currDecl.getType());
			
				toReturn.addElement(node);
			}
			setTypeList(toReturn);
		}
	}

	/** To get the declList for the name
	 * @return declList The SOM_DeclarationList to be returned
	 */
	public SOM_DeclarationList getDeclarations() {
		return declList;
	}

	/** 
	 * @param declsIn The SOM_DeclarationList to be set
	 * @return void
	 */
	/*
	public void setTypeDeclarations(SOM_DeclarationList declsIn) {
		declList = declsIn;

		SOM_TreeNodeList toReturn = new SOM_TreeNodeList();
		ListEnumeration enum = declsIn.elements();
		SOM_TreeNode node;
		SOM_Declaration currDecl;
		while (enum.hasMoreElements()) {
			currDecl = (SOM_Declaration)enum.nextElement();
			if (currDecl instanceof SOM_VariableDeclaration) {
				node = new SOM_TreeNode(currDecl.getType());
				toReturn.addElement(node);
			} else {
				node = new SOM_TreeNode((SOM_TypeDeclaration)currDecl);
				toReturn.addElement(node);
			}
		}
		setTypeList(toReturn);
	}
	*/

	/** Overrides the semCheck in AbstractExpression
	 * @exception SOM_MethodNotSupportedException if called because it doesnot
	 * make sense to call semCheck(). On SOM_Name objects setDeclarations()
	 * should be invoked
	 */
	public void semCheck() {
		throw new SOM_MethodNotSupportedException
			(getClassName() + " does not support semCheck()");
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		ListEnumeration declEnum = declList.elements();
		SOM_TypeDeclaration resolvedType = getResolvedType();
		SOM_Declaration currDecl;
	  //System.out.println("In name.getNames() of: " + toDotSeparatedString());
		while (declEnum.hasMoreElements()) {
		  currDecl = (SOM_Declaration)declEnum.nextElement();
			if(!SOM_TypeDeclaration.isDistinguishable
				 (currDecl.getType(),resolvedType))
				return new SOM_DeclarationList(currDecl);
			// Note that we can afford to return since any ambiguous multiple
			// type would have been eliminated by the overload resolution algo
			// hence there can be only one decl corresponding to the
			// resolved type 
		}
		// shouldn't reach here since the resolved type HAS to be one from the
		// declList. But if we do there is something horribly wrong. So throw
		// an Error
		throw  new RuntimeException("Something horribly wrong with" +
									" locating proper declaration for" +
									" resolved type");
	}

	/** To get the string representation for the given SOM_Name - it does not
	 * consider the association elements in the formalId....
	 * @return String representation of the SOM_Name
	 */
	public String toDotSeparatedString() {
		if (this instanceof SOM_Identifier)
			return toString();
		else { // SelectedName, FormalName, ReferencedName,
			String innerString =
				getName().toDotSeparatedString();
			if (this instanceof SOM_SelectedName)
				return innerString + "." + 
					((SOM_SelectedName)this).getIdentifier().toString();
			else return innerString;
		}
	}

	/** To get a SOM_Name for a given dot separated identifier list
	 * @param stringIn A dot separated string
	 * @return A SOM_Name object for stringIn
	 */
	public static SOM_Name forString(String stringIn) {
		//System.out.println("String in: " + stringIn);
		StringTokenizer tokenizer = new StringTokenizer(stringIn, ".");
		return forTokenizer(tokenizer);
	}

	/** To get a SOM_Name given a string in a tokenizer
	 * @param tokenizer The StrinTokenizer from which SOM_Name is to 
	 * be generated
	 * @return The SOM_Name corresponding to the string in tokenizier
	 */
	private static SOM_Name forTokenizer(StringTokenizer tokenizer) {

    // Assume The tokenizer has atleast one token
    SOM_Identifier id = new SOM_Identifier(tokenizer.nextToken());
		
		SOM_Name prevName = id;
		SOM_SelectedName currName;
		while (tokenizer.hasMoreTokens()) {
			currName = new SOM_SelectedName();
			currName.setName(prevName);
			currName.setIdentifier(new SOM_Identifier(tokenizer.nextToken()));
			prevName = currName;
		}
		return prevName;
	}
	
	/**
     * publishSOM routine for this class
     */
    public abstract void publishSOM(PrintWriter pw);

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
		
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}
		/**
     * publishPVS routine for this class
     */
    public abstract void publishPVS(PrintWriter pw);
	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public abstract Expression M__Value(Factory romFactory,BaseObject
										parent,Universe root);
}


