//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_QuantifiedExpression.java,v 1.14 2000/12/07 01:23:19 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_QuantifiedExpression.java,v $
//  Revision 1.14  2000/12/07 01:23:19  cvs
//  Adding the M__Value to all the Quantifiers. Making minor changes to the other classes
//  --srinivas
//
//  Revision 1.13  2000/11/18 00:18:28  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.12  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.11  2000/08/18 15:17:41  cvs
//  modified publishPVS routines to support quantified expression - changes for soccer, makarand
//
//  Revision 1.10  2000/08/01 22:48:55  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.9  2000/06/22 17:22:46  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.8  2000/05/18 18:03:47  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I 
//  am committing the files again.
//  -- Cindy
//
//  Revision 1.8  2000/05/05 19:28:34  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.7  2000/04/06 19:13:24  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.6  2000/02/10 00:37:29  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.5 2000/02/08 18:42:41 cvs Completed populating the
//  object model and publishing the object model.  Added
//  'publishSOM(PrintWriter)' method to all necessary classes (with
//  the help of Makarand and Srini). Removed some unnecessary classes
//  (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model.  
//  -- Cindy.
//
//  Revision 1.4  1999/11/15 22:59:16  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.3  1999/11/15 03:16:29  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.2  1999/11/05 23:43:11  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:46  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:51:20  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.3  1999/07/14 23:24:10  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.2  1999/06/30 22:20:12  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//  Revision 1.1  1999/06/30 02:19:09  rkamath
//  Added a lot more files dealing with the expression OM.
//  Also modified some previous classes
//  Now the ROOT in the hierarchy is the SOM_Unifier interface.
//
//
//=============================================================================
package SOM;

import SYM.SYM_TypeInconsistencyException;
import java.io.PrintWriter;
import util.ListEnumeration;
import ROM.*;
/** This represents the <B>QuantifiedExpression</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_QuantifiedExpression
extends SOM_AbstractExpression
implements SOM_Factor {

	private SOM_Quantifier quantifier;
	private SOM_AbstractExpression expression;
	
	private SOM_InterfaceItemList interfaceList = 
		new SOM_InterfaceItemList(); 

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_QuantifiedExpression() {
		super();
	}

	/** To set the quantifier
	 * @param quantifierIn The SOM_Quantifier to be set
	 * @return void
	 */
	public void setQuantifier(SOM_Quantifier quantifierIn) {
		quantifier = quantifierIn;
	}

	/** To add to the interface list
	 * @param itemIn The SOM_InterfaceItem to be added
	 * @return void
	 */
	public void addInterfaceItem(SOM_InterfaceItem itemIn)
	{ 
		interfaceList.addElement(itemIn);
	}

	/** To set the expression
	 * @param expressionIn the SOM_Expression to be set
	 * @return void
	 */
	public void setExpression(SOM_Expression expressionIn) {
		expression = expressionIn;
	}


	/**To get the expression
	 * @param void
	 * @return SOM_Expression
	 */
	public SOM_Expression getExpression(){
		if (expression instanceof SOM_Expression){
			return (SOM_Expression)expression;
		}else
			return null;
	}

	/**To get the interface List
	 * @param void
	 * @return SOM_InterfaceItemList
	 */
	public SOM_InterfaceItemList getInterfaceItemList(){
		return interfaceList;
	}


	/** Overrides the semCheck from AbstractExpression
	 * @return void
	 */
	public void semCheck() {
		if (quantifier instanceof SOM_Sel) {
			SOM_InterfaceItem item = (SOM_InterfaceItem)interfaceList.
				getFirst();
			setTypeList(item.getIdentifier().getTypeList());
		} else
			setTypeList(expression.getTypeList());
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		if (quantifier instanceof SOM_Sel) {
			SOM_InterfaceItem item = (SOM_InterfaceItem)interfaceList.
				getFirst();
			setResolvedType(item.getType());
		} else {
			expression.resolveFinalType();
			setResolvedType(expression.getResolvedType());
		}
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		if (quantifier instanceof SOM_Sel) {
			SOM_InterfaceItem item = (SOM_InterfaceItem)interfaceList.
				getFirst();
			if (SOM_TypeDeclaration.isDistinguishable(typeIn, item.getType()))
				throw new SYM_TypeInconsistencyException
					("Type returned by sel is not as expected");
			else
				setResolvedType(item.getType());
		} else {
			expression.resolveFinalType(typeIn);
			setResolvedType(expression.getResolvedType());
		}
	}

	/** To retrieve the listEnumeration of the interfaceItemList
	 * @return The ListEnumeration corresponding to the interfaceItemList
	 */
	public ListEnumeration interfaces() {
		return interfaceList.elements();
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		return expression.getNames();
	}

	//changed by srinivas on Dec 6th, 2000 so that it returns an ROM.Expression
	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		return quantifier.M__Value(romFactory,root,parent,getExpression(),interfaces());
	}
	

	public void publishSOM(PrintWriter pw) {
		ListEnumeration interfaceItemEnum = interfaces();
		SOM_InterfaceItem interfaceItem;

		quantifier.publishSOM(pw);
		pw.print("(");
		((SOM_InterfaceItem)interfaceItemEnum.nextElement()).
			publishSOM(pw);
		while (interfaceItemEnum.hasMoreElements()) {
			pw.print("; ");
			interfaceItem = (SOM_InterfaceItem) 
				interfaceItemEnum.nextElement(); 
			interfaceItem.publishSOM(pw);
		}
		pw.print("| ");
        expression.publishSOM(pw);
		pw.print(") ");
    }

	// dummy methods
	public SOM_Label getLabel() {
		return null;
	}

	public SOM_Expression getLabelExpression() {
		return null;
	}

		/** The publishPVS routine
		 * @return void
		 * To be implemented as needed
		 */
		public void publishPVS(PrintWriter pw) {
				ListEnumeration interfaceItemEnum = interfaces();
		SOM_InterfaceItem interfaceItem;
		quantifier.publishSOM(pw);
		pw.print("(");
		((SOM_InterfaceItem)interfaceItemEnum.nextElement()).
			publishPVS(pw,null,null,null);
		while (interfaceItemEnum.hasMoreElements()) {
			pw.print(", ");
			interfaceItem = (SOM_InterfaceItem) 
				interfaceItemEnum.nextElement(); 
			interfaceItem.publishPVS(pw,null,null,null);
		}
		pw.print("): ");
        expression.publishPVS(pw);
				//pw.print(") ");
		}
}
