//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//
//  $Id: SOM_RecordListing.java,v 1.6 2000/12/05 03:05:03 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_RecordListing.java,v $
//  Revision 1.6  2000/12/05 03:05:03  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.5  2000/11/18 00:18:28  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.4  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the terms
//  (SOM_LabelledExpression) and it returns a ROM Expression.  
//  --srinivas
//
//  Revision 1.3  2000/08/01 22:48:55  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.2  2000/07/24 23:41:02  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//  Revision 1.1  2000/06/22 17:22:47  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;

/** This represents the <B>RecordListing</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 * @see SOM.SOM_BaseClass
 */
public class SOM_RecordListing
extends SOM_AbstractExpression
implements SOM_Factor{

	private SOM_List itemList = new SOM_List();

	// declaration is true if declaration of record
	//                false if use of record
	private boolean declaration = false;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_RecordListing() {
		super();
	}

	/** To set declaration to true
	 * @return void
	 */
	public void isDeclaration() {
		declaration = true;
	}
		
	/** To get the declaration
	 * @return boolean, value of declaration
	 */
	public boolean getDeclaration() {
		return declaration;
	}

	/** To add to the itemList
	 * @param item The SOM_RecordItem to be added
	 * @return void
	 */
	public void addToItemList(SOM_Identifier id, SOM_Expression expr)
	{
		SOM_RecordItem item = new SOM_RecordItem(id, expr);
		itemList.addElement(item);
	}

	/** to retrieve the item list
	 * @return SOM_List 
	 */
	public SOM_List getItemList() {
		return itemList;
	}

	/** 
	 * @param declsIn The SOM_DeclarationList to be set
	 * @return void
	 */
	public void setTypeDeclaration(SOM_TypeDeclaration currDecl) {

		SOM_TreeNodeList toReturn = new SOM_TreeNodeList();
		SOM_TreeNode node;

		node = new SOM_TreeNode(currDecl);
		toReturn.addElement(node);
		setTypeList(toReturn);
	}

	/** To do the semcheck. Overrides that in SOM_AbstractExpression
	 * Note that it assumes a semChecked expression - consistent with
	 * the semChecks elsewhere..
	 * @return void
	 */
	public void semCheck() {
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		setResolvedType(((SOM_TreeNode)getTypeList().first()).
						getResultType()); 
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		SOM_TypeDeclaration manifestType = 
			((SOM_TreeNode)getTypeList().first()).getResultType();
		if (SOM_TypeDeclaration.resolveFinalType(manifestType, 
												 typeIn))
			setResolvedType(manifestType);
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		return null;
	}

	/** To retrieve an enumeration of the record items
	 * @return A ListEnumeration corresponding to the record items stored
	 * in the itemList
	 */
	public ListEnumeration items() {
		return itemList.elements();
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		RecordValue recordVal = new RecordValue(parent);
		return (Expression)recordVal;
	}

	/** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
		ListEnumeration itemEnum = items();
		SOM_RecordItem item;
		
		pw.print("record");
		pw.print("[");
		item = (SOM_RecordItem)itemEnum.nextElement();
		item.getIdentifier().publishSOM(pw);
		if (declaration)
			pw.print(":: ");
		else 
			pw.print("= ");
		item.getExpression().publishSOM(pw);
		while (itemEnum.hasMoreElements()) {
			pw.print("| ");
			item = (SOM_RecordItem)itemEnum.nextElement(); 
			item.getIdentifier().publishSOM(pw);
			if (declaration)
				pw.print(":: ");
			else
				pw.print("= ");
			item.getExpression().publishSOM(pw);
		}
		pw.print("]");
    }

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
		
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}

		/** The publishPVS routine for this class
     * @return void
		 * To be implemented later
     */
    public void publishPVS(PrintWriter pw) {
				publishSOM(pw);
    }
}

