//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_SelectedName.java,v 1.15 2001/03/12 23:54:39 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_SelectedName.java,v $
//  Revision 1.15  2001/03/12 23:54:39  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.14  2001/03/06 02:29:37  cvs
//  Implementing the M__Value method for the SelectedName and removing
//  errors in the other Meta functions 
//
//  --srinivas
//
//  Revision 1.13  2000/11/18 00:18:28  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.12  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas
//
//  Revision 1.11  2000/08/01 22:48:55  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.10  2000/07/11 00:11:13  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are 
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.9  2000/06/22 17:22:48  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.8  2000/04/12 18:41:05  cvs
//  Implemented some type checking for "use" clause.
//  -- Cindy
//
//  Revision 1.7  2000/04/06 19:13:24  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.6  2000/02/17 01:27:05  cvs
//  Implemented the facet expression grammar to incorporate facet composition.
//  Also updated the "use" clause with the above changes.
//  - krishna.
//
//  Revision 1.5  2000/02/10 00:37:31  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:42  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.3  1999/11/17 18:31:55  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.2  1999/11/13 20:01:28  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/11/11 00:04:25  cvs
//  Fresh changes as part of the streamlining process with new
//  object model and grammar
//
//
//=============================================================================
package SOM;

import SYM.SYM_UnresolvableException;
import ROM.*;
import util.ListEnumeration;
import java.io.*;

/** This represents the <B>SelectedName</B> object
 * <selected_name> ::== <name> <DOT> <identifier>
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_SelectedName
extends SOM_Name {

	/** The name in this formalName */
	private SOM_Name name;
	/** The selectedidentifier for this formalName */
	private SOM_Identifier selectedId = null;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_SelectedName() {
		super();
	}

	/** To set the name
	 * @param nameIn The SOM_Name to be set
	 * @return void
	 */
	public void setName(SOM_Name nameIn) {
		name = nameIn;
	}

	/** To get the name - overrides the getName in SOM_Name
	 * @return SOM_Name that is the name in this formalName
	 */
	public SOM_Name getName() {
		return name;
	}

	/** To set the identifier
	 * @param idIn The SOM_Identifier to be set as the selected identifier
	 * @return void
	 */
	public void setIdentifier(SOM_Identifier idIn) {
		selectedId = idIn;
	}

	/** To get the identifier
	 * @return The SOM_Identifier corresponding to the selected identifier
	 */
	public SOM_Identifier getIdentifier() {
		return selectedId;
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		// The set should have resulted in a single type.
		SOM_TreeNodeList typeList = getTypeList();
		if (typeList.countElements() != 1)
			throw  new SYM_UnresolvableException
				("Cannot resolve final type for expression (selected name)");
		
		// else the final type is the sole element of the typeList.
		SOM_TreeNode onlyNode = (SOM_TreeNode)typeList.first();

		// The type resolving is made on the whole name - Cindy May 23, 2000
		// selectedId.resolveFinalType(onlyNode.getResultType());
		setResolvedType(onlyNode.getResultType());
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		// The type resolving is made on the whole name - Cindy May23, 2000
		// selectedId.resolveFinalType(typeIn);

		// typeIn should be one of the types in typeList
		ListEnumeration typeEnum = getTypeList().elements();
		SOM_TreeNode currNode;
		SOM_TypeDeclaration currType;

		//System.out.println("Fresh TCC Generation");
		while (typeEnum.hasMoreElements()) {
			//System.out.println("New Cycle");
			try {
				currNode = (SOM_TreeNode)typeEnum.nextElement();
				currType = currNode.getResultType();
				if (SOM_TypeDeclaration.resolveFinalType(currType, typeIn)) {
					setResolvedType(currType);
					//System.out.println("End Fresh TCC Generation");
					return;
				}
			} catch (SYM_UnresolvableException except) {
				// means that some assocation failed to resolve
				// go to the next currType in typeEnum
				//System.out.println("Flush previous cycle");
				//System.out.println("New Cycle");
			}
		}
		//System.out.println("End Fresh TCC Generation");
		
		// typeIn was not found in the overload resolved types.
		throw  new SYM_UnresolvableException
			("Cannot resolve final type for expression in Selected Name");
	}

	/** toString routine
	 * @return the string representing the selected name
	 */
	public String toString() {
		String toReturn = new String(name.toString());
		if (selectedId != null) 
			toReturn = toReturn.concat("." + selectedId.toString());
		return toReturn;
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		ItemReference itemRef = null;
		Item item = null;                    ;
		if (name instanceof SOM_Identifier){
			String str = ((SOM_Identifier)name).toString();
			Label label =new Label(str);
			item = parent.labelLookup(label);
			if (item == null){
				try{
					FileInputStream in = new
						FileInputStream(str+"_ROM.ser");
					ObjectInputStream objin = new ObjectInputStream(in);
					item = (LabelledItem)objin.readObject();
					objin.close();
				}catch(Exception e){
					System.out.println(e);
				}
			}
		}else{
			Expression itemExpr =
				name.M__Value(romFactory,parent,root);
			if (itemExpr instanceof ItemReference){
				item = ((ItemReference)itemExpr).getItemReferred();
			}
		}
		if (item != null){
			String selectedStr = selectedId.toString();
			Label selectedLab = new Label(selectedStr);
			Item selectedItem = item.getValue().labelLookup(selectedLab);
			itemRef = new ItemReference(parent,selectedItem);
		}
		return itemRef;
	}
	
	/** publishSOM routine 
     * @return void
     */
    public void publishSOM(PrintWriter pw){
		name.publishSOM(pw);
		if(selectedId != null) {
			pw.print(".");
			selectedId.publishSOM(pw);
		}
	}

	/** publishPVS routine 
     * @return void
	 * to be implemented as needed
     */
    public void publishPVS(PrintWriter pw){
		publishSOM(pw);	
	}
}












