//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//
//  $Id: SOM_SequenceDeclaration.java,v 1.1 2000/07/24 23:41:03 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_SequenceDeclaration.java,v $
//  Revision 1.1  2000/07/24 23:41:03  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

/** This represents the <B>SequenceDeclaration</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 */
public class SOM_SequenceDeclaration
extends SOM_Declaration {

	protected SOM_Identifier sequenceName;
	protected SOM_ExpressionList listElements = new SOM_ExpressionList();

	/** The primary constructor
	 * @param sequenceNameIn The SOM_Identifier corresponding to the name
	 * of the sequence
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * type of the sequence
	 * @return Not applicable
	 */
	public SOM_SequenceDeclaration(SOM_Identifier sequenceNameIn,
							  SOM_TypeDeclaration myType) {
		super();
		sequenceName = sequenceNameIn;
		setKind(SOM_Declaration.SEQUENCE);
		setType(myType);
	}

	/** To add to the list of elements in sequence 
	 * @param exprIn The SOM_Expression to be added
	 * @return void
	 */
	public void addToElements(SOM_Expression exprIn) {
		listElements.add(exprIn);
	}

	/** To set the list of elements in sequence
	 * @param listIn The SOM_ExpressionList to be sequence
	 * @return void
	 */
	public void setListElements(SOM_ExpressionList listIn) {
		listElements = listIn;
	}

	/** To get the list of elements in sequence
	 * @return The SOM_List containing the elements
	 */
	public SOM_ExpressionList getListElements() {
		return listElements;
	}

	/** To get the number of elements in sequence
	 * @return int, number of elements
	 */
	public int getNumberElements() {
		return listElements.size();
	}

	/** To get the ListEnumeration of the elements
	 * @return ListEnumeration containing the elements
	 */
	public ListEnumeration elements() {
		return listElements.elements();
	}

	/** To get the string representation of this object. Overrides the
	 * method in SOM_Declaration
	 * @return String representation of this object - name of the sequence
	 */
	public String toString() {
		return sequenceName.toString();
	}

	/** To implement my own version of isDistinguishable for sequence
	 * declarations
	 * @param declOne The first SOM_SequenceDeclaration
	 * @param declTwo The next SOM_SequenceDeclaration to be distinguished
	 * against declOne
	 * @return true if they are distinguishable, false otherwise
	 */
	public static boolean isDistinguishable(SOM_SequenceDeclaration declOne,
											SOM_SequenceDeclaration declTwo) { 
		SOM_SequenceType typeOne = (SOM_SequenceType)declOne.getType();
		SOM_SequenceType typeTwo = (SOM_SequenceType)declTwo.getType();
		if (!declOne.toString().equalsIgnoreCase(declTwo.toString()))
			return true;
		else
			return(SOM_SequenceType.isDistinguishable(typeOne, typeTwo));
	}

	/** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
	 * @param declLHS The SequenceDecl corresponding to the lhs object
	 * @param declRHS The SequenceDecl corresponding to the rhs object
	 * @return true, if the rhs is assignable to the lhs, false otherwise
	 */
	public static boolean
		isLeftCompatibleWithRight(SOM_SequenceDeclaration declLHS,
								  SOM_SequenceDeclaration declRHS) { 
		SOM_SequenceType typeLHS = (SOM_SequenceType)declLHS.getType();
		SOM_SequenceType typeRHS = (SOM_SequenceType)declRHS.getType();

		return SOM_SequenceType.isLeftTypeCompatibleWithRightType
			(typeLHS, typeRHS);
	}
}
