//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//
//  $Id: SOM_SetDeclaration.java,v 1.2 2000/07/24 23:41:03 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_SetDeclaration.java,v $
//  Revision 1.2  2000/07/24 23:41:03  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//  Revision 1.1  2000/07/11 00:11:14  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are keywords. Typechecking for set is done. Added SetType
//  and FunctionType. 
//  False and true are predefined as boolean.
//  -- Cindy
//
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

/** This represents the <B>SetDeclaration</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 */
public class SOM_SetDeclaration
extends SOM_Declaration {

	protected SOM_Identifier setName;
	protected SOM_ExpressionList listElements = new SOM_ExpressionList();

	/** The primary constructor
	 * @param setNameIn The SOM_Identifier corresponding to the name
	 * of the set
	 * @param myType The SOM_TypeDeclaration corresponding to the
	 * type of the set
	 * @return Not applicable
	 */
	public SOM_SetDeclaration(SOM_Identifier setNameIn,
							  SOM_TypeDeclaration myType) {
		super();
		setName = setNameIn;
		setKind(SOM_Declaration.SET);
		setType(myType);
	}

	/** To add to the list of elements in set 
	 * @param exprIn The SOM_Expression to be added
	 * @return void
	 */
	public void addToElements(SOM_Expression exprIn) {
		listElements.add(exprIn);
	}

	/** To set the list of elements in set
	 * @param listIn The SOM_ExpressionList to be set
	 * @return void
	 */
	public void setListElements(SOM_ExpressionList listIn) {
		listElements = listIn;
	}

	/** To get the list of elements in set
	 * @return The SOM_List containing the elements
	 */
	public SOM_ExpressionList getListElements() {
		return listElements;
	}

	/** To get the number of elements in set
	 * @return int, number of elements
	 */
	public int getNumberElements() {
		return listElements.size();
	}

	/** To get the ListEnumeration of the elements
	 * @return ListEnumeration containing the elements
	 */
	public ListEnumeration elements() {
		return listElements.elements();
	}

	/** To get the string representation of this object. Overrides the
	 * method in SOM_Declaration
	 * @return String representation of this object - name of the set
	 */
	public String toString() {
		return setName.toString();
	}

	/** To implement my own version of isDistinguishable for set
	 * declarations
	 * @param declOne The first SOM_SetDeclaration
	 * @param declTwo The next SOM_SetDeclaration to be distinguished
	 * against declOne
	 * @return true if they are distinguishable, false otherwise
	 */
	public static boolean isDistinguishable(SOM_SetDeclaration declOne,
											SOM_SetDeclaration declTwo) { 
		SOM_SetType typeOne = (SOM_SetType)declOne.getType();
		SOM_SetType typeTwo = (SOM_SetType)declTwo.getType();
		if (!declOne.toString().equalsIgnoreCase(declTwo.toString()))
			return true;
		else
			return(SOM_SetType.isDistinguishable(typeOne, typeTwo));
	}

	/** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
	 * @param declLHS The SetDecl corresponding to the lhs object
	 * @param declRHS The SetDecl corresponding to the rhs object
	 * @return true, if the rhs is assignable to the lhs, false otherwise
	 */
	public static boolean
		isLeftCompatibleWithRight(SOM_SetDeclaration declLHS,
								  SOM_SetDeclaration declRHS) { 
		SOM_SetType typeLHS = (SOM_SetType)declLHS.getType();
		SOM_SetType typeRHS = (SOM_SetType)declRHS.getType();

		return SOM_SetType.isLeftTypeCompatibleWithRightType(typeLHS, typeRHS);
	}
}
