//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//           Cindy ckong@ittc.ukans.edu
//
//  $Id: SOM_SetListing.java,v 1.11 2000/11/18 00:18:28 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_SetListing.java,v $
//  Revision 1.11  2000/11/18 00:18:28  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.10  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.9  2000/09/23 20:58:00  cvs
//  The format of the setListing is changed from {1,2,3} format to {1++2++3} format The class SOM_ExpressionListing is no longer used so removing the class SOM_ExpressionListing.java.
//  --srinivas
//
//  Revision 1.8  2000/08/01 22:48:56  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.7  2000/07/11 00:11:15  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and sequence are
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.6  2000/06/22 17:22:48  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.5  2000/04/06 19:13:24  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.4  2000/03/23 00:22:49  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.3  2000/02/10 00:37:31  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  2000/02/08 18:42:42  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.1  2000/02/02 20:16:52  cvs
//  I am committing all the new class files I have created for the new
//  operators and new rules Krishna has added to the grammar. 
//  -- Cindy
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import ROM.*;
/** This represents the <B>SetListing</B> object
 * @author <I>Roshan Kamath and Cindy Kong</I> 
 * (rkamath@ececs.uc.edu, ckong@ittc.ukans.edu)
 * @see SOM.SOM_BaseClass
 */
public class SOM_SetListing
extends SOM_AbstractExpression
implements SOM_Factor{

	private SOM_Expression expression = null;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_SetListing() {
		super();
	}

	/** To set the expression
	 * @param exprIn The SOM_Expression to be set
	 * @return void
	 */
	public void setExpression(SOM_Expression  exprIn){
		expression = exprIn;
	}

	/** To get the expression corresponding to the expression
	 * @return The SOM_Expression
	 */
	public SOM_Expression getExpression(){
		return expression;
	}

	/** 
	 * @param currDecl The SOM_TypeDeclaration to be set
	 * @return void
	 */
	public void setTypeDeclaration(SOM_TypeDeclaration currDecl) {

		SOM_TreeNodeList toReturn = new SOM_TreeNodeList();
		SOM_TreeNode node;

		node = new SOM_TreeNode((SOM_TypeDeclaration)currDecl);
		toReturn.addElement(node);
		setTypeList(toReturn);
	}

	/** To do the semcheck. Overrides that in SOM_AbstractExpression
	 * Note that it assumes a semChecked expression - consistent with
	 * the semChecks elsewhere..
	 * @return void
	 */
	public void semCheck() {
		//setTypeList(expression.getTypeList());
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		//expression.resolveFinalType();
		//setResolvedType(expression.getResolvedType());
		setResolvedType(((SOM_TreeNode)getTypeList().first()).
						getResultType()); 
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		//expression.resolveFinalType(typeIn);
		//setResolvedType(expression.getResolvedType());
		setResolvedType(((SOM_TreeNode)getTypeList().first()).
						getResultType()); 
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		//return expression.getNames();
		return null;
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory, BaseObject parent,
							   Universe root){
		SetValue setVal = new SetValue(parent);
		return setVal;
	}


	/** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
		pw.print(" { ");
		if (expression != null) {
			expression.publishSOM(pw);
		}
		pw.print(" } ");
    }

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
	
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}

		/** The publishPVS routine for this class
     * @return void
		 * To be implemented as needed
     */
    public void publishPVS(PrintWriter pw) {
				publishSOM(pw);
		}
		
}

