//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Srinivas akki@ittc.ukans.edu
//
//  $Id: SOM_TickOperator.java,v 1.6 2000/08/01 22:48:58 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_TickOperator.java,v $
//  Revision 1.6  2000/08/01 22:48:58  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.5  2000/07/11 00:11:16  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and sequence are
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.4  2000/06/22 17:22:49  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.3  2000/04/09 01:50:04  cvs
//   The identifier had a space printed in the publishSOM which was not required for the tick and the at operators so changing the classes and the grammar had the tick and single quote interchanged so modifing the grammar also
//  --srinivas
//
//  Revision 1.2  2000/04/08 04:05:38  cvs
//  The tick and the single quote symbols were interchanged so modified the grammar and the classes to correct that .
//  --srinivas
//
//  Revision 1.1  2000/04/05 07:25:46  cvs
//  The at and tick operators have been added .There were some changes to the predefined unary function name .The grammar has been changed and some classes ahve been added like SOM_AtOperator SOM_TickOperator and SOM_AtTickExpression.
//  --srinivas
//
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import SOM.*;
import util.*;
import SYM.*;

/** This represents the <B>TickOperator</B> object
 * @author <I>Srinivas</I> (akki@ittc.ukans.edu)
 */
public class SOM_TickOperator
extends SOM_Operator implements SOM_Terminal {

	protected SOM_AbstractExpression operand;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_TickOperator() {
		super();
	}

	/** To set the operand
	 * @param operandIn The SOM_AbstractExpression to be set as the operand
	 * @return void
	 */
	public void setOperand(SOM_AbstractExpression operandIn) {
		operand = operandIn;
	}

	/** Overrides the semCheck in SOM_AbstractExpression
	 * @return void
	 */
	public void semCheck() {
		System.out.println("Doing semCheck() in " + getClassName());
    SOM_TreeNodeList typeListNow = 
			SYM_StaticFunctions.buildTree(getDeclarations(),new SOM_ArgumentList(operand.getTypeList()));
		if (typeListNow.isEmpty())
			throw new SYM_UnresolvableException("No declaration with the correct argument types found for " + getClassName());
		setTypeList(typeListNow);
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world.
	 * @return void
	 * @exception SYM.SYM_UnresolvableException if a final type cannot be
	 * resolved
	 */
	public void resolveFinalType() {
		// The final overload resolution should have resulted in a single 
		// type.
		SOM_TreeNodeList typeList = getTypeList();
		if (typeList.countElements() != 1) 
			throw  new SYM_UnresolvableException
				("Cannot resolve final type for expression (TickOperator)");
		
		// else the final type is the sole element of the typeList.
		SOM_TreeNode onlyNode = (SOM_TreeNode)typeList.first();
		setResolvedType(onlyNode.getResultType());

	  ListEnumeration childEnum = onlyNode.children();
		operand.resolveFinalType(((SOM_TreeNode)childEnum.nextElement()).getResultType());
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 * @exception SYM.SYM_UnresolvableException if a final type cannot be
	 * resolved
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		// typeIn should be one of the types in typeList

		ListEnumeration typeEnum = getTypeList().elements();
		SOM_TreeNode currNode;
		SOM_TypeDeclaration currType;

		//System.out.println("Fresh TCC Generation");

		while (typeEnum.hasMoreElements()) {
			//System.out.println("New Cycle");

			currNode = (SOM_TreeNode)typeEnum.nextElement();
			currType = currNode.getResultType();

			try {
				if (SOM_TypeDeclaration.resolveFinalType(currType, typeIn)) {
					setResolvedType(currType);
					ListEnumeration childEnum = currNode.children();
					operand.resolveFinalType
						(((SOM_TreeNode)childEnum.nextElement()).
						 getResultType());
					//System.out.println("End Fresh TCC Generation");
					return;
				}
			} catch (SYM_UnresolvableException  except) {
				// means that some resolution failed
				// go to the next type enum
				//System.out.println("Flush previous cycle");
				//System.out.println("New Cycle");
			}
		}

		//System.out.println("End Fresh TCC Generation");

		// typeIn was not found in the overload resovled types.
		throw  new SYM_UnresolvableException
			("Cannot resolve final type for expression in TickOperator");
	}



 /** The publishRosetta routine for this terminal
     * @return void
     */
    public void publishRosettaTerminal(PrintWriter pw) {
	pw.print("`");
    }

	/** To get the string representation of this object
	 * @return String representation of this operator
	 */
	public String toString() {
			return new String("`");
	}

    /** publishSOM routine needed to print the object module
     * @return void
     */
    public void publishSOM(PrintWriter pw){
				pw.print("'");
    }

		/** The publishPVS routine for this class
		 * @return void */
		public void publishPVS(PrintWriter pw) {
				pw.print("_");
		} 
}

