//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy  ckong@ittc.ukans.edu
//
//  $Id: SOM_TupleListing.java,v 1.4 2000/11/18 00:18:29 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_TupleListing.java,v $
//  Revision 1.4  2000/11/18 00:18:29  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.3  2000/10/30 22:13:25  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.2  2000/08/01 22:48:58  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.1  2000/06/22 17:22:50  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;

/** This represents the <B>TupleListing</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 * @see SOM.SOM_BaseClass
 */
public class SOM_TupleListing
extends SOM_AbstractExpression
implements SOM_Factor{

	private SOM_ExpressionList exprList = new SOM_ExpressionList();

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_TupleListing() {
		super();
	}

	/** To add to the expressionList
	 * @param item The SOM_Expression to be added
	 * @return void
	 */
	public void addToExpressionList(SOM_Expression item) {
		exprList.addElement(item);
	}

	/** to retrieve the expression list
	 * @return SOM_ExpressionList 
	 */
	public SOM_ExpressionList getExpressionList() {
		return exprList;
	}

	/** 
	 * @param declsIn The SOM_DeclarationList to be set
	 * @return void
	 */
	public void setTypeDeclaration(SOM_TypeDeclaration currDecl) {

		SOM_TreeNodeList toReturn = new SOM_TreeNodeList();
		SOM_TreeNode node;

		node = new SOM_TreeNode(currDecl);
		toReturn.addElement(node);
		setTypeList(toReturn);
	}


	/** To do the semcheck. Overrides that in SOM_AbstractExpression
	 * Note that it assumes a semChecked expression - consistent with
	 * the semChecks elsewhere..
	 * @return void
	 */
	public void semCheck() {
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		setResolvedType(((SOM_TreeNode)getTypeList().first()).
						getResultType()); 
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		SOM_TypeDeclaration manifestType = 
			((SOM_TreeNode)getTypeList().first()).getResultType();
		if (SOM_TypeDeclaration.resolveFinalType(manifestType, typeIn))
			setResolvedType(manifestType);
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		return null;
	}

	/** To retrieve an enumeration of the expressions
	 * @return A ListEnumeration corresponding to the expressions stored
	 * in the expressionList
	 */
	public ListEnumeration expressions() {
		return exprList.elements();
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		TupleValue tupleVal = new TupleValue(parent);
		return tupleVal;
	}

	/** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
		ListEnumeration expressionEnum = expressions();
		SOM_Expression expression;
		
		pw.print("tuple");
		pw.print("[");
		expression = (SOM_Expression)expressionEnum.nextElement();
		expression.publishSOM(pw);
		while (expressionEnum.hasMoreElements()) {
			pw.print(" | ");
			expression = (SOM_Expression) expressionEnum.nextElement(); 
			expression.publishSOM(pw);
		}
		pw.print("]");
    }

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
		
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}

		/** The publishPVS routine for this class
		 * @return void
     * To be implemented as needed */
		public void publishPVS(PrintWriter pw) {
        publishSOM(pw);
		}
}

