//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//           Cindy ckong@ittc.ukans.edu
//  $Id: SOM_UnnamedFunction.java,v 1.11 2000/11/18 00:18:29 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_UnnamedFunction.java,v $
//  Revision 1.11  2000/11/18 00:18:29  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.10  2000/10/30 22:13:25  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.9  2000/08/01 22:48:58  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.8  2000/06/22 17:22:50  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.7  2000/04/28 19:34:08  cvs
//  Added some more of typechecking. Added parsing of the $ operator
//  and constant literal. 
//  -- Cindy
//
//  Revision 1.6  2000/04/06 19:13:26  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.5  2000/03/23 00:22:51  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.4  2000/03/07 23:01:02  cvs
//  Modified the class due the change in syntax of function , facet
//  dedclarations and the unnamed functions 
//  --srinivas
//
//  Revision 1.3  2000/02/10 00:37:35  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  2000/02/08 18:42:45  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.1  2000/02/02 20:16:48  cvs
//  I am committing all the new class files I have created for the new
//  operators and new rules Krishna has added to the grammar. 
//  -- Cindy
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import ROM.*;
import util.ListEnumeration;

/** This represents the <B>UnnamedFunction</B> object
 * @author <I>Roshan Kamath and Cindy Kong</I> 
 * (rkamath@ececs.uc.edu, ckong@ittc.ukans.edu)
 */
public class SOM_UnnamedFunction
extends SOM_AbstractExpression
implements SOM_Factor {
    private SOM_InterfaceListing interList;
	private SOM_Expression exprType;
	private SOM_Expression exprValue = null;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_UnnamedFunction() {
		super();
	}

	/** To set the interfaceListing
	 * @param interfaceIn The SOM_InterfaceListing object to be set
	 * @return void
	 */
	public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
		interList = interfaceIn;
	}

	/** To get the interfaceListing
	 * @return The SOM_InterfaceListing object for this unnamed function
	 */
	public SOM_InterfaceListing getInterfaceListing() {
		return interList;
	}

	/** To set the expression representing the type
	 * @param exprIn The SOM_Expression object to be set
	 * @return void
	 */
	public void setExpressionType(SOM_Expression exprIn) {
		exprType = exprIn;
	}

	/** To get the expression representing the type
	 * @return The SOM_Expression object giving the type
	 */
	public SOM_Expression getExpressionType() {
		return exprType;
	}

	/** To set the expression representing the value
	 * @param exprIn The SOM_Expression object to be set
	 * @return void
	 */
	public void setExpressionValue(SOM_Expression exprIn) {
		exprValue = exprIn;
	}

	/** To get the expression representing the value
	 * @return The SOM_Expression object giving the type
	 */
	public SOM_Expression getExpressionValue() {
		return exprValue;
	}

	/** To do the semcheck. Overrides that in SOM_AbstractExpression
	 * Note that it assumes a semChecked expression - consistent with
	 * the semChecks elsewhere..
	 * @return void
	 */
	public void semCheck() {
	    setTypeList(exprType.getTypeList());
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
	    //exprType.resolveFinalType();
	    setResolvedType(exprType.getResolvedType());
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
	    //exprType.resolveFinalType(typeIn);
	    setResolvedType(exprType.getResolvedType());
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
	    //	return expression.getNames();
	    return null;
	}

	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		return null;
	}

    /** publishSOM routine to print the object module
     * @parm PrintWriter
     * @return void - mak
     */
    public void publishSOM(PrintWriter pw){
		pw.print("<* ");
		interList.publishSOM(pw);
		pw.print (" ::  ");
		exprType.publishSOM(pw);
		if (exprValue != null){
		    pw.print(" IS ");
			exprValue.publishSOM(pw);
		}
		pw.print("*> ");
    }

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
		
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}
	
		/** publishPVS routine to print the object module
     * @parm PrintWriter
     * @return void - makarand
		 * To be implemented later as needed
     */
    public void publishPVS(PrintWriter pw){
				publishSOM(pw);
		}
}
