//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna krishna@ittc.ukans.edu
//
//  $Id: SOM_UseClause.java,v 1.14 2001/03/26 19:04:58 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_UseClause.java,v $
//  Revision 1.14  2001/03/26 19:04:58  cvs
//  Completed the implementation for the change in export clauses. Also
//  added code in SOM_PackageDeclaration to take care of use clauses (case
//  was missing for when these clauses are found in declaration part of
//  packages).
//  -- Cindy
//
//  Revision 1.13  2000/12/05 03:05:04  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.12  2000/11/06 18:12:02  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.11  2000/10/22 18:42:02  cvs
//  Meta functions are added to the SOM_InterfaceItem class and the
//  meta function M__Domain is changed so that it returns a string and
//  not SOM_FacetExpression 
//  --srinivas
//
//  Revision 1.10  2000/07/14 23:01:25  cvs
//  Made changes to the meta functions and added some meta functions
//  to the identifier object. 
//  --srinivas
//
//  Revision 1.9  2000/06/28 22:13:19  cvs
//  added publishPVS routine for component retrieval - makarand
//
//  Revision 1.8  2000/06/22 17:22:50  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.7  2000/05/18 18:03:50  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I am committing the files again.
//  -- Cindy
//
//  Revision 1.7  2000/05/05 19:28:37  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.6  2000/04/11 03:17:54  cvs
//  The dummy M__Label and M__Value functions have been removed .
//  --srinivas
//
//  Revision 1.5 2000/04/08 02:21:58 cvs The Meta Functions have been
//  implemented and these are the files that had to be modified.Addded
//  some accessor methods to classes like SOM_DeclarationStatement,
//  SOM_Expression, SOM_RecordTupleDeclarationStatement.The Meta
//  functions for facet terms and the term items are implemented
//  --srinivas
//
//  Revision 1.4 2000/03/23 00:22:51 cvs Huge commit! Added all the
//  essential accessor methods for the following classes 
//  - Krishna and Makarand
//
//  Revision 1.3  2000/03/03 18:38:30  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.2  2000/02/17 01:27:06  cvs
//  Implemented the facet expression grammar to incorporate facet composition.
//  Also updated the "use" clause with the above changes.
//  - krishna.
//
//  Revision 1.1 2000/02/13 01:15:57 cvs 
//
//  Completed the implementation for package declarations, and the
//  "use" clause.  Modified the grammar to incorporate these
//  changes. Also populated the object model with these new objects
//  and completed the pretty-printing for these new constructs.  Also
//  modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.  
//  - Krishna.
//
//
//
//=============================================================================
package SOM;

import util.Link;
import util.ListEnumeration;

import java.io.PrintWriter;
import java.io.*;

import ROM.*;

/** This represents the <B>UseClause</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 * @see SOM.SOM_Declaration
 */
public class SOM_UseClause
extends SOM_Declaration
implements SOM_DesignUnit, SOM_FacetDeclarativeItem, 
		   SOM_ComponentDeclarativeItem, SOM_PackageDeclarativeItem {

	private SOM_SelectedNameList selectedNameList = new SOM_SelectedNameList();
	private	SOM_InterfaceListing interfaceList;

	private transient SOM_DeclarationList domainList;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_UseClause() {
		super();
	}

	/** To add a SelectedName
	 * @param selectedName The SOM_SelectedName to be added
	 * @return void
	 */
	public void addToSelectedNameList(SOM_SelectedName selectedNameIn) {
		selectedNameList.addElement(selectedNameIn);
	}

	/** to retrieve the SelectedName list
	 * @return SOM_SelectedNameList
	 */
	public SOM_SelectedNameList getSelectedNameList() {
		return selectedNameList;
	}

	/** To retrieve an enumeration of the selectedNames
	 * @return A ListEnumeration corresponding to the selectedNames stored
	 * in the selectedNameList
	 */
	public ListEnumeration selectedNames() {
		return selectedNameList.elements();
	}

	/** To get the exports
	 * @return The ListEnumeration corresponding to the export clauses
	 */
	public ListEnumeration exports() {
		return new ListEnumeration(null);  
		// consistent with SOM_ExportBlock.exports()
	}

	/** added by krishna on Mar 5, 2000
	 * To get the desired object by traversal of SOM
	 * just used to instantiate the method in SOM_DesignUnit()
	 * no real use of this method here. :o)
	 */
	public ListEnumeration M__Terms() {
		return new ListEnumeration(null);
	}

	/** added by srinivas on Mar 27, 2000
	 * To get the desired object by traversal of SOM
	 * just used to instantiate the method in SOM_DesignUnit()
	 * no real use of this method here. :o)
	 */
	public ListEnumeration M__Params() {
		return new ListEnumeration(null);
	}

	/** added by srinivas on Mar 27, 2000
	 * To get the desired object by traversal of SOM
	 * just used to instantiate the method in SOM_DesignUnit()
	 * no real use of this method here. :o)
	 */
	public ListEnumeration M__Pvars() {
		return new ListEnumeration(null);
	}

	/** added by srinivas on Mar 29, 2000
	 * To get the desired object by traversal of SOM
	 * to be implemented later
	 */
	public ListEnumeration M__Visible() {
		//System.out.println("Compo decl entered");
		//return getFacetDefinition().getFacetBody().items();
		return new ListEnumeration(null);
	}

	/** added by srinivas on Mar 29, 2000
	 * To get the desired object by traversal of SOM
	 * to be implemented later
	 */
	public ListEnumeration M__Types() {
		//System.out.println("Compo decl entered");
		//return getFacetDefinition().getFacetBody().items();
		return new ListEnumeration(null);
	}

	/** added by srinivas on Mar 29, 2000
	 * To get the desired object by traversal of SOM
	 * to be implemented later
	 */
	public ListEnumeration M__Items() {
		//System.out.println("Compo decl entered");
		//return getFacetDefinition().getFacetBody().items();
		return new ListEnumeration(null);
	}
	
	
	/** added by srinivas on Mar 29, 2000
	 * To get the desired object by traversal of SOM
	 * to be implemented later
	 */
	public String M__Domain() {
		//System.out.println("Compo decl entered");
		//return getFacetDefinition().getFacetBody().items();
		return null;
	}
	
	/** The buildROM method for this high-level item
	 * @param romFactory, Factory object used to create objects in ROM
	 * @param root, Universe object 
	 * @param parentIn, BaseObject parent of item
	 * @return LabelledItem, corresponding to this package
	 */
	public LabelledItem buildROM(Factory romFactory, Universe root, 
								 BaseObject parentIn){
		return null;
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
    ListEnumeration selectedNameEnum = selectedNames();
		SOM_SelectedName selectedName;
	    
		pw.print("USE ");
		selectedName = (SOM_SelectedName)selectedNameEnum.nextElement();
		selectedName.publishSOM(pw);
		while (selectedNameEnum.hasMoreElements()) {
		    pw.print(", ");
		    selectedName=(SOM_SelectedName)selectedNameEnum.nextElement();
				selectedName.publishSOM(pw);
		}
		pw.println(";");
	}
	
	/** The publishPVS routine for this class
	 * @return void
	 * added by makarand 6/27/00 for SOCCER
	 */
	public void publishPVS(PrintWriter pw){
		ListEnumeration selectedNameEnum = selectedNames();
		SOM_SelectedName selectedName;
		
		pw.print("IMPORTING ");
		selectedName = (SOM_SelectedName)selectedNameEnum.nextElement();
		selectedName.publishSOM(pw);
		while (selectedNameEnum.hasMoreElements()) {
			pw.print(", ");
			selectedName=(SOM_SelectedName)selectedNameEnum.nextElement();
			selectedName.publishSOM(pw);
		}
	}
	
	// the next three productions are not actually needed per se
	// they are added only to instantiate the methods for the abstract
	// class SOM_PackageDeclarativeItem - krishna on Feb 12, 2000
	/** To set the SOM_InterfaceListing object for this designUnit
	 * @param interfaceIn The SOM_InterfaceListing object to be set
	 * @return void
	 */
	public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
	}

	/** To get the interfaceListing
	 * @return The SOM_InterfaceListing object for this designUnit
	 */
	public SOM_InterfaceListing getInterfaceListing() {
		return null;
	}

	/** To set the domainList - implements the function from
	 * SOM_PackageDeclarativeItem
	 * @param domainListIn The SOM_DeclarationList object to be set
	 * @return void
	 */
	public void setDomainList(SOM_DeclarationList domainListIn) {
	}

	/** To get the domainList - implements the function from
	 * SOM_PackageDeclarativeItem
	 * @return The SOM_DeclarationList object for this designUnit
	 */
	public SOM_DeclarationList getDomainList() {
		return null;
	}

}

