//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//  authors: Makarand makarand@ittc.ukans.edu
//
//  $Id: SOM_VerificationBlock.java,v 1.3 2001/03/12 23:54:40 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_VerificationBlock.java,v $
//  Revision 1.3  2001/03/12 23:54:40  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.2  2000/11/06 16:58:00  cvs
//  Added the Meta functions to the component item and some accessor
//  methods to Verification block and the definition block classes 
//  --srinivas
//
//  Revision 1.1  2000/02/17 01:20:13  cvs
//  Completed the implementation for component declarations (with the help of
//  makarand); this involved grammar changes, populating the object model and
//  also the pretty-printing.
//  - krishna
//
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;

/** This represents the <B>VerificationBlock</B> object
 * @author <I>Makarand Patil</I> (makarand@ittc.ukans.edu)
 */
public class SOM_VerificationBlock
extends SOM_Declaration {
    
    // The individual internal facets
    private SOM_FacetDeclaration verificationFacet;
    private SOM_FacetExpression facetExpr = null;
	
    /** The primary constructor
     * @param compTypeIn The typeDeclaration corresponding to the
     * "component type"
     * @return Not applicable
     */
    public SOM_VerificationBlock() {
				super();
    }

    /** To set the definition facet
     * @param facetIn The SOM_FacetDeclaration to be set as the
     * definition facet
     * @return void
     */
    public void setFacetDefinition(SOM_FacetDeclaration facetDeclIn){
				verificationFacet = facetDeclIn;
    }	
    
	/** To get the definition facet
     * @param void
	 * @return SOM_FacetDeclaration
     */
    public SOM_FacetDeclaration getFacetDefinition(){
		return verificationFacet;
    }	
    
    /** To set the facet expression
     * @param facetExprIn The SOM_FacetExpression to be set as the
     * definition facet
     * @return void
     */
    public void setFacetExpression(SOM_FacetExpression facetExprIn){
		facetExpr = facetExprIn;
    }    

	/** To get the facet expression
	 * @return SOM_FacetExpression
	 */
	public SOM_FacetExpression getFacetExpression() {
		return facetExpr;
	}

    // added by Makarand
    /** publishSOM routine to print object model
     *  @param PrintWriter
     *  @return void
     */
    public void publishSOM(PrintWriter pw) {
				pw.print("VERIFICATION ");
				pw.println("IS");
				if(facetExpr != null){
						facetExpr.publishSOM(pw);
				}
				else{
						verificationFacet.getFacetDefinition().publishSOM(pw);
						pw.print("VERIFICATION ");
				}
				pw.println(";");
    }
}
