//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SYM_SymboltableEntryList.java,v 1.4 2000/05/18 18:04:46 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SYM_SymboltableEntryList.java,v $
//  Revision 1.4  2000/05/18 18:04:46  cvs
//  Due to /projects directory failing and the latest version available is old, I
//  am committing the file again.
//  -- Cindy
//
//  Revision 1.4  2000/05/05 19:29:42  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  - Cindy
//
//  Revision 1.3  2000/02/10 00:37:38  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  1999/11/16 04:05:24  cvs
//  Added support for export clause and visiblity
//
//  Revision 1.1  1999/10/27 20:43:56  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:52:21  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.3  1999/07/13 19:33:36  rkamath
//  Intermediate commit for records..
//
//  Revision 1.2  1999/07/10 21:34:25  rkamath
//  Added some more code - a generic commit. Some rudimentary form of declaration
//  checking exists. Now to populate the symbol table and code it in the .jj file
//
//  Revision 1.1  1999/07/08 23:25:27  rkamath
//  A fresh repository going towards an elementary type checker - and mebbe more
//
//
//=============================================================================
package SYM;

import util.List;
import util.ListEnumeration;

import java.io.Serializable;

/** This represents the <B>SYM_SymboltableEntryList</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SYM_SymboltableEntryList
extends List
implements Serializable {

	/** This variable, if true, indicates that all the entries in this list
	 * are visible to the external world
	 */
	private transient boolean isVisible = false;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SYM_SymboltableEntryList() {
		super();
	}

	/** The second constructor
	 * @param entry The first SYM_SymboltableEntry to be added
	 * @return void
	 */
	public SYM_SymboltableEntryList(SYM_SymboltableEntry entry) {
		super(entry);
	}

	/** To prepend a SYM_SymboltableEntry to the list
	 * @param entry The SYM_SymboltableEntry object to be prepended
	 * @return void
	 */
	public void prependElement(SYM_SymboltableEntry entry) {
		super.prependElement(entry);
	}

	/** To append a SYM_SymboltableEntry to the list
	 * @param entry The SYM_SymboltableEntry object to be appended
	 * @return void
	 */
	public void addElement(SYM_SymboltableEntry entry) {
		super.addElement(entry);
	}

	/** To set the isVisible value
	 * @param visibility The boolean value to be set for isVisible
	 * @return void
	 */
	public void setIsVisible(boolean visibility) {
		isVisible = visibility;
	}

	/** To get the isVisible value
	 * @return The boolean value of isVisible
	 */
	public boolean getIsVisible() {
		return isVisible;
	}

	/** To flush all the entries that have a given scope.
	 * Note: Only looks at the head of the list and stops the moment an
	 * entry of a different scope is found. This is justified by the
	 * usual way in which entries are inserted at the head of each list
	 * @param scope An integer corresponding to the scope for which
	 * entries are to be expunged
	 * @return void	 
	 */
	public void flushEntriesOfScope(int scope) {
		SYM_SymboltableEntry entry;
		ListEnumeration enum = elements();
		while (enum.hasMoreElements()) {
			entry = (SYM_SymboltableEntry)enum.nextElement();
			// This remove will be hazardous....it might make the enum
			// invalid.. 
			// My implementation does not have this problem
			// currently... 
			//if (entry.getScope() == scope) removeElement(entry);
			// Using ListIterator to implement ListEnumeration
			if (entry.getScope() == scope) enum.remove();
			else return; // ensures faster return!
		}
	}
}
