//=============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
// By using or copying this Software, Licensee agrees to abide by the
// intellectual property laws, and all other applicable laws of the
// U.S., and the terms of this license.
//
// authors: Roshan rkamath@ececs.uc.edu (SLDL grammar)
// 
// $Id: SldlParser.jj.h,v 1.41 2001/04/04 22:22:05 cvs Exp $
//
// Revision History:
// -----------------
// $Log: SldlParser.jj.h,v $
// Revision 1.41  2001/04/04 22:22:05  cvs
// Changed syntax for record formation from record[x = 4] to record[x is
// 4].
// -- Cindy
//
// Revision 1.40  2001/03/20 23:59:57  cvs
// Added the "design" mode for parameters. Changed the keyword "type" to
// "subtype". Removed all the calls to addToParseOrderList(xxx). Changed
// the semantics for export clauses.
// -- Cindy
//
// Revision 1.39  2001/03/14 23:27:11  cvs
// Added some typechecking in quantified expressions.
// -- Cindy
//
// Revision 1.38  2001/03/12 23:53:03  cvs
// Changed the order of precedence of the "=" operator. It now has lowest
// precedence of all operators except for the "==" operator. Added some
// type checking for facet expressions and as a result, domain names have
// to be legal facet names.
// -- Cindy
//
// Revision 1.37  2000/10/04 03:08:21  cvs
// Changes made so that only the package declaration and use clause
// can implement the design unit.The package declarative block can
// have a component declaration. 
// --srinivas
//
// Revision 1.36  2000/09/23 20:51:06  cvs
// Changed the syntax for the setListing from {1,2,3} to {1++2++3}.
// --srinivas
//
// Revision 1.35  2000/07/24 23:34:59  cvs
// Removed an error in grammar from last changes.
// Added some more type checking for sequence and record.
// Added code to catch exceptions being thrown because of incompleteness.
// -- Cindy
//
// Revision 1.34  2000/07/11 00:08:47  cvs
// Change in the grammar: type, set, bunch, array, record, tuple and
// sequence are keywords. Typechecking for set is done. Added SetType
// and FunctionType. 
// False and true are predefined as boolean.
// -- Cindy
//
// Revision 1.33  2000/06/22 17:33:22  cvs
// I modified the parser to remove error in parsing of records and tuples.
// -- Cindy
//
// Revision 1.32  2000/05/26 21:37:58  cvs
// Modified the Makefile,sldl and deserialze scripts to support both
// linux and solaris. Made the Sldl Class public as needed for
// DVTG. The last change was lost during the file system crash -
// makarand 
//
// Revision 1.31  2000/05/18 17:13:36  cvs
// Due to /projects directory failing and the latest version available
// is two versions old, I am committing the file again. 
// -- Cindy
//
// Revision 1.32  2000/05/05 19:26:19  cvs
// Added:
// 	- use of record and tuple keywords for record and tuple formers
// 	- new syntax for sel, forall, exists, min, max and ran functions
// 	- all expressions between "begin" and "end" are labelled.
// - Cindy
//
// Revision 1.31  2000/05/04 20:57:46  cvs
// Modified the declarationStatement() method to populate
// declarationStatementZ() 
// into the SOM. It was accidently commented out..
// - Krishna
//
// Revision 1.30  2000/04/28 19:36:23  cvs
// Added some more typechecking. Also added parsing of $ operator and
// constant literal. 
// -- Cindy
//
// Revision 1.29  2000/04/12 18:42:13  cvs
// Implemented some type checking for "use" clause.
// -- Cindy
//
// Revision 1.28  2000/04/09 01:50:05  cvs
//  The identifier had a space printed in the publishSOM which was not
//  required for the tick and the at operators so changing the classes
//  and the grammar had the tick and single quote interchanged so
//  modifing the grammar also 
// --srinivas
//
// Revision 1.27  2000/04/08 04:05:39  cvs
// The tick and the single quote symbols were interchanged so modified
// the grammar and the classes to correct that . 
// --srinivas
//
// Revision 1.26  2000/04/06 19:15:17  cvs
// Type checking is done for declaration of variables (simple types), for
// declaration of new types and for use of basic operators.
//  -- Cindy
//
// Revision 1.25  2000/04/05 07:25:47  cvs
// The at and tick operators have been added .There were some changes
// to the predefined unary function name .The grammar has been changed
// and some classes ahve been added like SOM_AtOperator
// SOM_TickOperator and SOM_AtTickExpression. 
// --srinivas
//
// Revision 1.24  2000/03/23 00:35:27  cvs
// Removed the production rule 'assignmentStatement()' from the grammar since
// it is redundant, and no longer needed.
// - Krishna
//
// Revision 1.23  2000/03/22 03:21:25  cvs
// Modified the grammar to remove thew production exptermtail and the
// classes are also modified 
// --srinivas
//
// Revision 1.22  2000/03/21 22:35:37  cvs
// Modified the grammar by adding the new production expTermTail.This
// is done so that all the productions are consistent.made changes to
// termexpressiontail for the same reason.Modified the classes for
// these productions 
// --srinivas
//
// Revision 1.21  2000/03/21 03:13:15  cvs
// Made changes to the grammar since the if expression production
// should have an else part compulsory.The expterm had to be modified 
// --srinivas
//
// Revision 1.20  2000/03/17 23:43:45  cvs
// :Made modifications to the production relational term expression
// --srinivas
//
//
//  ----------------------------------------------------------------------
//
// Revision 1.19  2000/03/10 22:36:37  cvs
// Modified the grammar to include the if expression
// --srinivas
//
// Revision 1.17  2000/03/08 01:07:20  cvs
// modified the grammar and the class file due to change in the let
// expression.the test cases have also been changed accordingly 
// --srinivas
//
// Revision 1.16  2000/03/07 22:36:17  akki
// The modified grammar since there were changes in the syntax of
// function declaration and facet declaration.Removed the productions
// constant declaration statement and function declaration statement 
// -- srinivas
//
// Revision 1.15  2000/02/29 21:11:23  cvs
// manually added the log messages corresponding to the previous commit
// -- srinivas
//
// Revision 1.14  2000/02/29 18:09:22  cvs
// Modified the grammar for incorporating facet declaration statement. made
// changes to the declaration statement.
//  -- srinivas
//
// Revision 1.13  2000/02/17 18:25:30  cvs
// Set the debug_parser flag correctly. If you want to be able to use
// the debug option, please do 'make debug' when you compile your
// files. 
// -- Cindy
//
// Revision 1.12  2000/02/17 01:23:47  cvs
// Modified the grammar for component declarations. Also added facet expression
// grammar, and updated facet & package declarations to incorporate the facet
// expressions. Also updated the grammar with changes made to the "use" clause.
// - krishna.
//
// Revision 1.11  2000/02/13 01:15:58  cvs
// Completed the implementation for package declarations, and the "use" clause.
// Modified the grammar to incorporate these changes. Also populated the object
// model with these new objects and completed the pretty-printing for these new
// constructs.
// Also modified the parser to now recognize the keyword "type" as a type.
// These above changes have been compiled and tested successfully.
// - Krishna.
//
// Revision 1.10  2000/02/10 03:56:26  cvs
// Modified the grammar for incorporating records, tuples and "let"
// expressions. Also populated the object model with these above
// constructs, and completed the pretty printing for them. 
// - krishna
//
// Revision 1.9  2000/02/10 00:37:39  cvs
// Changed Copyright information to University of Kansas
// - by makarand
//
// Revision 1.8  2000/02/08 18:42:46  cvs
// Completed populating the object model and publishing the object model.
// Added 'publishSOM(PrintWriter)' method to all necessary classes
// (with the help of Makarand and Srini). Removed some unnecessary
// classes (due to changes brought to the grammar). Renamed
// SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
// SldlParser.jj.h to populate the object model. 
// -- Cindy.
//
// Revision 1.7  2000/02/02 20:28:43  cvs
// Committing the SldlParser.jj.h file with the object model being populated.
// No tests have been done so far on the object model. The commit is
// to allow Makarand, Srini and I to work on the publish
// part. Regression tests passed. 
// -- Cindy
//
// Revision 1.6  2000/01/31 19:48:38  cvs
// Modified the grammar by putting the call to
// 'predefUnaryFunctionName' in 'factor' instead of 'name'. 
// -- Cindy
//
// Revision 1.5  2000/01/29 22:37:02  cvs
// parser update by krishna -
// completed some major changes to the expression grammar to
// incorporate all the basic and composite types (including arrays,
// sets and bunches), and their operations as given in the usage
// guide. now the parser also recognizes the various forms of function 
// declarations and function usage. The code compiles, but has not been
// tested comprehensively as yet.
//
// Revision 1.4  2000/01/21 05:51:34  cvs
// parser update by krishna -
// modified the expression grammar to handle the various types and new
// operators as defined in the usage guide. added a couple of new
// productions in the expression grammar as well as modified the
// earlier ones to take care of operator precedence. 
//
// Revision 1.3  1999/11/16 04:05:24  cvs
// Added support for export clause and visiblity
//
// Revision 1.2  1999/11/15 22:59:17  cvs
// Changes to incorporate unfolding of overloading
//
// Revision 1.1  1999/11/15 03:16:32  cvs
// Many changes - especially with unfolding of overloading. Everything
// compiles now. Some features are yet to be coded...but it is now again
// a rudimentary parser (as it was two weeks ago :)
//
// Revision 1.3  1999/11/13 20:01:25  cvs
// Changed to the new OM and grammar. Some more features are to be added
// and many bugs to be corrected. This is a commit for record and it compiles
//
// Revision 1.2  1999/11/11 19:44:44  cvs
// Commited the new grammar for safety. It does not work yet...
//
// Revision 1.1  1999/10/28 20:46:18  cvs
// Changed SldlParser.jj.pre to SldlParser.jj.cc since gcc -E (used as the cpp
// preprocessor) does not recognize files that do not end in .cc!! <WEIRD>
// Had to use gcc -E (instead of cpp) since University of Kansas was lethargic
// in installing cpp by itself
// Modified Makefile to incorporate this change
//
// Revision 1.1  1999/10/27 20:43:58  cvs
// The huge commit - shifting the sldl-lrm archive from uc to kansas. The
// SldlParser.jj still has holes - but committing it anyway.
//
// Revision 1.1  1999/10/20 21:52:41  rkamath
// The HUGE commit (moving from my personal archive to the "kbse" archive)
// before it moves to Kansas...
// These files will have the log-info from my archive; they can be ignored.
// Work on this can start afresh once all the files are in place
// The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
// Revision 1.5  1999/10/20 21:21:00  rkamath
// Commit before the archive moves over to Kansas. This is not complete - and
// may not compile...
//
// Revision 1.4  1999/10/01 20:02:12  rkamath
// Need to complete the semantic checking for operators...
//
// Revision 1.3  1999/10/01 15:24:25  rkamath
// Commit for record and semantic checking added in jj.pre
//
// Revision 1.2  1999/07/28 01:58:30  rkamath
// Small changes to reflect use of SYM_SymboltableEntryList inplace of
// SOM_Declaration
//
// Revision 1.1  1999/07/27 19:23:18  rkamath
// Included pre-processing stuff so that the jj file can be generated
// according to the functionality demanded. Hence the current grammar will be
// in SldlParser.jj.pre which will be preprocessed to give SldlParser.jj
//
//
//=============================================================================


//*****************************************************************************
// The JavaCC options are here
//*****************************************************************************
options {
  STATIC = false; // Multi-threading
#ifdef DEBUG
  DEBUG_PARSER = true;
  ERROR_REPORTING = true; //Slower performance
#else
  DEBUG_PARSER = false;
  ERROR_REPORTING = false; //Faster performance
#endif
  OPTIMIZE_TOKEN_MANAGER = true;
  OUTPUT_FILE = "../html/SldlParserBnf.html"; /* for JJDoc - JAVACC doesn't
												 recognize this option */
}


//****************************************************************************/
// This is the Parser Generation code
//****************************************************************************/
PARSER_BEGIN(SldlParser)
package src;

import SOM.*;
import SYM.*;
import util.*;
import java.io.Reader;
import java.lang.Exception;

public class SldlParser extends Object {

#ifdef SEMCHECK
  private SYM_SymboltableManager manager;
  private boolean isADeclaration = false;
  private boolean isATypeDeclaration = false;
#endif

  public SldlParser(Reader fileReader, String managerName) {
    this(fileReader);
#ifdef SEMCHECK
	createSymboltableManager(managerName);
#endif
  }

#ifdef SEMCHECK
  /** To get the line number for error reporting. Note that this may not
   * always give right results since this function will be called at the
   * semantic check level - a level often higher than the one at which
   * the parsing was done
   * @return A string with the current linenumber and a colon followed
   * by a space
   */
  private String getLine() {
	return new String("Line " + token.beginLine + ": ");
  }
	
  /** To create a symboltable manager.
   * @param name The String which is to be the name of the manager
   */
  private void createSymboltableManager(String name) {
	manager = new SYM_SymboltableManager(name);
  }

  /** To process a SOM_InterfaceListing. If the mode is SOM_Mode.IN each
   * interfaceItem is checked to be SOM_Mode.IN.
   * @param interfaceListing The SOM_InterfaceListing to be processed
   * @param int the mode that each interfaceItem is to be checked for.
   * @return void
   */
  private void processInterfaceListing(SOM_InterfaceListing interfaceListing,
                                      int modeIn) {
    ListEnumeration interfaceEnum = interfaceListing.interfaces();
    SOM_InterfaceItem currItem;
    SOM_Mode mode;
    ListEnumeration idEnum;
    SOM_TypeDeclaration currTypeDecl;

    while (interfaceEnum.hasMoreElements()) {
      //Now process each interfaceItem
      try{ 
        currItem = (SOM_InterfaceItem)interfaceEnum.nextElement();

        if (modeIn == SOM_Mode.IN) {
          mode = currItem.getMode();
          if (mode != null && mode.getMode() != SOM_Mode.IN)
			throw new SYM_InvalidModeException
			  ("Expecting mode IN for interface!"); 
        }
        // else don't worry!

        // Commented out the old code using lookUpUniqueForKind
        // SOM_TypeDeclaration typeDecl = (SOM_TypeDeclaration)
        //  manager.lookUpUniqueForKind(currtypeName, SOM_Declaration.TYPE);
		
        SOM_DeclarationList typeDeclList = 
          manager.lookUpForKind(currItem.getTypeName(), SOM_Declaration.TYPE);

        // Since the type resolution must be unique, choose the first one only
        SOM_TypeDeclaration typeDecl = 
           (SOM_TypeDeclaration)typeDeclList.first();
				try {
					manager.addDeclaration(new SOM_VariableDeclaration 
										  (currItem.getIdentifier(),typeDecl));
					// Also need to set the currItem's type - will help in
					// processing AssociationListing for a formalName
					currItem.setType(typeDecl);

				} catch (SYM_IllegalRedeclarationException except) {
					System.err.println(getLine() + except);
				}
      } catch (SYM_NoDeclarationException except) {
        System.err.println(getLine() + except);
      } 
    }
  }
#endif
}

PARSER_END(SldlParser)


//*****************************************************************************
// These are the token recognition patterns
//*****************************************************************************

//-----------------------------------------------------------------------------
// "White Space" patterns
//-----------------------------------------------------------------------------
SKIP : {
	" "  |
	"\t" |
	"\n" |
	"\r"
}

//-----------------------------------------------------------------------------
// C++ style comment
//-----------------------------------------------------------------------------
SPECIAL_TOKEN : {
	<COMMENT: "/*" (~["*"] | "*"~["/"])* "*/" | 
            "//" (~["\n","\r"])* ("\n"|"\r"|"\r\n") >
}

//-----------------------------------------------------------------------------
// Reserved words and symbols
//-----------------------------------------------------------------------------
TOKEN [IGNORE_CASE] : {
  <ALL: "ALL" > |
  <ARRAY: "ARRAY"> |
  <ASSUMPTION: "ASSUMPTION" > |
  <BEGIN: "BEGIN" > |
  <BUNCH: "BUNCH" > |
  <COLON: ":" > |
  <COMMA: "," > |
  <COMPONENT: "COMPONENT" > |
  <DEFINITION: "DEFINITION" > |
  <DESIGN: "DESIGN" > |
  <DOT: "." > |
  <ELSE: "ELSE" > |
  <END: "END" > |
  <ENDIF: "ENDIF" > |
  <EXISTS: "EXISTS"> |
  <EXPORT: "EXPORT" > |
  <FORALL: "FORALL"> |
  <FACET: "FACET" > |
  <IF: "IF" > |
  <IN: "IN" > |
  <INOUT: "INOUT" > |
  <IS: "IS" > |
  <LBRACE: "{" > |
  <LET: "LET" > |
  <LFUNCDELIMITER: "<*" > |
  <LPAREN: "(" > |
  <LSQUARE: "[" > |
  <MEMBER: "::" > |
  <OUT: "OUT" > |
  <PACKAGE: "PACKAGE" > |
  <PIPE: "|" > |
  <RBRACE: "}" > |
  <RECORD: "RECORD" > |
  <RFUNCDELIMITER: "*>" > |
  <RPAREN: ")" > |
  <RSQUARE: "]" > |
  <SEPARATOR: ";" > |
  <SEQUENCE: "SEQUENCE" > |
  <SET: "SET" > |
  <SINGLEQUOTE: "`" > |
  <RAN: "RAN"> |
  <SEL: "SEL"> |
  <THEN: "THEN" > |
  <TUPLE: "TUPLE" > |
  <SUBTYPE: "SUBTYPE" > |
  <USE: "USE" > |
  <VAR: "VAR" > |
  <VERIFICATION: "VERIFICATION" >
}

//-----------------------------------------------------------------------------
// Operators
//-----------------------------------------------------------------------------
TOKEN [IGNORE_CASE] : {
	<AND: "AND" > |
	<ARROW: "->" > |
	<AT: "@" > |
	<CATENATION: ";;" > |
	<DIVIDE: "/" > |
	<DOLLAR: "$"> |
	<EQEQ: "==" > |
	<EQUAL: "=" > |
	<EXPONENT: "^" > |
	<GREATER: ">" > |
	<GREATEREQUAL: ">=" > |
	<HASH: "#" > |
	<IMPLIES: "=>" | "implies"> |
	<IMPLIEDBY: "<=" > |
	<LESS: "<" >|
    <LESSEQUAL: "=<" > |
	<MAX: "max" > |
	<MIN: "min" > |
	<MINUS: "-" > |
	<MINUSMINUS: "--" > |
	<MULTIPLY: "*" > |
	<NAND: "NAND" > |
	<NMAX: "nmax" > |
	<NMIN: "nmin" > |
	<NOR: "NOR" > |
	<NOT: "NOT" > |
	<NOTEQUAL: "/=" > |
	<NULL: "NULL" > |
	<OR: "OR" > |
	<PERCENT: "%" > |
	<PLUS: "+" > |
	<PLUSPLUS: "++" > |
	<STARSTAR: "**" > |
	<TICK: "'" > |
	<TILDA: "~" > |
	<UNDERSCORE: "_" > |
	<XNOR: "XNOR" > |
	<XOR: "XOR" > 
}

//-----------------------------------------------------------------------------
// Other primitive patterns
//-----------------------------------------------------------------------------
TOKEN : {
	<IDENTIFIER: <letter> (("_")? <letter_or_digit>)* > |
	<CHARACTER_LITERAL: "`" <letter> "`" > |
	<CONSTANT_LITERAL: "`" <IDENTIFIER> "`" > |
	<INTEGER_LITERAL: (<MINUS>)? <integer> > |
	<FLOAT_LITERAL: (<MINUS>)? <integer> (<DOT> <integer>)* (("e" | "E") ("+"|"-")? <integer>)? > |
	<#integer: (<digit>)+ > |
	<#letter_or_digit: <letter> | <digit> > |
	<#letter: ["a"-"z","A"-"Z"] > |
	<#digit: ["0"-"9"] > 
}


//****************************************************************************/
// These are the SLDL grammar productions
//****************************************************************************/

SOM_DesignFile designFile() :
{
	SOM_DesignFile dFile = new SOM_DesignFile();
	SOM_DesignUnit dUnit;
}
{
	( dUnit = designUnit()
		{
			dFile.addDesignUnit(dUnit);
		}
	)+
	<EOF>
	{
  	return dFile;
	}
}

SOM_DesignUnit designUnit() :
{
	SOM_DesignUnit dUnit;
}
{
 (
	// added by krishna on Feb 10, 2000
 	dUnit = packageDeclaration() |
 	dUnit = useClause()
 )
 {
	return dUnit;
 }
}


//****************************************************************************/
// <facet_declaration> syntax
//****************************************************************************/

SOM_ExportBlock exportBlock() :
{
    SOM_ExportClause eClause;
    SOM_ExportBlock eBlock = new SOM_ExportBlock();
}
{

	(LOOKAHEAD(2)
	 <EXPORT> <ALL> <SEPARATOR> 
   {
    eBlock.setExportAll(); 
   } |
   (eClause = exportClause()
    {
     eBlock.addToExportClauseList(eClause);
    }
   )+
	 )
	{
	 return eBlock;
	}
}

SOM_ExportClause exportClause() :
{
 SOM_IdentifierListing idList;
 SOM_ExportClause eClause = new SOM_ExportClause();
}
{
 <EXPORT>
 idList = identifierListing()
 {
  eClause.setIdentifierListing(idList);
 }
 <SEPARATOR>
 {
  return eClause;
 }
}

SOM_FacetBody facetBody() :
{
 SOM_FacetBodyItem item;
 SOM_FacetBody body = new SOM_FacetBody();
}
{
 (item = facetBodyItem()
  {
	body.addToFacetBodyItemList(item);
  }
 )+
 {
  return body;
 }
}

SOM_FacetBodyItem facetBodyItem() :
{
 SOM_FacetBodyItem item = new SOM_FacetBodyItem();
 SOM_LabelledExpression labelExpr;
}
{
  labelExpr = labelledExpression() <SEPARATOR> 
  {
	item.setLabelledExpression(labelExpr);
  }
 {
  return item;
 }
}

SOM_FacetDeclaration facetDeclaration() :
{
 SOM_Identifier id;
 SOM_InterfaceListing interfaceList;
 SOM_FacetDefinition facetDef;
 SOM_FacetDeclaration decl = new SOM_FacetDeclaration(manager.getFacetType());
}
{
  <FACET>
  id = identifier()
  { 
   decl.setIdentifier(id);
  }
#ifdef SEMCHECK
  {
	// Note that we don't add the decl to the manager yet!! If we do, it
	// will be visible inside the facet allowing us to instantiate the
	// facet within itself!!
	manager.createNewCurrentSymboltable(id.toString(),decl);
	decl.setMySymboltable(manager.getCurrentSymboltable());
  }
#endif
  [ interfaceList = interfaceListing()
	{
	  decl.setInterfaceListing(interfaceList);
	}
#ifdef SEMCHECK
	{
	  try {
		processInterfaceListing(interfaceList, SOM_Mode.INOUT);
	  } catch (SYM_InvalidModeException except) {
		System.err.println(getLine() + except);
	  }
	}
#endif
  ]
	<IS>
  // Note that decl has to be passed down to facetDef so that the
  // facetDefinition can be added to the facet before parsing
  // for facetDefinition begins. This is necessary to enable correct lookup
  // within the parents for decl
  facetDefinition(decl)
	[
	 id = identifier()
	  {
		try {
		  if (!decl.toString().equals(id.toString()))
		  throw new SOM_NameMismatchException
		  ("Closing name \"" + id.toString() + 
		   "\" does not match declaration name \"" + decl.toString() + "\"");
		} catch (SOM_NameMismatchException except) {
		  // catch the same exception that we just threw
		  System.err.println(getLine() + except);
		}
	  }
	]
	<SEPARATOR>
#ifdef SEMCHECK
  {
	try {
	  manager.getCurrentSymboltable().processExports();
	} catch (SYM_NoDeclarationException except) {
	  // if an illegal label is exported
	  System.err.println(getLine() + except);
	}
	manager.serializeCurrentSymboltable(id.toString());
	manager.doneWithCurrentSymboltable();
	// finally add the decl!
	try {
	  manager.addDeclaration(decl);
	} catch (SYM_IllegalRedeclarationException except) {
	  System.err.println(getLine() + except);
	}
  }
#endif
  {
	return decl;
  }
}

SOM_FacetDeclarationStatement facetDeclarationStatement
                              (SOM_DeclarationStatement declStmt) :
{
    SOM_FacetExpression facetExpr;
    SOM_FacetDeclarationStatement decl = new SOM_FacetDeclarationStatement();
	SOM_Identifier id = declStmt.getIdentifier();
	SOM_IdentifierListing idList = declStmt.getIdentifierListing();
}
{
 <FACET>
#ifdef SEMCHECK
	{
	  try {
		manager.addDeclaration(new SOM_VariableDeclaration
							   (id, manager.getFacetType()));
	  } catch (SYM_IllegalRedeclarationException except) {
		System.err.println(getLine() + except);
	  }
	  if (idList != null) {
		ListEnumeration facetEnum = idList.identifiers();
		SOM_Identifier facetId;
		while (facetEnum.hasMoreElements()) {
		  facetId = (SOM_Identifier)facetEnum.nextElement();
		  System.out.println("In facetDeclaration, facetId: " + 
							 facetId.toString());
		  try {
			manager.addDeclaration(new SOM_VariableDeclaration
									   (facetId, manager.getFacetType()));
		  } catch (SYM_IllegalRedeclarationException except) {
			System.err.println(getLine() + except);
		  }
		}
	  }
	  isADeclaration = false;
	}
#endif
 [ 
  <IS>
  facetExpr = facetExpression()
  {
	decl.setFacetExpression(facetExpr);
  }
 ]
   /*#ifdef SEMCHECK
 {
  // Separate out each facet declaration and add to the symbol table
  ListEnumeration idEnum = idList.identifiers();
  SOM_FacetDeclaration facetDecl;
  SOM_Identifier id;
  while (idEnum.hasMoreElements()) {
    facetDecl = new SOM_FacetDeclaration(manager.getFacetType());
    id = (SOM_Identifier)idEnum.nextElement();
    facetDecl.setIdentifier(id);
    try {
      manager.addDeclaration(facetDecl);
    } catch (SYM_IllegalRedeclarationException except) {
      System.err.println(getLine() + except);
    }
    manager.createNewCurrentSymboltable(id.toString(),facetDecl);
	  facetDecl.setMySymboltable(manager.getCurrentSymboltable());
    if (interList != null) {
      // Add each element from interList into the facet's symboltable
      facetDecl.setInterfaceListing(interList);
      try {
        processInterfaceListing(interList,SOM_Mode.INOUT);
      } catch (SYM_InvalidModeException except) {
        System.err.println(getLine() + except);
      }
    }
    manager.doneWithCurrentSymboltable();
  }
 }
#endif*/
 {
  return decl;
 }
}

SOM_FacetDeclarativeBlock facetDeclarativeBlock() :
{
 SOM_FacetDeclarativeItem item;
 SOM_FacetDeclarativeBlock declBlock = new SOM_FacetDeclarativeBlock();
}
{
 (item = facetDeclarativeItem()
  {
   if (item instanceof SOM_VariableDeclarationStatement) {
     // now separate out the variables...
     // <id> :: <type name>;

     SOM_DeclarationStatement varDecl = (SOM_DeclarationStatement)item;
     ListEnumeration idEnum = varDecl.getIdentifierListing().identifiers();
     SOM_TypeDeclaration typeDecl = varDecl.getType();
     SOM_Name typeName = varDecl.getTypeName();
     SOM_Identifier currId;
     SOM_VariableDeclaration decl;

     while (idEnum.hasMoreElements()) {
       currId = (SOM_Identifier)idEnum.nextElement();
       decl = new SOM_VariableDeclaration(currId,typeDecl);
       decl.setTypeName(typeName);
       declBlock.addToFacetDeclarativeItemList(decl);
     }
   }
   else {
     declBlock.addToFacetDeclarativeItemList(item);
   }
  }
 )+
 {
  return declBlock;
 }
}

SOM_FacetDeclarativeItem facetDeclarativeItem() :
{
 SOM_VariableDeclarationStatement varDecl;
 SOM_FacetDeclarativeItem item = null;
}
{
 (
  item = facetDeclaration() |
  //item = typeDeclaration() |
  item = declarationStatement() |
  item = useClause()
 )

 {
  return item;
 }
}

// Note that decl has to be passed down to facetDef so that the
// facetDefinition can be added to the facet before parsing
// for facetDefinition begins. This is necessary to enable correct lookup
// within the parents for decl
void facetDefinition(SOM_FacetDeclaration decl) :
{
 SOM_FacetDeclarativeBlock declBlock;
 SOM_ExportBlock exportBlock;
 SOM_FacetExpression domExpr;
 SOM_FacetBody body;
 SOM_FacetDefinition facetDef = new SOM_FacetDefinition();
}
{
 {
	 // First add the facetDef to the decl before doing any parsing
    decl.setFacetDefinition(facetDef);
 }
 [declBlock = facetDeclarativeBlock()
  {
   facetDef.setFacetDeclarativeBlock(declBlock);
  }
 ]
 [exportBlock = exportBlock()
  {
   facetDef.setExportBlock(exportBlock);
  }
 ]
 <BEGIN>

 (domExpr = facetExpression()

  // commented out by krishna on Feb 16, 2000
#ifdef SEMCHECK
   {
	 // Commented out this code after having found a better way to unfold
	 // the overload resolution
	 //	if (SYM_StaticFunctions.selectTreeNode(domExpr.getTypeList(),
	 //	manager.getFacetType()) == null
	 // ) 
	 //throw new SYM_TypeInconsistencyException
	 //("Expecting a FacetExpression"); 
			 
	try {
	  //	  domExpr.resolveFinalType(manager.getFacetType());
	  facetDef.setDomainExpression(domExpr);
	} catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except + 
						 "\n\tExpecting a FACET expression for the domain");
	} catch (NullPointerException except) {
	  // will be thrown if a particular name was not located in the
	  // Symboltable. Safely ignore since a NoDeclarationException has
	  // been printed out already
	  // System.err.println(getLine() + except + " - to be ignored");
	}
  }
#endif

  | <NULL> 
 )
 [
  body = facetBody()
  {
   facetDef.setFacetBody(body);
  }
 ]
 <END>

}


// added by krishna on Feb 15, 2000
//****************************************************************************/
// <facet_expression> syntax
//****************************************************************************/

SOM_FacetExpression facetExpression() :
{
	SOM_FacetExpression facetExpr = new SOM_FacetExpression();
	SOM_FacetAndExpression facetAndExpr;
	SOM_FacetExpressionTail facetExprTail;
}
{
	facetAndExpr = facetAndExpression()
	{
	  facetExpr.setFacetAndExpression(facetAndExpr);
	}

	(
	 facetExprTail = facetExpressionTail()
		{
			facetExpr.addToFacetExpressionTailList(facetExprTail);
		}
	)*

	{
		try {
		  facetExpr.semCheck();
		  facetExpr.resolveFinalType(manager.getFacetType());
		} catch (SYM_UnresolvableException except) {
		  System.err.println(getLine() + except);
		} catch (NullPointerException except) {
		  // This may be thrown by semCheck if a declaration is not found
		  // Do nothing - a NoDeclarationException will already have
		  // been thrown 
		  System.err.println(getLine() + except + 
		  				 " - in facetExpression to be ignored");
		}
	}
	{
		return facetExpr;
	}
}

SOM_FacetExpressionTail facetExpressionTail() :
{
	Token tok;
	SOM_DyadicOperator op = null;
	SOM_FacetAndExpression facetAndExpr;
	SOM_FacetExpressionTail facetExprTail = new SOM_FacetExpressionTail();
}
{
	(tok=<IMPLIES> { op = new SOM_ImpliesOperator();  } |
	 tok=<OR>      { op = new SOM_OrOperator();   }
	)		
#ifdef SEMCHECK
 { op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
                                            SOM_Declaration.FUNCTION)
                     ); 
 }
#endif
	{
		facetExprTail.setImpliesOperator(op);
	}

  facetAndExpr = facetAndExpression()
  {
   facetExprTail.setFacetAndExpression(facetAndExpr);
  }
  {
   return facetExprTail;
  }
}

SOM_FacetAndExpression facetAndExpression() :
{
  SOM_FacetSelectedName facetSelectedName;
  SOM_FacetAndExpressionTail facetAndExprTail;
  SOM_FacetAndExpression facetAndExpr = new SOM_FacetAndExpression();
}
{
  facetSelectedName = facetSelectedName()
  {
   facetAndExpr.setFacetSelectedName(facetSelectedName);
  }
  (LOOKAHEAD(2)
   facetAndExprTail = facetAndExpressionTail()
   {
    facetAndExpr.addToFacetAndExpressionTailList(facetAndExprTail);
   }
  )*
	{
	  try {
		facetAndExpr.semCheck();
	  } catch (SYM_UnresolvableException except) {
		System.err.println(getLine() + except);
	  } catch (NullPointerException except) {
		// This may be thrown by semCheck if a declaration is not found
		// Do nothing - a NoDeclarationException will already have been thrown
		System.err.println(getLine() + except + 
						   " - in facetAndExpression to be ignored");
	  }
	}

  {
   return facetAndExpr;
  }
}


SOM_FacetAndExpressionTail facetAndExpressionTail() :
{
  Token tok;
  SOM_DyadicOperator op = null;
  SOM_FacetSelectedName facetSelectedName;
  SOM_FacetAndExpressionTail facetAndExprTail = new SOM_FacetAndExpressionTail();
}
{
  tok=<AND>
  {
	op = new SOM_AndOperator();
	facetAndExprTail.setOperator(op);
  }
#ifdef SEMCHECK
 { op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
                                            SOM_Declaration.FUNCTION)
                     ); 
 }
#endif
  facetSelectedName = facetSelectedName()
  {
	facetAndExprTail.setFacetSelectedName(facetSelectedName);
  }
  {
	return facetAndExprTail;
  } 
}

SOM_FacetSelectedName facetSelectedName() :
{
	Token tok;
	SOM_MonoadicOperator op = null;
	//SOM_SelectedName selectedName;
	SOM_Name name;
	SOM_FacetSelectedName facetSelectedName = new SOM_FacetSelectedName();
}
{
	[ tok = <NOT>
	{ op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
                                            SOM_Declaration.FUNCTION)
                     ); 
	  op = new SOM_NotOperator();
	  facetSelectedName.setOperator(op);
	}
	]

	//selectedName = selectedName()
	name = name()
	{
		//facetSelectedName.setSelectedName(selectedName);
		facetSelectedName.setName(name);
		name.setDeclarations(manager.lookUp(name));		
		try {
			facetSelectedName.semCheck();
		} catch (SYM_UnresolvableException except) {
			System.err.println(getLine() + except);
		} catch (NullPointerException except) {
			// This may be thrown by semCheck if a declaration is not found
			// Do nothing - a NoDeclarationException will already have
			// been thrown 
			System.err.println(getLine() + except + 
							   " - in facetSelectedName to be ignored");
		}	
	}
	{
		return facetSelectedName;
	}
}


// modified by makarand and krishna on Feb 15, 2000
//****************************************************************************/
// <component_declaration> syntax
//****************************************************************************/

SOM_ComponentDeclaration componentDeclaration() :
{
 SOM_Identifier id;
 SOM_InterfaceListing interList;
 SOM_ComponentDeclarativeBlock compDeclBlock;
 SOM_ExportBlock expBlock;

 /* commented out by makarand on Feb 14, 2000
 //SOM_FacetDeclaration facetDecl;
 //SOM_FacetDefinition facetDef;
 */

 // Added by makarand to incorporate new component structure
 SOM_AssumptionBlock assumpBlock;
 SOM_DefinitionBlock defBlock;
 SOM_VerificationBlock verifBlock;

 SOM_ComponentDeclaration decl = new SOM_ComponentDeclaration
   (manager.getComponentType());
}
{
 <COMPONENT>
 id = identifier()
 { 
  decl.setIdentifier(id);
 }
#ifdef SEMCHECK
 {
   manager.createNewCurrentSymboltable(id.toString(),decl);
   decl.setMySymboltable(manager.getCurrentSymboltable());
 }
#endif
 [ interList = interfaceListing()
   {
	 decl.setInterfaceListing(interList);
   }
#ifdef SEMCHECK
   {
	 try {
	   processInterfaceListing(interList,SOM_Mode.INOUT);
	 } catch (SYM_InvalidModeException except) {
	   System.err.println(getLine() + except);
	 }
   }
#endif
 ]
   <IS>
 [ compDeclBlock = componentDeclarativeBlock()
   {
	 decl.setComponentDeclarativeBlock(compDeclBlock);
   }
 ]
   [ expBlock = exportBlock()
	 {
	   decl.setExportBlock(expBlock);
	 }
   ]
   
   // added by makarand
   assumpBlock = assumptionBlock()
 {
   decl.setAssumptionBlock(assumpBlock);
 }
 
 defBlock = definitionBlock()
 {
   decl.setDefinitionBlock(defBlock);
 }

 verifBlock = verificationBlock()
 {
   decl.setVerificationBlock(verifBlock);
 }

 <END>
 [ id = identifier() 
   {
	 try {
	   if (!decl.toString().equals(id.toString()))
	   throw new SOM_NameMismatchException("Closing name \"" + 
			 id.toString() + "\" does not match declaration name \"" 
			 + decl.toString() + "\"");
	 } catch (SOM_NameMismatchException except) {
	   // catch the same exception that we just threw
	   System.err.println(getLine() + except);
	 }
   }
 ]
   <SEPARATOR>
 
#ifdef SEMCHECK
 {
   try {
	 manager.getCurrentSymboltable().processExports();
   } catch (SYM_NoDeclarationException except) {
	 // if an illegal label is exported
	 System.err.println(getLine() + except);
   }
   manager.serializeCurrentSymboltable(id.toString());
   manager.doneWithCurrentSymboltable();
   // finally add the decl!
   try {
	 manager.addDeclaration(decl);
   } catch (SYM_IllegalRedeclarationException except) {
	 System.err.println(getLine() + except);
   }
 }
#endif
 
 {
  return decl;
 }
}

// added by makarand on Feb 14, 2000
SOM_AssumptionBlock assumptionBlock() :
{
  SOM_FacetDeclaration facetDecl;
  SOM_FacetExpression facetexpr;
  SOM_AssumptionBlock assumpBlock = new SOM_AssumptionBlock();
}
{
  <ASSUMPTION>
  <IS>

	(LOOKAHEAD(2)
	{ 
		facetDecl = new SOM_FacetDeclaration(manager.getFacetType()); 
		facetDecl.setIdentifier(new SOM_Identifier("assumption")); 
	} 
	facetDefinition(facetDecl) 
	{ 
		assumpBlock.setFacetDefinition(facetDecl); 
	}

	[
	 <ASSUMPTION>
	]
	|
	 facetexpr = facetExpression()
		{
			assumpBlock.setFacetExpression(facetexpr);
		}
	)
	<SEPARATOR>
	{
		return assumpBlock;
	}
}

// added by makarand on Feb 14, 2000
SOM_DefinitionBlock definitionBlock() :
{
  SOM_FacetDeclaration facetDecl;
  SOM_FacetExpression facetexpr;
  SOM_DefinitionBlock defBlock = new SOM_DefinitionBlock();
}
{
  <DEFINITION>
  <IS>

  (LOOKAHEAD(2)
	{ 
		facetDecl = new SOM_FacetDeclaration(manager.getFacetType()); 
		facetDecl.setIdentifier(new SOM_Identifier("definition")); 
	} 
	facetDefinition(facetDecl) 
	{ 
		defBlock.setFacetDefinition(facetDecl); 
	} 
	
	[
	 <DEFINITION>
	]
	|
	 facetexpr = facetExpression()
		{
			defBlock.setFacetExpression(facetexpr);
		}
	)
  <SEPARATOR>
  {
    return defBlock;
  }
}

// added by makarand on Feb 14, 2000
SOM_VerificationBlock verificationBlock() :
{
 
  SOM_FacetDeclaration facetDecl;
  SOM_FacetExpression facetexpr;
  SOM_VerificationBlock verifBlock = new SOM_VerificationBlock();
}
{
  <VERIFICATION>
  <IS>

  (LOOKAHEAD(2)
 	 { 
		 facetDecl = new SOM_FacetDeclaration(manager.getFacetType()); 
		 facetDecl.setIdentifier(new SOM_Identifier("verification")); 
	 } 
	 facetDefinition(facetDecl) 
	 { 
		 verifBlock.setFacetDefinition(facetDecl); 
	 } 
	 [
		<VERIFICATION>
	 ]
		|
		facetexpr = facetExpression()
		{
			verifBlock.setFacetExpression(facetexpr);
		}
	)
	<SEPARATOR>
  {
    return verifBlock;
  }
}

SOM_ComponentDeclarativeBlock componentDeclarativeBlock() :
{
 SOM_ComponentDeclarativeItem item;
 SOM_ComponentDeclarativeBlock block = new SOM_ComponentDeclarativeBlock();
}
{
 (item = componentDeclarativeItem()
  {
   if (item instanceof SOM_VariableDeclarationStatement) {
     // now separate out the variables...
     // <id> :: <type name> ;

     SOM_VariableDeclarationStatement varDecl = 
       (SOM_VariableDeclarationStatement)item;
     ListEnumeration idEnum = varDecl.getIdentifierListing().identifiers();
     SOM_TypeDeclaration typeDecl = varDecl.getType();
     SOM_Name typeName = varDecl.getTypeName();
     SOM_Identifier currId;
     SOM_VariableDeclaration decl;

     while (idEnum.hasMoreElements()) {
       currId = (SOM_Identifier)idEnum.nextElement();
       decl = new SOM_VariableDeclaration(currId,typeDecl);
       decl.setTypeName(typeName);
       block.addToComponentDeclarativeItemList(decl);
     }
   }
   else {
     block.addToComponentDeclarativeItemList(item);
   }
  }
 )+
 {
  return block;
 }
}

SOM_ComponentDeclarativeItem componentDeclarativeItem() :
{
 SOM_ComponentDeclarativeItem item = null;
}
{
 (
  item = declarationStatement() |
  item = useClause()
 )
 {
  return item;
 }
}

// added by krishna on Feb 10, 2000
//****************************************************************************/
// <package_declaration> syntax
//****************************************************************************/

SOM_PackageDeclaration packageDeclaration() :
{
	SOM_PackageDeclaration decl = 
		new SOM_PackageDeclaration(manager.getPackageType());
	SOM_Identifier id;
	SOM_InterfaceListing interfaceList;
	SOM_PackageDefinition packageDef;
}
{
  <PACKAGE>
	 id = identifier()
   {
	 decl.setIdentifier(id);
   }
#ifdef SEMCHECK
  {
	// Note that we don't add the decl to the manager yet!! If we do, it
	// will be visible inside the package allowing us to instantiate the
	// package within itself!!
	manager.createNewCurrentSymboltable(id.toString(),decl);
	decl.setMySymboltable(manager.getCurrentSymboltable());
  }
#endif
  [ interfaceList = interfaceListing()
	{
	  decl.setInterfaceListing(interfaceList);
	}
#ifdef SEMCHECK
	{
	  try {
		processInterfaceListing(interfaceList,SOM_Mode.INOUT);
	  } catch (SYM_InvalidModeException except) {
		System.err.println(getLine() + except);
	  }
	}
#endif
  ]
  <IS>
  // Note that decl has to be passed down to packageDef so that the
  // packageDefinition can be added to the package before parsing
  // for packageDefinition begins. This is necessary to enable
  // correct lookup within the parents for decl
  packageDefinition(decl)
  [
   id = identifier()
   {
	 try {
	   if (!decl.toString().equals(id.toString()))
	   throw new SOM_NameMismatchException
	   ("Closing name \"" + id.toString() + 
	   "\" does not match declaration name \"" + decl.toString() + "\""); 
	 } catch (SOM_NameMismatchException except) {
	   // catch the same exception that we just threw
	   System.err.println(getLine() + except);
	 }
   }
  ]
  <SEPARATOR>
#ifdef SEMCHECK
	{
	  try {
		manager.getCurrentSymboltable().processExports();
	  } catch (SYM_NoDeclarationException except) {
		// if an illegal label is exported
		System.err.println(getLine() + except);
	  }
	  manager.serializeCurrentSymboltable(id.toString());
	  manager.doneWithCurrentSymboltable();
	  // finally add the decl!
	 try {
	   manager.addDeclaration(decl);
	 } catch (SYM_IllegalRedeclarationException except) {
     System.err.println(getLine() + except);
   }
	}
#endif
	{
	return decl;
	}
}

// Note that decl has to be passed down to packageDef so that the
// packageDefinition can be added to the package before parsing
// for packageDefinition begins. This is necessary to enable correct lookup
// within the parents for decl
void packageDefinition(SOM_PackageDeclaration decl) :
{
 SOM_PackageDeclarativeBlock declBlock;
 SOM_ExportBlock exportBlock;
 SOM_FacetExpression domExpr;
 SOM_PackageDefinition packageDef = new SOM_PackageDefinition();
}
{
 {
	 // First add the packageDef to the decl before doing any parsing
	 decl.setPackageDefinition(packageDef);
 }

 <BEGIN>
 (domExpr = facetExpression()

#ifdef SEMCHECK
  {
		try {
		  //domExpr.resolveFinalType(manager.getFacetType());
		  packageDef.setDomainExpression(domExpr);
		} catch (SYM_UnresolvableException except) {
			System.err.println(getLine() + except + 
						  "\n\tExpecting a FACET expression for the domain");
		} catch (NullPointerException except) {
			// will be thrown if a particular name was not located in the
			// Symboltable. Safely ignore since a NoDeclarationException has
			// been printed out already
		  //System.err.println(getLine() + except + " - to be ignored");
		}
  }
#endif

 |  <NULL> 

 )

 [declBlock = packageDeclarativeBlock()
  {
   packageDef.setPackageDeclarativeBlock(declBlock);
  }
 ]
 [exportBlock = exportBlock()
  {
   packageDef.setExportBlock(exportBlock);
  }
 ]

 <END>
}

SOM_PackageDeclarativeBlock packageDeclarativeBlock() :
{
 SOM_PackageDeclarativeItem item;
 SOM_PackageDeclarativeBlock declBlock = new SOM_PackageDeclarativeBlock();
}
{
 (item = packageDeclarativeItem()
  {
   if (item instanceof SOM_VariableDeclarationStatement) {
     // now separate out the variables...
     // <id> :: <type name>;

     SOM_VariableDeclarationStatement varDecl = 
       (SOM_VariableDeclarationStatement)item;
     ListEnumeration idEnum = varDecl.getIdentifierListing().identifiers();
     SOM_TypeDeclaration typeDecl = varDecl.getType();
     SOM_Name typeName = varDecl.getTypeName();
     SOM_Identifier currId;
     SOM_VariableDeclaration decl;

     while (idEnum.hasMoreElements()) {
       currId = (SOM_Identifier)idEnum.nextElement();
       decl = new SOM_VariableDeclaration(currId,typeDecl);
       decl.setTypeName(typeName);
       declBlock.addToPackageDeclarativeItemList(decl);
     }
   }
   else {
     declBlock.addToPackageDeclarativeItemList(item);
   }
  }
 )+
 {
  return declBlock;
 }
}

SOM_PackageDeclarativeItem packageDeclarativeItem() :
{
 SOM_VariableDeclarationStatement varDecl;
 SOM_PackageDeclarativeItem item = null;
}
{
 (
  item = facetDeclaration() | 
  item = packageDeclaration() |
  item = declarationStatement() |
  item = useClause() |
  item = componentDeclaration()
 )

 {
  return item;
 }
}

// added by krishna on Feb 10, 2000
//****************************************************************************/
// <use_clause> syntax
//****************************************************************************/

SOM_UseClause useClause() :
{
 SOM_SelectedName selectedName;
 SOM_UseClause uClause = new SOM_UseClause();
}
{
  <USE>
  selectedName = selectedName()
	 {
		 uClause.addToSelectedNameList(selectedName);
	 }
#ifdef SEMCHECK
	{
	  //System.out.println("Use - selected name: " +
	  //					 selectedName.toString());
	  manager.deserializeSymboltable(selectedName.toString());
	}
#endif

	(LOOKAHEAD(2)
	 <COMMA>
	 selectedName = selectedName()
		{
			uClause.addToSelectedNameList(selectedName);
		}
	)*

	<SEPARATOR>

 {
  return uClause;
 }
}

SOM_SelectedName selectedName() :
{
	SOM_SelectedName selectedName = new SOM_SelectedName();
	SOM_Identifier id;
}
{
	id = identifier()
	 {
		 selectedName.setName((SOM_Name)id);
	 }

	(LOOKAHEAD(2)
	 <DOT>

	 id = identifier()
		{
			SOM_SelectedName selName = new SOM_SelectedName();
			selName.setName(selectedName);
			selName.setIdentifier(id);
			selectedName = selName;
		}
  )*
	{
		return selectedName;
	}
}

//****************************************************************************/
// <expression> syntax
//****************************************************************************/

SOM_Expression expression() :
{
	SOM_TermExpression termExpression;
	SOM_ExpressionTail expressionTail;
	SOM_Expression expression = new SOM_Expression();
}
{
	termExpression = termExpression()
  {
   expression.setTermExpression(termExpression);
  }
	(LOOKAHEAD(2)
   expressionTail = expressionTail()
   {
    expression.addToExpressionTailList(expressionTail);
   }
	)*

#ifdef SEMCHECK
	  {
		try {
		  expression.semCheck();
		} catch (SYM_UnresolvableException except) {
		  System.err.println(getLine() + except);
		} catch (NullPointerException except) {
		  // This may be thrown by semCheck if a declaration is not found
		  // Do nothing - a NoDeclarationException will already have
		  // been thrown 
		  //System.err.println(getLine() + except + " - to be ignored");
		}
	  }
#endif
  {
   return expression;
  }
}


SOM_ExpressionTail expressionTail() :
{
  Token tok;
  SOM_EqualEqualOperator eqEqOp = new SOM_EqualEqualOperator();
  SOM_TermExpression termExpr;
  SOM_ExpressionTail expressionTail = new SOM_ExpressionTail();
}
{
	tok=<EQEQ>  
#ifdef SEMCHECK
  {
	  eqEqOp.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
	   						 SOM_Declaration.FUNCTION)
	  					    );
  }
#endif
  {
	  expressionTail.setEqualEqualOperator(eqEqOp);
  }

  termExpr = termExpression()
  {
   expressionTail.setTermExpression(termExpr);
  }
  {
   return expressionTail;
  }
}

SOM_TermExpression termExpression() :
{
 SOM_TermExpression termExpr = new SOM_TermExpression();
 SOM_ImpliesExpression impliesExpr;
 SOM_TermExpressionTail termExprTail;
}
{
  impliesExpr = impliesExpression()
  {
	termExpr.setImpliesExpression(impliesExpr);
  }
 
 // added by krishna on Jan 12, 2000
 (LOOKAHEAD(2)
  termExprTail = termExpressionTail()
  {
	termExpr.addToTermExpressionTailList(termExprTail);
  }
 )*
#ifdef SEMCHECK
   {
	 try {
     termExpr.semCheck();
	 } catch (SYM_UnresolvableException except) {
	   System.err.println(getLine() + except);
	 } catch (NullPointerException except) {
	   // This may be thrown by semCheck if a declaration is not found
	   // Do nothing - a NoDeclarationException will already have been thrown
	   //System.err.println(getLine() + except + " - to be ignored");
	 }
   }
#endif
 {
  return termExpr;
 }
}

// added by krishna on Jan12, 2000
SOM_TermExpressionTail termExpressionTail() :
{
 SOM_TermExpressionTail termExprTail = new SOM_TermExpressionTail();
 Token tok;
 SOM_ImpliesExpression impliesExpr ;
 SOM_DyadicOperator op = null;
}
{
 ( LOOKAHEAD(2)
   tok=<EQUAL>     { op = new SOM_EqualOperator();           } |
   tok=<NOTEQUAL>  { op = new SOM_InEqualityOperator();         }
 )
#ifdef SEMCHECK
 { op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
                                            SOM_Declaration.FUNCTION)
                     ); 
 }
#endif
 {
   termExprTail.setOperator(op);
 }
 impliesExpr = impliesExpression()
 {
   termExprTail.setImpliesExpression(impliesExpr);
 }
 {
   return termExprTail;
 }
}

SOM_ImpliesExpression impliesExpression() :
{
	SOM_AndExpression andExpression;
	SOM_ImpliesExpressionTail eTail;
	SOM_ImpliesExpression impliesExpression = new SOM_ImpliesExpression();
}
{
  andExpression = andExpression()
  {
   impliesExpression.setAndExpression(andExpression);
  }
  (LOOKAHEAD(2)
   eTail = impliesExpressionTail()
   {
    impliesExpression.addToImpliesExpressionTailList(eTail);
   }
  )*
#ifdef SEMCHECK
	{
   try {
     impliesExpression.semCheck();
   } catch (SYM_UnresolvableException except) {
     System.err.println(getLine() + except);
   } catch (NullPointerException except) {
		 // This may be thrown by semCheck if a declaration is not found
		 // Do nothing - a NoDeclarationException will already have been thrown
	 //System.err.println(getLine() + except + " - to be ignored");
   }
	}
#endif
  {
   return impliesExpression;
  }
}

SOM_ImpliesExpressionTail impliesExpressionTail() :
{
	Token tok;
	SOM_DyadicOperator op = null;
	SOM_AndExpression andExpr;
	SOM_ImpliesExpressionTail tailExpr = new SOM_ImpliesExpressionTail();
}
{
	(tok=<IMPLIES>   { op = new SOM_ImpliesOperator();  } |
	 tok=<IMPLIEDBY> { op = new SOM_ImpliedByOperator();} |
	 tok=<MAX>       { op = new SOM_MaxOperator();  } |
	 tok=<NMIN>      { op = new SOM_NMinOperator(); } |
	 tok=<NMAX>      { op = new SOM_NMaxOperator(); } |
	 tok=<OR>      { op = new SOM_OrOperator();   } |
	 tok=<NOR>     { op = new SOM_NorOperator();  } |
	 tok=<XOR>     { op = new SOM_XorOperator();  } |
	 tok=<XNOR>    { op = new SOM_XnorOperator(); }
	 )		
#ifdef SEMCHECK
 { op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
                                            SOM_Declaration.FUNCTION)
                     ); 
 }
#endif
	{
		tailExpr.setImpliesOperator(op);
	}
	andExpr = andExpression()
	{
		tailExpr.setAndExpression(andExpr);
	}
	{
		return tailExpr;
	}
}

SOM_AndExpression andExpression() :
{
  SOM_RelationalTermExpression relTermExpr;
  SOM_AndExpressionTail andTail;
  SOM_AndExpression andExpr = new SOM_AndExpression();
}
{
  relTermExpr = relationalTermExpression()
  {
   andExpr.setRelationalTermExpression(relTermExpr);
  }
  (LOOKAHEAD(2)
   andTail = andExpressionTail()
   {
    andExpr.addToAndExpressionTailList(andTail);
   }
  )*
#ifdef SEMCHECK
	{
   try {
     andExpr.semCheck();
   } catch (SYM_UnresolvableException except) {
     System.err.println(getLine() + except);
   } catch (NullPointerException except) {
	 // This may be thrown by semCheck if a declaration is not found
	 // Do nothing - a NoDeclarationException will already have been thrown
	 //System.err.println(getLine() + except + " - to be ignored");
   }
	}
#endif
  {
   return andExpr;
  }
}

SOM_AndExpressionTail andExpressionTail() :
{
	Token tok;
	SOM_DyadicOperator op = null;
	SOM_RelationalTermExpression relTermExpr;
	SOM_AndExpressionTail exprTail = new SOM_AndExpressionTail();
}
{
	(tok=<AND>  { op = new SOM_AndOperator();  } |
	 tok=<NAND> { op = new SOM_NandOperator(); } |
	 tok=<MIN>  { op = new SOM_MinOperator();   }
	)
#ifdef SEMCHECK
 { op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
                                            SOM_Declaration.FUNCTION)
                     ); 
 }
#endif
 {
  exprTail.setOperator(op);
 }
 relTermExpr = relationalTermExpression()
 {
  exprTail.setRelationalTermExpression(relTermExpr);
 }
 {
  return exprTail;
 } 
}

SOM_RelationalTermExpression relationalTermExpression() :
{
 Token tok;
 SOM_SimpleExpression simpExpr;
 SOM_DyadicOperator op = null;
 SOM_RelationalTermExpressionTail relTermExprTail;
 SOM_RelationalTermExpression relExpr = new SOM_RelationalTermExpression();
}
{
 simpExpr = simpleExpression()
 {
  relExpr.setSimpleExpression(simpExpr);
 }
 (LOOKAHEAD(2)
  relTermExprTail = relationalTermExpressionTail()
  {
	relExpr.addToRelationalTermExpressionTailList(relTermExprTail);
  }
  )*
#ifdef SEMCHECK
 {
  try {
    relExpr.semCheck();
  } catch (SYM_UnresolvableException except) {
    System.err.println(getLine() + except);
  } catch (NullPointerException except) {
	// This may be thrown by semCheck if a declaration is not found
	// Do nothing - a NoDeclarationException will already have been thrown
	//System.err.println(getLine() + except + " - to be ignored");
  }
 }
#endif
 {
  return relExpr;
 }
}

SOM_RelationalTermExpressionTail relationalTermExpressionTail():
{
 Token tok;
 SOM_DyadicOperator op;
 SOM_SimpleExpression simpExpr;
 SOM_RelationalTermExpressionTail relTermExprTail = new SOM_RelationalTermExpressionTail();
}
{
    (
     tok=<GREATER> {op = new SOM_GreaterThanOperator();} |
     tok = <GREATEREQUAL> { op = new SOM_GreaterThanOrEqualOperator(); } |
     tok = <LESS> { op = new SOM_LesserThanOperator();} |
     tok = <LESSEQUAL> { op = new SOM_LesserThanOrEqualOperator(); }
     )
#ifdef SEMCHECK
 { op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
                                            SOM_Declaration.FUNCTION)
                     );
 }
#endif

	{
	    relTermExprTail.setOperator(op);
	}
    simpExpr = simpleExpression()
    {
	relTermExprTail.setSimpleExpression(simpExpr);
    }
    {
	return relTermExprTail;
    }
}

SOM_SimpleExpression simpleExpression() :
{
 Token tok;
 SOM_DyadicOperator op = null;
 SOM_Term term;
 SOM_SimpleExpressionTail exprTail;
 SOM_SimpleExpression expr = new SOM_SimpleExpression();
}
{
 term = term()
 {
  expr.setTerm(term);
 }
 (LOOKAHEAD(2)
  exprTail = simpleExpressionTail()
  {
   expr.addToSimpleExpressionTailList(exprTail);
  }
 )*
#ifdef SEMCHECK
 {
  try {
    expr.semCheck();
  } catch (SYM_UnresolvableException except) {
    System.err.println(getLine() + except);
  } catch (NullPointerException except) {
	// This may be thrown by semCheck if a declaration is not found
	// Do nothing - a NoDeclarationException will already have been thrown
	//System.err.println(getLine() + except + " - to be ignored");
  }
 }
#endif
 {
  return expr;
 }
}

SOM_SimpleExpressionTail simpleExpressionTail() :
{
 Token tok;
 SOM_DyadicOperator op;
 SOM_Term term;
 SOM_SimpleExpressionTail eTail = new SOM_SimpleExpressionTail();
}
{
 (
  tok=<PLUS>  { op = new SOM_PlusOperator();    } |
  tok=<MINUS> { op = new SOM_MinusOperator(); } |
  tok=<PLUSPLUS>   { op = new SOM_PlusPlusOperator();    } |
  tok=<MINUSMINUS> { op = new SOM_MinusMinusOperator();    } |
  tok=<CATENATION> { op = new SOM_SemiColonSemiColonOperator();    }
 )
#ifdef SEMCHECK
 {
  op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
                                           SOM_Declaration.FUNCTION)
                    );
 }
#endif
 {
  eTail.setOperator(op);
 }
	
 term = term()
 {
  eTail.setTerm(term);
 }
 {
  return eTail;
 } 
}

SOM_Term term() :
{
 SOM_ExpTerm expTerm;
 SOM_TermTail termTail;
 SOM_Term term = new SOM_Term();
}
{
  expTerm = expTerm()   // added by krishna
  {
	term.setExpTerm(expTerm);
  }

 (LOOKAHEAD(2)
  termTail = termTail()
  {
   term.addToTermTailList(termTail);
  }
 )*
#ifdef SEMCHECK
 {
  try {
    term.semCheck();
  } catch (SYM_UnresolvableException except) {
    System.err.println(getLine() + except);
  } catch (NullPointerException except) {
	// This may be thrown by semCheck if a declaration is not found
	// Do nothing - a NoDeclarationException will already have been thrown
	//System.err.println(getLine() + except + " - to be ignored");
  }
 }
#endif
 {
  return term;
 } 
}

SOM_TermTail termTail() :
{
 Token tok;
 SOM_DyadicOperator op;
 SOM_ExpTerm expTerm;
 SOM_TermTail termTail = new SOM_TermTail();
}
{
 (
  tok=<MULTIPLY> { op = new SOM_StarOperator(); } |
  tok=<DIVIDE>   { op = new SOM_SlashOperator();       } |
  tok = <STARSTAR> {op = new SOM_StarStarOperator();}
 )

#ifdef SEMCHECK
 {
  op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
                                           SOM_Declaration.FUNCTION)
                    );
 }
#endif 
 
 {
  termTail.setOperator(op);
 }
 expTerm = expTerm()   // added by krishna
 {
   termTail.setExpTerm(expTerm);
 }
 {
  return termTail;
 }
}

// added by krishna on Jan 18, 2000
// this production rule handles the exponent operator with its correct
// level of precedence
SOM_ExpTerm expTerm() :
{
 Token tok;
 SOM_Factor factor;
 SOM_DyadicOperator op;
 SOM_ExpTerm secondTerm;
 SOM_ExpTerm expTerm = new SOM_ExpTerm();
}
{
  factor = factor()
  {
	expTerm.setFactor(factor);
  }

  [LOOKAHEAD(2)
  (LOOKAHEAD(2)
   tok = <EXPONENT> {op = new SOM_CaratOperator();}|
   tok = <MEMBER> {op = new SOM_MemberOperator();}|
   tok = <IN> {op = new SOM_InOperator();}
   )
  {
	expTerm.setOperator(op);
  }
#ifdef SEMCHECK
  { 
	op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
											 SOM_Declaration.FUNCTION)
					   );
  }
#endif
	secondTerm = expTerm()
	{
	  expTerm.setExpTerm(secondTerm);
	}
   ]	

#ifdef SEMCHECK
	{
	  try {
		expTerm.semCheck();
	} catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except);
	} catch (NullPointerException except) {
	  // This may be thrown by semCheck if a declaration is not found
	  // Do nothing - a NoDeclarationException will already have been
	  // thrown
	  // System.err.println(getLine() + except + " -to be ignored");
	}
  }
#endif 
  {
	return expTerm;
  }
}

SOM_Factor factor() :
{
 SOM_Factor factor = null;
}
{
 // The production rule is
 // <factor> ::==
 //    <literal> |
 //    <grouped expression> |
 //    <array listing> |
 //    <set listing> |
 //    <unnamed function> |
 //    <labelled expression> |
 //    <atTickExpression> |
 //    <predefined unary functions> |
 //    <let expression> |
 //    <ifExpression> |
 //    <quantifiedExpression >  
 // However, <name> and <labelled expression> have a common prefix <identifier>
 // which lookahead cannot solve
 // THIS IS NOT WORKING...CHECK IT OUT....
 // Order VERY important here since that determines which production is
 // used!!

 (
  factor = literal() 
  | factor = groupedExpression() 
  
  // the following was added by krishna on Jan 27, 2000
  // this was done to allow declarations using the set & array notations
  // for eg. {a,b,c} and [1;;2;;3]
  | factor = arrayListing() 
#ifdef SEMCHECK
  {
	((SOM_ArrayListing)factor).setTypeDeclaration(manager.getArrayType());
  }
#endif
  
	//| factor = recordTupleListing()
  | factor = recordListing()

  | factor = tupleListing()

  | factor = setListing() 

  // this was added by krishna on Jan 27, 2000
  // to handle unnamed functions used within expressions
  | factor = unnamedFunction()

  | factor = atTickExpression()

  // the following was added by krishna on Jan 25, 2000
  // this was done to incorporate negation, #, %, ~ operators at the same
  // level of precedence as functions
  // the position was changed by cindy on Jan 31, 2000
  | factor = predefUnaryFunctionName() 
  
  // the following was added by krishna on Feb 6, 2000
  // this was done to incorporate "let" expressions
  | factor = letExpression()

  | factor = ifExpression()

  | factor = quantifiedExpression()

  )
  // No sem check here since it will be done in the productions
 {
  return factor;
 }
}

SOM_Literal literal() :
{
	Token tok;
}
{
	 tok = <FLOAT_LITERAL>
   {
    SOM_FloatLiteral floatLit = new SOM_FloatLiteral(tok.toString());
#ifdef SEMCHECK
    floatLit.setDeclarations(new SOM_DeclarationList(manager.getRealType()));
#endif
    return floatLit;
   } |
   tok = <INTEGER_LITERAL>
   {
    SOM_IntegerLiteral intLit = new SOM_IntegerLiteral(tok.toString());
#ifdef SEMCHECK
	String lit = tok.toString();
	if (lit.equalsIgnoreCase("0") || lit.equalsIgnoreCase("1"))
	    intLit.setDeclarations
          (new SOM_DeclarationList(manager.getBitType()));
    else
      if (lit.startsWith("-")) 
	    intLit.setDeclarations
	      (new SOM_DeclarationList(manager.getIntegerType()));
	  else
	    intLit.setDeclarations
          (new SOM_DeclarationList(manager.getNaturalType()));
#endif
    return intLit;
   } |
     (LOOKAHEAD(2)
	   tok = <CHARACTER_LITERAL>
	   {
       SOM_CharacterLiteral charLit = 
	     new SOM_CharacterLiteral(tok.toString());
#ifdef SEMCHECK
       charLit.setDeclarations(new SOM_DeclarationList
							(manager.getCharacterType()));
#endif
       return charLit;
	   }
     |
       tok = <CONSTANT_LITERAL>
       {
       SOM_ConstantLiteral constLit =
	   new SOM_ConstantLiteral(tok.toString());
#ifdef SEMCHECK
       constLit.setDeclarations(new SOM_DeclarationList
								 (manager.getIntegerType()));
#endif
       return constLit;
	   }
	 )
}

SOM_LabelledExpression labelledExpression() :
{
 SOM_Label label;
 SOM_Expression expr;
 SOM_LabelledExpression labExpr = new SOM_LabelledExpression();
}
{
 label = label()
 {
  labExpr.setLabel(label);
 }
 expr = expression()
 {
  labExpr.setExpression(expr);

#ifdef SEMCHECK
  {
	// This is a top level term. So I can unfold the overload resolution
	try {
	  expr.resolveFinalType();
	} catch (SYM_UnresolvableException except) {
	  // This means that item resolved to multiple types - or none!
	  // currently I'll leave it at that...however in future we must
	  // implement type selection rules for such toplevel terms that resolve
	  // to multiple types			 
	  System.err.println(getLine() + except);
	} catch (SYM_TypeInconsistencyException except) {
	  // This means that it reached an assignment expression where the
	  // right operand is not of matching type for the left operand
	  System.err.println(getLine() + except);
	} catch (NullPointerException except) {
	  // will be thrown if a particular name was not located in the
	  // Symboltable. Safely ignore since a NoDeclarationException has
	  // been printed out already
	  // System.err.println(getLine() + except +
	  //			 " In labelledExpression - to be ignored in the future");
	}
  }
#endif

 }
#ifdef SEMCHECK
 {
   try {
	 SOM_LabelDeclaration labelDecl;
	 //System.out.println("Type of label " + label.toString() + " is: " + 
	 //					expr.getResolvedType().toString());
	 if (!SOM_TypeDeclaration.isDistinguishable(expr.getResolvedType(), 
												manager.getFacetType())) {
	   SOM_DeclarationList declList = expr.getNames();
	   labelDecl = new SOM_LabelDeclaration(label, expr.getResolvedType(),
											(SOM_Declaration)declList.first());
	 } else {
	  //System.out.println("Not a facet type for label in labelledExpression");
	   labelDecl = new SOM_LabelDeclaration(label, expr.getResolvedType());
	 }
	 try {
	   manager.addDeclaration(labelDecl);
	   // This call succeeds on the fact that isDistinguishable() returns
	   // true for labels!
	 } catch (SYM_IllegalRedeclarationException except) {
	   System.err.println(getLine() + except);
	 }
	 labExpr.semCheck();
   } catch (RuntimeException except) {
	 System.err.println(getLine() + except);
   }
 }
#endif
 {
   return labExpr;
 }
}

SOM_AtTickExpression atTickExpression():
{
	Token tok;
    SOM_Name name;
    SOM_TickOperator tickOp;
    SOM_AtOperator atOp;
    SOM_DyadicOperator op;
    SOM_Identifier timeId;
    SOM_Identifier valueId;
    SOM_Literal literal;
    SOM_AtTickExpression atTickExp = new SOM_AtTickExpression();
}
{
  name = name()
  {
	atTickExp.setName(name);
  }
#ifdef SEMCHECK
  {
	try {
		name.setDeclarations(manager.lookUp(name));
		atTickExp.setTree(name.getTypeList());
    } catch (SYM_NoDeclarationException except) {
	  System.err.println(getLine() + except);
    } catch (NullPointerException except) {
	  //System.err.println(getLine() + except + " - to be ignored");
	}
  }
#endif

  [LOOKAHEAD(2)
  <TICK>
  { 
	tickOp = new SOM_TickOperator();
	atTickExp.setTickOperator(tickOp);
  }
  |
  (
   <AT> 
   { 
	 atOp = new SOM_AtOperator();
	 atTickExp.setAtOperator(atOp);
   }
   timeId = identifier()
   {
	 atTickExp.setTimeIdentifier(timeId);
#ifdef SEMCHECK
	 timeId.setDeclarations(manager.lookUp(timeId));
	 try {
	   timeId.resolveFinalType(manager.getRealType());
	 } catch (SYM_UnresolvableException except) {
	   System.err.println(getLine() + except);
	   System.err.println("Time variable is of wrong time type");
	 } catch (NullPointerException except) {
	   System.err.println(getLine() + except +
	  		 " In atTickExpression (timeId) - to be ignored in the future");
	 }
#endif
   }
   [LOOKAHEAD(2)
   (
	(
	 tok = <PLUS> { op = new SOM_PlusOperator(); } |
	 tok = <MINUS> { op = new SOM_MinusOperator();}
	)
	{
	  atTickExp.setOperator(op);
#ifdef SEMCHECK
      op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
											   SOM_Declaration.FUNCTION)); 
	  op.setLeftOperand(timeId);
#endif
	}
	(
	 valueId = identifier()
	 {
	   atTickExp.setValueIdentifier(valueId);
#ifdef SEMCHECK
	   valueId.setDeclarations(manager.lookUp(valueId));
	   op.setRightOperand(valueId);
	   try {
		 valueId.resolveFinalType(manager.getRealType());
	   } catch (SYM_UnresolvableException except) {
		 System.err.println(getLine() + except);
		 System.err.println("Time interval is of wrong time type");
	   } catch (NullPointerException except) {
		 System.err.println(getLine() + except +
	  		 " In atTickExpression (valueId) - to be ignored in the future");
	   }
	   
#endif
	 }
	 |
	 literal = literal()
	 {
	   atTickExp.setLiteral(literal);
	   op.setRightOperand(literal);
#ifdef SEMCHECK
	   try {
		 literal.resolveFinalType(manager.getRealType());
	   } catch (SYM_UnresolvableException except) {
		 System.err.println(getLine() + except);
		 System.err.println("Time interval is of wrong time type");
	   } catch (NullPointerException except) {
		 System.err.println(getLine() + except +
	  		 " In atTickExpression (literal) - to be ignored in the future");
	   }
#endif
	 }
	 )
#ifdef SEMCHECK
   {
	   try {
		   if (op != null) {
			   op.semCheck();
			   op.resolveFinalType();
		   }
	   } catch (NullPointerException except) {
		   System.err.println(getLine() + except +
							  " In atTickExpression (plus operator)" +
							  " - to be ignored in the future");
	   }
   }
#endif
   )
   ]
   )
  ]
	{
	  return atTickExp;
	}
}

SOM_GroupedExpression groupedExpression() :
{
 SOM_Expression expr;
 SOM_GroupedExpression grpExpr = new SOM_GroupedExpression();
}
{
#ifdef SEMCHECK
 {
  manager.openScope();
 }
#endif
 <LPAREN>
 expr = expression()
 {
  grpExpr.setExpression(expr);
 }
 <RPAREN>
#ifdef SEMCHECK
 {
   // No try catch needed here...
   grpExpr.semCheck();
   manager.closeScope();
 }
#endif
 {
  return grpExpr;
 }
}

SOM_QuantifiedExpression quantifiedExpression() : 
{
 SOM_Quantifier quant = null; 
 SOM_Expression expr; 
 SOM_QuantifiedExpression qExpr = new SOM_QuantifiedExpression(); 
 SOM_IdentifierListing idList;
 SOM_Name typeName;
 ListEnumeration idEnum;
 SOM_Identifier currId;
 SOM_InterfaceItem currItem;
} 
{ 
 ( 
  <FORALL>   { quant = new SOM_Forall(); }
  | <EXISTS> { quant = new SOM_Exists(); }  
  | <SEL>    { quant = new SOM_Sel();}
  | <MIN>    { quant = new SOM_MinFunction();}
  | <MAX>    { quant = new SOM_MaxFunction();}
  | <RAN>    { quant = new SOM_Ran();}
 ) 
 { 
   qExpr.setQuantifier(quant); 
 } 
#ifdef SEMCHECK 
 { 
  manager.openScope(); 
 } 
#endif
 
  <LPAREN>
  idList = identifierListing()
  <MEMBER>
  typeName = name()
  {
	// Now break idList into individual ids
	idEnum = idList.identifiers();
	while (idEnum.hasMoreElements()) {
	  currId = (SOM_Identifier)idEnum.nextElement();
	  currItem = new SOM_InterfaceItem();
	  currItem.setIdentifier(currId);
	  currItem.setTypeName(typeName);
	  qExpr.addInterfaceItem(currItem);

#ifdef SEMCHECK 
	  try { 
		SOM_DeclarationList typeDeclList = 
		  manager.lookUpForKind(currItem.getTypeName(), SOM_Declaration.TYPE);
		
		// Since the type resolution must be unique, choose the first one only
		SOM_TypeDeclaration typeDecl = 
		  (SOM_TypeDeclaration)typeDeclList.first();
		try {
		  manager.addDeclaration(new SOM_VariableDeclaration 
								 (currItem.getIdentifier(), typeDecl));
		  // Also need to set the currItem's type - will help in
		  // processing AssociationListing for a formalName
		  currItem.setType(typeDecl);
	   
		} catch (SYM_IllegalRedeclarationException except) {
		  System.err.println(getLine() + except);
		}
	  } catch (SYM_NoDeclarationException except) {
		System.err.println(getLine() + except);
	  }
	}
#endif
  }
  (
   <SEPARATOR>
   idList = identifierListing()
   <MEMBER>
   typeName = name()
  {
	// Now break idList into individual ids
	idEnum = idList.identifiers();
	while (idEnum.hasMoreElements()) {
	  currId = (SOM_Identifier)idEnum.nextElement();
	  currItem = new SOM_InterfaceItem();
	  currItem.setIdentifier(currId);
	  currItem.setTypeName(typeName);
	  qExpr.addInterfaceItem(currItem);
#ifdef SEMCHECK 
	  try { 
		SOM_DeclarationList typeDeclList = 
		  manager.lookUpForKind(currItem.getTypeName(), SOM_Declaration.TYPE);

		// Since the type resolution must be unique, choose the first one only
		SOM_TypeDeclaration typeDecl = 
		  (SOM_TypeDeclaration)typeDeclList.first();
		try {
		  manager.addDeclaration(new SOM_VariableDeclaration 
								 (currItem.getIdentifier(), typeDecl));
		  // Also need to set the currItem's type - will help in
		  // processing AssociationListing for a formalName
		  currItem.setType(typeDecl);
	   
		} catch (SYM_IllegalRedeclarationException except) {
		  System.err.println(getLine() + except);
		}
	  } catch (SYM_NoDeclarationException except) {
		System.err.println(getLine() + except);
	  }
#endif
	}
  }
  )*

 <PIPE>
 expr = expression() 
 { 
  qExpr.setExpression(expr); 
#ifdef SEMCHECK
  try {
	expr.resolveFinalType();
  } catch (SYM_UnresolvableException except) {
	System.err.println(getLine() + except);
  } catch (NullPointerException except) {
	System.err.println(getLine() + except + 
					   "In quantifiedExpression - to be ignored");
  }
  try {
	if (((quant instanceof SOM_Forall) || (quant instanceof SOM_Exists) ||
		 (quant instanceof SOM_Sel)) &&
		// Note: can be simplified by making use of the resolved type
		// ckong March 14, 2001
		SYM_StaticFunctions.selectTreeNode(expr.getTypeList(),
										   manager.getBooleanType()) == null)
	  throw new SYM_TypeInconsistencyException
		("Expecting a Boolean Conditional Expression");
  } catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
  } catch (NullPointerException except) {
	// This may be thrown by selectTreeNode if a declaration is not found
	// Do nothing - a NoDeclarationException will already have been
	// thrown
	// System.err.println(getLine() + except + " - to be ignored");
  }
#endif
 } 
 <RPAREN> 
#ifdef SEMCHECK 
 { 
   // No try catch needed here...
   qExpr.semCheck(); 
   manager.closeScope(); 
 } 
#endif 
 { 
  return qExpr; 
 } 
} 

SOM_IfExpression ifExpression() :
{
 SOM_Expression condExpr, thenExpr, elseExpr;
 SOM_IfExpression ifExpr = new SOM_IfExpression();
}
{
 <IF>
 
 condExpr = expression()
 {
  ifExpr.setConditionalExpression(condExpr);
 }
#ifdef SEMCHECK
 {
   try {
	 if (SYM_StaticFunctions.selectTreeNode(condExpr.getTypeList(),
											manager.getBooleanType()) == null)
	   throw new SYM_TypeInconsistencyException
		 ("Expecting a Boolean Conditional Expression");
	 condExpr.resolveFinalType(manager.getBooleanType());
   } catch (SYM_UnresolvableException except) {
	 System.err.println(getLine() + except + 
						"\n\tExpecting a BOOLEAN expression for if condition");
   } catch (NullPointerException except) {
	 System.err.println(getLine() + except + 
						" In condExpression - to be ignored in the future");
   }
 }
#endif 
 <THEN>
 
 thenExpr = expression()
 {
  ifExpr.setThenExpression(thenExpr);
 }
 
 <ELSE>
 
 elseExpr = expression()
 {
  ifExpr.setElseExpression(elseExpr);
 }

 <ENDIF>

#ifdef SEMCHECK
 {
   // ifExpr and elseExpr should have the SAME type - in the  overloaded
   // world their type intersection should be the return type of the
   // ifExpression
   try {
	 ifExpr.semCheck();
   } catch (NullPointerException except) {
	 System.err.println(getLine() + except);
   }
 }
#endif

 {
  return ifExpr;
 }
}


//****************************************************************************/
// Miscellaneous syntax
//****************************************************************************/

// Is it better to keep this guy or just impose it in the production
// for name??
SOM_ArrayListing arrayListing() : 
{
	SOM_Expression expr;
	SOM_ArrayListing arrayList = new SOM_ArrayListing();
}
{
	(LOOKAHEAD(2)
	<LSQUARE>
    expr = expression()
   {
    arrayList.addToExpressionList(expr);
#ifdef SEMCHECK
	{
	  // This is a top level term. So I can unfold the overload resolution
	  try {
		expr.resolveFinalType();
	  } catch (SYM_UnresolvableException except) {
		System.err.println(getLine() + except);
	  } catch (NullPointerException except) {
		System.err.println(getLine() + except + 
						   "In arrayListing - to be ignored");
	  }
	}
#endif
   }
   <RSQUARE>
 	)+
	{
	 return arrayList;
	}
}

SOM_AssociationListing associationListing() :
{
  SOM_AssociationListing listing = new SOM_AssociationListing();
  SOM_Expression expr;
  SOM_Underscore underscore;
}
{
  <LPAREN>
  // added by krishna on Jan 24, 2000
  (LOOKAHEAD(2)
   expr = expression() 
   {
	 listing.addToList(expr); 
   }
	 
   | <UNDERSCORE> 
   { 
     underscore = new SOM_Underscore();
	 listing.addToList(underscore); 
   }
  )

  (
   <COMMA>

   // added by krishna on Jan 24, 2000
   (LOOKAHEAD(2)
	expr = expression() 
    { 
      listing.addToList(expr); 
    }

    | <UNDERSCORE> 
    { 
	  underscore = new SOM_Underscore();
      listing.addToList(underscore); 
    }
   )
  )*
  <RPAREN>
  {
   return listing;
  }
}

SOM_Identifier identifier() :
{
	Token tok;
	SOM_Identifier id; 
}
{
	tok = <IDENTIFIER>
	{
	 id = new SOM_Identifier(tok.toString());
	 return id;
	}
}

SOM_IdentifierListing identifierListing() :
{
 SOM_Identifier id;
 SOM_IdentifierListing idList = new SOM_IdentifierListing();
}
{
	id = identifier()
  {
   idList.addToIdentifierList(id);
  }
	(
	 <COMMA>
	 id = identifier()
   {
    idList.addToIdentifierList(id);
   }
  )*
  {
   return idList;
  }
}

// This breaks up the SOM_IdentifierListing into individual variables
// and returns the cannonical form <id> :: [ <mode> ] <type name>
// Hence it returns nothing
void interfaceItem(SOM_InterfaceListing interList) :
{
	SOM_IdentifierListing varList;
	SOM_Mode mode = null;
	SOM_Name typeName;
}
{
	varList = identifierListing()
	<MEMBER>
  [
   mode = mode()
  ]
  typeName = name()
  {
   // Now break varList into individual ids
   ListEnumeration idEnum = varList.identifiers();
   SOM_Identifier currId;
   SOM_InterfaceItem currItem;
   while (idEnum.hasMoreElements()) {
     currId = (SOM_Identifier)idEnum.nextElement();
     currItem = new SOM_InterfaceItem();
     currItem.setIdentifier(currId);
     currItem.setMode(mode);
     currItem.setTypeName(typeName);
     interList.addToInterfaceItemList(currItem);
   }
  }
}

SOM_InterfaceListing interfaceListing() :
{
 SOM_InterfaceListing interfacevar = new SOM_InterfaceListing();
}
{
 <LPAREN>
 interfaceItem(interfacevar)
 (
  <SEPARATOR> // Separator is added in interfaceItem!!
  interfaceItem(interfacevar)
 )*
 <RPAREN>
 {
  return interfacevar;
 }
}

SOM_Label label() :
{
 SOM_Identifier id;
 SOM_Label label = new SOM_Label();
}
{
 id = identifier()
 {
  label.setIdentifier(id);
 }
#ifdef SEMCHECK
 {
  //Check that this is not a redeclaration of a label/id
  try {
    SOM_DeclarationList declList = manager.lookUp(id.toString());
    // If no SYM_NoDeclarationException is thrown by lookUp, then it means
    // that there is already a declaration for id
 	  throw new SYM_IllegalRedeclarationException(id.toString() + 
				" cannot be used as a label: It has been declared previously");
  }	catch (SYM_NoDeclarationException except) {
    // ignore.. it just means that no previous declaration exists!!
    // this is exactly what we want here!
  } catch (SYM_IllegalRedeclarationException except) {
	// catch the exception the I've myself thrown :)
	System.err.println(getLine() + except);
  }
 }
#endif
 <COLON>
 {
  return label;
 }
}

SOM_Mode mode() :
{
  SOM_Mode mode;
}
{
  (
   <IN>     { mode = new SOM_In();  }   |
   <OUT>    { mode = new SOM_Out(); }   |
   <INOUT>  { mode = new SOM_InOut(); } |
   <DESIGN> { mode = new SOM_Design(); }
  )
  {
   return mode;
  }
}

// Note that there can be no semcheck here! So care has to be taken to do 
// a lookup whereever name() is used in other productions.
//
// The production rule for name was
// <name> ::==
//   <identifier> |
//   <name> <DOT> <identifier> |
//   <name> <association_listing> |
//   <name> <array_listing>
//-----------------------------------------------------------------------------
// Note that this allows formation of id1(..)(..) etc!!
// I'm yet to think of a general way to handle these things in the symbol
// table - especially those dealing with instantiation - that is a beast
//-----------------------------------------------------------------------------
// Removal of left recursion yields
// <name> ::== <identifier> [ <name_z> ]
// <name_z> ::==
//   <DOT> <identifier> [ <name_z> ] |
//   <assoc> [ <name_z> ] |
//   <array> [ <name_z> ]
//
SOM_Name name() :
{
 SOM_Identifier id;
 SOM_Name toReturn;
}
{
	id = identifier()
 [ LOOKAHEAD(2) toReturn = name_z(id)
  {
   return toReturn;
  }
 ]
 {
  return id;
 }
}

// This is the production for removal of left recursion in name
// Note the order of adding to ParseOrder List is important
//
SOM_Name name_z(SOM_Name nameIn) :
{
 SOM_SelectedName selectedName;
 SOM_FormalName formalName;
 SOM_ReferencedName refName;
 SOM_Name nameZ;
 SOM_Identifier id;
 SOM_AssociationListing assocList;
 SOM_ArrayListing arrayList;
 SOM_Name toReturn;
}
{
 (LOOKAHEAD(2)
  <DOT>
  id = identifier()
  {
   selectedName = new SOM_SelectedName();
   selectedName.setName(nameIn);
   selectedName.setIdentifier(id);
   nameZ = selectedName;
  } |
  assocList = associationListing()
  {
   formalName = new SOM_FormalName();
   formalName.setName(nameIn);
   formalName.setAssociationListing(assocList);
   nameZ = formalName;
  } |
  arrayList = arrayListing()
  {
   refName = new SOM_ReferencedName();
   refName.setName(nameIn);
   refName.setArrayListing(arrayList);
   nameZ = refName;
  }
 )
 [LOOKAHEAD(2)
  toReturn = name_z(nameZ)
  {
	 return toReturn;
  }
 ]
 {
  return nameZ;
 }
}

// added by krishna on Jan 23, 2000
// this allows declarations using the set notation
SOM_SetListing setListing() :
{ 
  SOM_SetListing setListing = new SOM_SetListing();
  SOM_Expression expr;
  SOM_SetType type;
}
{
	<LBRACE>
	[LOOKAHEAD(2)
	 expr= expression()
	{ 
	  setListing.setExpression(expr);
#ifdef SEMCHECK
	  type = new SOM_SetType(manager.getSetType());
	  expr.resolveFinalType();
	  type.setMemberType(expr.getResolvedType());
	  setListing.setTypeDeclaration(type);
	  try {
		setListing.resolveFinalType();
	  } catch (SYM_UnresolvableException except) {
		// This means that item resolved to multiple types - or none!
		// currently I'll leave it at that...however in future we must
		// implement type selection rules for such toplevel terms that resolve
		// to multiple types			 
		System.err.println(getLine() + except);
	  } catch (SYM_TypeInconsistencyException except) {
		// This means that it reached an assignment expression where the
		// right operand is not of matching type for the left operand
		System.err.println(getLine() + except);
	  } catch (NullPointerException except) {
		System.err.println(getLine() + except +
					 " In setListing - to be ignored in the future");
	  }
#endif
	}
	]
	<RBRACE>

	{
	  return setListing;
	}
}
/*
// added by krishna on Jan 23, 2000
// this is called from within setTerm()
// it is used inside set Declarations
SOM_ExpressionListing expressionListing() :
{
  SOM_ExpressionListing exprListing = new SOM_ExpressionListing();
  SOM_Expression expr;
}
{
  expr = expression()
  { 
	exprListing.addToExpressionList(expr);
#ifdef SEMCHECK
	try {
	  expr.resolveFinalType();
	  exprListing.setType(expr.getResolvedType());
	} catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except);
	} catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
	} catch (NullPointerException except) {
	  System.err.println(getLine() + except +
				" In expressionListing - to be ignored in the future");
	} 
#endif
  }

 (<COMMA>
  expr = expression()
  {
	exprListing.addToExpressionList(expr);
#ifdef SEMCHECK
	try {
	  expr.resolveFinalType();
    } catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except);
	} catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
	} catch (NullPointerException except) {
	  System.err.println(getLine() + except +
				" In expressionListing - to be ignored in the future");
	}
	try {
	  if (!exprListing.setType(expr.getResolvedType())) {
		throw new SYM_TypeInconsistencyException
		("Element of expression listing of type " + expr.getResolvedType()
		 + " of wrong type.");		
	  }
	} catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
	}
#endif
  }
 )*
#ifdef SEMCHECK
   {
	 exprListing.semCheck();
	 try {
	   exprListing.resolveFinalType();
	 } catch (SYM_UnresolvableException except) {
	   // This means that item resolved to multiple types - or none!
	   // currently I'll leave it at that...however in future we must
	   // implement type selection rules for such toplevel terms that resolve
	   // to multiple types			 
	   System.err.println(getLine() + except);
	 } catch (SYM_TypeInconsistencyException except) {
	   // This means that it reached an assignment expression where the
	   // right operand is not of matching type for the left operand
	   System.err.println(getLine() + except);
	 } catch (NullPointerException except) {
	   // will be thrown if a particular name was not located in the
	   // Symboltable. Safely ignore since a NoDeclarationException has
	   // been printed out already
	   System.err.println(getLine() + except +
	  				   " In expressionListing - to be ignored in the future");
	 }
   }
#endif
  {
	return exprListing;
  }
}
*/
// added by krishna on Jan 25, 2000
// this was done to incorporate negation, #, %, ~ operators at the
// same level of precedence as functions
SOM_PredefUnaryFunctionName predefUnaryFunctionName() :
{ 
  // definition added by cindy Jan 31, 2000
  Token tok;
  SOM_PredefUnaryFunctionName predUnaryFuncName = new SOM_PredefUnaryFunctionName();
  SOM_Factor factor;
  SOM_MonoadicOperator op;
}
{
	(LOOKAHEAD(2)
	 tok = <NOT> { op = new SOM_NotOperator(); }|
	 tok = <MINUS> { op = new SOM_NegationOperator(); }|
	 tok = <TILDA> { op = new SOM_TildeOperator(); }|
	 tok = <HASH> { op = new SOM_HashOperator(); }|
	 tok = <DOLLAR> { op = new SOM_DollarOperator();}|
	 tok = <PERCENT> { op = new SOM_PercentageOperator(); }
	)
#ifdef SEMCHECK
    { op.setDeclarations(manager.lookUpForKind(tok.image.toLowerCase(),
	                                  SOM_Declaration.FUNCTION)); 
	}
#endif
	{
	  predUnaryFuncName.setOperator(op);
	}
	factor=factor()
	{ // added by cindy Jan 31, 2000
	  predUnaryFuncName.setFactor(factor);
	}
#ifdef SEMCHECK
	{
	  try {
		predUnaryFuncName.semCheck();
	  } catch (SYM_UnresolvableException except) {
		System.err.println(getLine() + except);
	  } catch (NullPointerException except) {
		// This may be thrown by semCheck if a declaration is not found
		// Do nothing - a NoDeclarationException will already have been
		// thrown
		System.err.println(getLine() + except + " -to be ignored");
	  }
	}
#endif 
	{
	  return predUnaryFuncName;
	}
}

// added by krishna on Feb 6, 2000
// this was done to incorporate "let" expressions
SOM_LetExpression letExpression() :
{
	SOM_LetExpression letExpr = new SOM_LetExpression();
	SOM_LetDeclarationStatementListing letDeclStmtList;
	SOM_Expression expr;
}
{
  <LET>
#ifdef SEMCHECK
  {
   manager.openScope();
  }
#endif
  
  <LPAREN>

  letDeclStmtList = letDeclarationStatementListing()
   {
	 letExpr.setLetDeclarationStatementListing(letDeclStmtList);
   }

  <RPAREN>
  
  <IN>
  
  expr = expression()
   {
	 letExpr.setExpression(expr);
   }
#ifdef SEMCHECK
  {
	try {
	  letExpr.resolveFinalType();
    } catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except);
	} catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
	} catch (NullPointerException except) {
	  System.err.println(getLine() + except +
				" In letExpression - to be ignored in the future");
	}
	manager.closeScope();
  }
#endif
  {
	return letExpr;
  }
}

// added by krishna on Feb 7, 2000
// this is called from within letExpression() and is used to declare
// "let" declaration Statements
SOM_LetDeclarationStatementListing letDeclarationStatementListing() :
{
  SOM_LetDeclarationStatementListing letDeclStmtList = 
	                                        new SOM_LetDeclarationStatementListing();
  SOM_LetDeclarationStatement letDeclStmt;
}
{
  letDeclStmt = letDeclarationStatement()
  {
	letDeclStmtList.addToLetDeclarationStatementList(letDeclStmt);
  }

  (LOOKAHEAD(2)
   <COMMA>
   letDeclStmt = letDeclarationStatement()
  {
	letDeclStmtList.addToLetDeclarationStatementList(letDeclStmt);
  }
   )*
	{
	  return letDeclStmtList;
	}
}

// added by krishna on Feb 6, 2000
// this is called from within letDeclarationStatementListing() and is used
// to declare constants for use within "let" expressions
SOM_LetDeclarationStatement letDeclarationStatement() :
{
  SOM_LetDeclarationStatement letDeclStmt = new SOM_LetDeclarationStatement();
  SOM_Identifier id;
  SOM_Expression expr;
  SOM_TypeDeclaration typeDecl = null;
}
{
  id = identifier()
  {
	letDeclStmt.setIdentifier(id);
  }

  <MEMBER>
	 
#ifdef SEMCHECK
	{
	  isADeclaration = true;
	}
#endif
  expr = expression()
   {
	 letDeclStmt.setTypeExpression(expr);
   }
#ifdef SEMCHECK
  {
	try {
	  typeDecl = letDeclStmt.getExpressionType();	  
      //System.out.println("Type declaration of " + id.toString() +
	  //" is " + typeDecl.toString());
	} catch (SYM_NoDeclarationException except) {
	  System.err.println(getLine() + except);
	}
	
	isADeclaration = false;
  }
#endif

  <IS>
	
  expr = expression()
   {
	 letDeclStmt.setValueExpression(expr);
   }
#ifdef SEMCHECK
  {
	// check type compatibility for assignment here...
	// almost Exactly as in overload resolution - except that the domain is
	// of one element and the type of the LHS is known
	SOM_TreeNodeList finalNodeList = new SOM_TreeNodeList();
	  
	SOM_TreeNode selectNode;
	SOM_TreeNode currNode = new SOM_TreeNode(typeDecl);

	// Now look at the declaration for id, and see which possible
	// types for the expression allow type match for assignment. Save
	// all the matches in finalNodeList - allowing for ambiguous results
	if (expr.isOfType(typeDecl)) {
	  //System.out.println("Result of isOfType is true");
	  // Prepare to add typeDecl - just as in overload resolution
	  selectNode = SYM_StaticFunctions.
		locateAmbiguousTreeNode(finalNodeList,typeDecl);
	  if (selectNode == null) finalNodeList.addElement(currNode);
	  else selectNode.setIsAmbiguous(true);
	}
	  
	try {
	  if (finalNodeList.countElements() == 0)
		throw new SYM_UnresolvableException
		  ("Expression on right hand side of assignment does not" +
		   "evaluate to the type of " + id.toString());
	  else {
		  finalNodeList.removeAmbiguousTreeNodes();
		  if (finalNodeList.countElements() != 1) 
			throw new SYM_UnresolvableException
			  ("None/Multiple type matches found for assignment, " +
			   "cannot resolve final type");
		}
		
		// otherwise the finaltype is finalNodeList.first().getResultType()
	  SOM_TypeDeclaration finalType = 
		((SOM_TreeNode)finalNodeList.first()).getResultType();
	} catch (SYM_UnresolvableException except) {
	  // Catch the same exception that we threw..or got from resolveFinalType
	  System.err.println(getLine() + except);
	}
	// Add variable to the symboltable		  
	try {
		manager.addDeclaration(new SOM_VariableDeclaration(id, typeDecl));
	  } catch (SYM_IllegalRedeclarationException except) {
		System.err.println(getLine() + except);
	  }
	
  }
#endif
  {
	return letDeclStmt;
  }
}

// added by krishna on Jan 27, 2000
// this was done to incorporate unnamed functions used from
// within an expression
SOM_UnnamedFunction unnamedFunction() :
{
  SOM_InterfaceListing interList;
  SOM_Expression exprType, exprValue;
  SOM_UnnamedFunction unnamedFunc = new SOM_UnnamedFunction();
}
{
  <LFUNCDELIMITER>
#ifdef SEMCHECK
  {
    manager.openScope();
  }
#endif
  interList = interfaceListing()
  {
	unnamedFunc.setInterfaceListing(interList);
  }
#ifdef SEMCHECK
  {  
	processInterfaceListing(interList, SOM_Mode.IN);
	isADeclaration = true;
  }
#endif 
   <MEMBER>
	exprType = expression()
    {
	  unnamedFunc.setExpressionType(exprType);
    }
#ifdef SEMCHECK
  {
	// This is a top level term. So I can unfold the overload resolution
	try {
	  exprType.resolveFinalType();
	} catch (SYM_UnresolvableException except) {
	  // This means that item resolved to multiple types - or none!
	  // currently I'll leave it at that...however in future we must
	  // implement type selection rules for such toplevel terms that resolve
	  // to multiple types			 
	  System.err.println(getLine() + except);
	} catch (SYM_TypeInconsistencyException except) {
	  // This means that it reached an assignment expression where the
	  // right operand is not of matching type for the left operand
	  System.err.println(getLine() + except);
	} catch (NullPointerException except) {
	  // will be thrown if a particular name was not located in the
	  // Symboltable. Safely ignore since a NoDeclarationException has
	  // been printed out already
	  //System.err.println(getLine() + except + " - to be ignored");
	}
	isADeclaration = false;
  }
#endif
  [
  <IS>
	exprValue = expression()
    {
	  unnamedFunc.setExpressionValue(exprValue);
    }
#ifdef SEMCHECK
  {
	// This is a top level term. So I can unfold the overload resolution
	try {
	  exprValue.resolveFinalType();
	} catch (SYM_UnresolvableException except) {
	  // This means that item resolved to multiple types - or none!
	  // currently I'll leave it at that...however in future we must
	  // implement type selection rules for such toplevel terms that resolve
	  // to multiple types			 
	  System.err.println(getLine() + except);
	} catch (SYM_TypeInconsistencyException except) {
	  // This means that it reached an assignment expression where the
	  // right operand is not of matching type for the left operand
	  System.err.println(getLine() + except);
	} catch (NullPointerException except) {
	  // will be thrown if a particular name was not located in the
	  // Symboltable. Safely ignore since a NoDeclarationException has
	  // been printed out already
	  //System.err.println(getLine() + except + " - to be ignored");
	}
	if (!(exprValue.getResolvedType()).
		isPredicateSubtypeOf(exprType.getResolvedType())) 
	  System.err.println(getLine() + 
						 "Function not returning type as expected");
  }
#endif

  ]
  <RFUNCDELIMITER>
#ifdef SEMCHECK
  {
    manager.closeScope();
  }
#endif

  {
	return unnamedFunc;
  }
}

// added by krishna on Jan 27, 2000
// this new production handles the declarations for variables, constants
// as well as function declarations
// modified by srinivas on feb28th,2000
// this is done to include the facetdeclarationstatement production
SOM_DeclarationStatement declarationStatement() :
{
  SOM_DeclarationStatement declStmt = new SOM_DeclarationStatement();
  SOM_Identifier id;
  SOM_InterfaceListing interList = null;
  SOM_IdentifierListing idList = null;
  SOM_DeclarationStatementZ declStmtZ = null;
  SOM_FacetDeclarationStatement facetDeclStmt;
  SOM_TypeDeclarationStatement typeDeclStmt;
  SOM_SetDeclarationStatement setDeclStmt;
  SOM_SequenceDeclarationStatement sequenceDeclStmt;
  SOM_ArrayDeclarationStatement arrayDeclStmt;
}
{
  id = identifier()
  {
	declStmt.setIdentifier(id);
  }
  [
   interList = interfaceListing()
	{
	  declStmt.setInterfaceListing(interList);
	}
   |
   <COMMA>
   idList = identifierListing()
  {
	declStmt.setIdentifierListing(idList);
  }
  ]
	<MEMBER>
	
	// added by krishna on Feb 6, 2000
	(LOOKAHEAD(2)
	 declStmtZ = declarationStatement_z(declStmt)
	 {
	   declStmt.setDeclarationStatementZ(declStmtZ);
	 }
	 |
	 facetDeclStmt = facetDeclarationStatement(declStmt)
	  {
		declStmt.setFacetDeclarationStatement(facetDeclStmt);
	  }
	 |
     typeDeclStmt = typeDeclarationStatement(declStmt)
 	  {
		declStmt.setTypeDeclarationStatement(typeDeclStmt);
      }
	 |
     setDeclStmt = setDeclarationStatement(declStmt)
	 {
        declStmt.setSetDeclarationStatement(setDeclStmt);
     }
     |
     sequenceDeclStmt = sequenceDeclarationStatement(declStmt)
     {
		declStmt.setSequenceDeclarationStatement(sequenceDeclStmt);
	 }
	 |
     arrayDeclStmt = arrayDeclarationStatement(declStmt)
	 {
		declStmt.setArrayDeclarationStatement(arrayDeclStmt);
     }
	 ) 	 
	<SEPARATOR>
	{
	  return declStmt;
	}
}

// added by krishna on Feb 6, 2000
// this new production invokes the declaration for either constants
// or function declarations
SOM_DeclarationStatementZ declarationStatement_z
                                (SOM_DeclarationStatement declStmt) :
{
  SOM_DeclarationStatementZ declStmtZ = new SOM_DeclarationStatementZ();
  SOM_Expression typeExpr;
  SOM_Expression valueExpr = null;
  SOM_InterfaceListing interList = declStmt.getInterfaceListing();
  SOM_IdentifierListing idList = declStmt.getIdentifierListing();
  SOM_Identifier id = declStmt.getIdentifier();
  SOM_TypeDeclaration typeDecl = null;
  SOM_TypeDeclaration valueDecl = null;
  SOM_FunctionDeclaration fncDecl = null;
  boolean isFunction = false;
}
{
  typeExpr = expression()
  {
	declStmtZ.setTypeExpression(typeExpr);
  }
#ifdef SEMCHECK
  {
	try {
		try {
	    typeExpr.resolveFinalType();
		} catch (SYM_UnresolvableException except) {
			System.err.println(getLine() + except);
		} catch (SYM_TypeInconsistencyException except) {
			System.err.println(getLine() + except);
		} catch (NullPointerException except) {
			System.err.println(getLine() + except +
				" In declarationStatementZ - to be ignored in the future");
		}
	  typeDecl = typeExpr.getResolvedType();
	  // The following is for populating the symbol table for declarations 
	  // of variables or constants
	  if (interList == null) {
		// Add first variable to the symboltable		  
		try {
		  if (typeDecl instanceof SOM_RecordType)
		    manager.addDeclaration(new SOM_RecordDeclaration(id, typeDecl));
		  else
		    manager.addDeclaration(new SOM_VariableDeclaration(id, typeDecl));
		} catch (SYM_IllegalRedeclarationException except) {
		  System.err.println(getLine() + except);
		}
		// Add rest of variables to the symboltable
		if (idList != null) {
		  ListEnumeration varEnum = idList.identifiers();
		  SOM_Identifier otherId;
		  while (varEnum.hasMoreElements()) {
			otherId = (SOM_Identifier)varEnum.nextElement();
			try {
			  if (typeDecl instanceof SOM_RecordType)
			   manager.addDeclaration(new SOM_RecordDeclaration(id, typeDecl));
			  else
			   manager.addDeclaration
				   (new SOM_VariableDeclaration(otherId, typeDecl));
			} catch (SYM_IllegalRedeclarationException except) {
			  System.err.println(getLine() + except);
			}
		  }
		}
	  } else {
		// Declaration of functions
		isFunction = true;
		fncDecl = new SOM_FunctionDeclaration();
		fncDecl.setFunctionName(id);
		manager.addFuncDeclaration(fncDecl);
		manager.openScope();
		processInterfaceListing(interList, SOM_Mode.IN);
		SOM_FunctionType fncType = 
			new SOM_FunctionType(manager.getBunchType());
		ListEnumeration interfaceEnum = interList.interfaces();
		SOM_InterfaceItem currItem;
		while (interfaceEnum.hasMoreElements()){
		  currItem = (SOM_InterfaceItem)interfaceEnum.nextElement();
		  fncType.addToDomainList(currItem.getType());
		}
		fncType.setRange(typeDecl);
		fncDecl.setType(fncType); 
	  }
	} catch (RuntimeException except) {
		System.err.println(getLine() + except);
	}
  }
#endif

#ifdef SEMCHECK
	{
	  isADeclaration = false;
	}
#endif
  [
   <IS>
   valueExpr = expression()
  {
	declStmtZ.setValueExpression(valueExpr);
#ifdef SEMCHECK
    try {
	  valueExpr.resolveFinalType();
	  valueDecl = valueExpr.getResolvedType();
	} catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except);
	} catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
	} catch (NullPointerException except) {
	  System.err.println(getLine() + except +
		" In declarationStatementZ (valueExpr) - to be ignored in the future");
	}
#endif
  }
  ]
#ifdef SEMCHECK
  {
	if (isFunction) {
	  manager.closeScope();
	}

	if (valueExpr != null) {
		try {
			if (!(SOM_TypeDeclaration.resolveFinalType(valueDecl, typeDecl)))
			  if (valueDecl != null)
				throw new SYM_TypeInconsistencyException
					("In SldlParser.DeclarationStatementZ " + 
					 "Trying to assign value of type " + valueDecl.toString() 
					 + " to variable of type " + typeDecl.toString());
              else
				throw new SYM_TypeInconsistencyException
                    ("Wrong type for initialization");
		} catch (SYM_TypeInconsistencyException except) {
			System.err.println(getLine() + except);
		} 
	}
  }
#endif
  {
	  return declStmtZ;
  }
}

// added by Cindy, June 28, 2000
// this production handles declaration of types
SOM_TypeDeclarationStatement typeDeclarationStatement
	(SOM_DeclarationStatement declStmt) :
{
  SOM_TypeDeclarationStatement typeDeclStmt = 
	  new SOM_TypeDeclarationStatement();
  Token tok;
  SOM_Expression typeExpr;
  SOM_Expression typeValue = null;
  SOM_Expression setExpr;
  SOM_Expression sequenceExpr;
  SOM_Expression arrayExpr;
  SOM_TypeDeclaration typeDecl = null;
  SOM_TypeDeclaration typeDef = null;
  SOM_TypeDeclaration valueDecl = null;
  SOM_InterfaceListing interList = declStmt.getInterfaceListing();
  SOM_IdentifierListing idList = declStmt.getIdentifierListing();
  SOM_Identifier id = declStmt.getIdentifier();
  SOM_FunctionDeclaration fncDecl = null;
  boolean isFunction = false;
}
{
  (
   tok = <SUBTYPE>
   |
   tok = <BUNCH>
  )
  {
    typeDeclStmt.setKeyWord(tok.image.toLowerCase());
  }
  <LPAREN>
  (
   typeExpr = expression()
   {
     typeDeclStmt.setType(typeExpr);
#ifdef SEMCHECK
   try {
	 typeExpr.resolveFinalType();
	 typeDecl = typeExpr.getResolvedType();
   } catch (SYM_UnresolvableException except) {
	 System.err.println(getLine() + except);
   } catch (SYM_TypeInconsistencyException except) {
	 System.err.println(getLine() + except);
   } catch (NullPointerException except) {
     System.err.println(getLine() + except +
	  " In typeDeclarationStatement (typeExpr) - to be ignored in the future");
   }
#endif
   }
   |
   <SET>
   <LPAREN>
   setExpr = expression()
   <RPAREN>
   {
     typeDeclStmt.setSetType(setExpr);
   }
   |
   <SEQUENCE>
   <LPAREN>
   sequenceExpr = expression()
   <RPAREN>
   {
     typeDeclStmt.setSequenceType(sequenceExpr);
   }
   |
   <ARRAY>
   <LPAREN>
   arrayExpr = expression()
   <RPAREN>
   {
	 typeDeclStmt.setArrayType(arrayExpr);
#ifdef SEMCHECK
   try {
	arrayExpr.resolveFinalType();
   } catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except);
   } catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
   } catch (NullPointerException except) {
	  System.err.println(getLine() + except +
		" In typeDeclarationStatement - to be ignored in the future");
   }
   SOM_ArrayType arrayType = new SOM_ArrayType(manager.getArrayType());
   arrayType.setMemberType(arrayExpr.getResolvedType());
   typeDecl = arrayType;
#endif
   }
  )
#ifdef SEMCHECK
  {
	if (interList == null) {
	  // Populating the symbol table for declarations of types
	  // Add first type to the symboltable		  
	  try {
		typeDef = new SOM_TypeDeclaration(id, manager.getBunchType()); 
		typeDef.setParentType(typeDecl);
		manager.addDeclaration(typeDef);
	  } catch (SYM_IllegalRedeclarationException except) {
		System.err.println(getLine() + except);
	  }
	  // Add rest of variables to the symboltable
	  if (idList != null) {
        ListEnumeration varEnum = idList.identifiers();
		SOM_Identifier otherId;
	    while (varEnum.hasMoreElements()) {
		  otherId = (SOM_Identifier)varEnum.nextElement();
		  try {
			typeDef = new SOM_TypeDeclaration(otherId, manager.getBunchType());
			typeDef.setParentType(typeDecl);
			manager.addDeclaration(typeDef);
		  } catch (SYM_IllegalRedeclarationException except) {
			System.err.println(getLine() + except);
		  }
		}
	  }
	} else {
	  // Declaration of functions
	  isFunction = true;
	  manager.openScope();
	  processInterfaceListing(interList, SOM_Mode.IN);
	  SOM_FunctionType fncType= new SOM_FunctionType(manager.getBunchType());
	  ListEnumeration interfaceEnum = interList.interfaces();
	  SOM_InterfaceItem currItem;
	  while (interfaceEnum.hasMoreElements()){
	    currItem = (SOM_InterfaceItem)interfaceEnum.nextElement();
	    fncType.addToDomainList(currItem.getType());
	  }
	  typeDef = new SOM_TypeDeclaration(id, manager.getBunchType()); 
	  typeDef.setParentType(typeDecl);			
	  fncType.setRange(typeDef);
	  fncDecl = new SOM_FunctionDeclaration(id, fncType); 
	}
  }
#endif

  <RPAREN>
  [
    <IS>
    typeValue = expression()
    {
      typeDeclStmt.setValue(typeValue);
#ifdef SEMCHECK
   try {
	    typeValue.resolveFinalType();
	    valueDecl = typeValue.getResolvedType();
   } catch (SYM_UnresolvableException except) {
	 System.err.println(getLine() + except);
   } catch (SYM_TypeInconsistencyException except) {
	 System.err.println(getLine() + except);
   } catch (NullPointerException except) {
     System.err.println(getLine() + except +
	 " In typeDeclarationStatement (typeValue) - to be ignored in the future");
   }
#endif
    }
  ]

#ifdef SEMCHECK
  {
	if (isFunction) {
	  manager.closeScope();
	  manager.addFuncDeclaration(fncDecl);
	}

	if (typeValue != null) {
		try {
			if (!(SOM_TypeDeclaration.resolveFinalType(valueDecl, typeDecl)))
				throw new SYM_TypeInconsistencyException
					("In SldlParser.TypeDeclarationStatement: " +
					 "Trying to assign value of type " + valueDecl.toString() 
					 + " to variable of type " + typeDecl.toString());
		} catch (SYM_TypeInconsistencyException except) {
			System.err.println(getLine() + except);
		} 
	}
  }
#endif

  {
    return typeDeclStmt;
  }
}   

// added by Cindy, June 29, 2000
// this production handles declaration of sets
SOM_SetDeclarationStatement setDeclarationStatement
	(SOM_DeclarationStatement declStmt) :
{
  SOM_SetDeclarationStatement setDeclStmt = new SOM_SetDeclarationStatement();
  Token tok;
  SOM_Expression typeExpr;
  SOM_Expression valueExpr;
  SOM_TypeDeclaration typeDecl = null;
  SOM_SetType setType;
  SOM_SetDeclaration setDecl;
  SOM_Identifier id = declStmt.getIdentifier();
  SOM_InterfaceListing interList = declStmt.getInterfaceListing();
  SOM_IdentifierListing idList = declStmt.getIdentifierListing();
  SOM_FunctionDeclaration fncDecl = null;
  boolean isFunction = false;
}
{
  tok = <SET>
  <LPAREN>
  typeExpr = expression()
  {
    setDeclStmt.setType(typeExpr);
#ifdef SEMCHECK
   try {
	typeExpr.resolveFinalType();
	typeDecl = typeExpr.getResolvedType();
   } catch (SYM_UnresolvableException except) {
	 System.err.println(getLine() + except);
   } catch (SYM_TypeInconsistencyException except) {
	 System.err.println(getLine() + except);
   } catch (NullPointerException except) {
     System.err.println(getLine() + except +
	 " In setDeclarationStatement (typeExpr) - to be ignored in the future");
   }
	setType = new SOM_SetType(manager.getSetType());
	setType.setMemberType(typeDecl);

	// The following is for populating the symbol table for declarations 
	// of variables or constants of type set
	if (interList == null) {
	  // Add first variable to the symboltable		  
	  try {
        manager.addDeclaration(new SOM_SetDeclaration(id, setType));
	  } catch (SYM_IllegalRedeclarationException except) {
	    System.err.println(getLine() + except);
	  }
	  // Add rest of variables to the symboltable
		if (idList != null) {
		  ListEnumeration varEnum = idList.identifiers();
		  SOM_Identifier otherId;
		  while (varEnum.hasMoreElements()) {
			otherId = (SOM_Identifier)varEnum.nextElement();
			try {
				manager.addDeclaration
					(new SOM_SetDeclaration(otherId, setType));
			} catch (SYM_IllegalRedeclarationException except) {
			  System.err.println(getLine() + except);
			}
		  }
		}
	} else {
	  // Declaration of functions
	  isFunction = true;
	  manager.openScope();
	  processInterfaceListing(interList, SOM_Mode.IN);
	  SOM_FunctionType fncType = 
	  	new SOM_FunctionType(manager.getBunchType());
	  ListEnumeration interfaceEnum = interList.interfaces();
	  SOM_InterfaceItem currItem;
	  while (interfaceEnum.hasMoreElements()){
	    currItem = (SOM_InterfaceItem)interfaceEnum.nextElement();
	    fncType.addToDomainList(currItem.getType());
	  }
	  fncType.setRange(setType);
	  fncDecl = new SOM_FunctionDeclaration(id, fncType); 
	}
#endif
  }
  <RPAREN>
  [
    <IS>
	 valueExpr = expression()
	 {
	   setDeclStmt.setValueExpr(valueExpr);
#ifdef SEMCHECK
	   try {
		   valueExpr.resolveFinalType();
	   } catch (SYM_UnresolvableException except) {
		   // This means that item resolved to multiple types - or none!
		   System.err.println(getLine() + except);
	   } catch (SYM_TypeInconsistencyException except) {
		   // This means that it reached an assignment expression where the
		   // right operand is not of matching type for the left operand
		   System.err.println(getLine() + except);
	   } catch (NullPointerException except) {
		   // Safely ignore since a NoDeclarationException has
		   // been printed out already
		   // System.err.println(getLine() + except +
		   //		  " In labelledExpression - to be ignored in the future");
	   }

	   try {
		   if (!(valueExpr.getResolvedType() instanceof SOM_SetType))
		     throw new SYM_TypeInconsistencyException 
		       ("Expecting set type ...");
		   if (!SOM_TypeDeclaration.resolveFinalType
			  (((SOM_SetType)valueExpr.getResolvedType()).getMemberType(), 
			   typeDecl)) 
			 throw new SYM_TypeInconsistencyException
			   ("Incorrect type for member of set ... ");
			  
		} catch (SYM_TypeInconsistencyException except) {
		  System.err.println(getLine() + except);
		} 
#endif
      }
  ]
#ifdef SEMCHECK
  {
	if (isFunction) {
	  manager.closeScope();
	  manager.addFuncDeclaration(fncDecl);
	}
  }
#endif
  {
    return setDeclStmt;
  }
}

// added by Cindy, June 29, 2000
// this production handles declaration of sequences
SOM_SequenceDeclarationStatement sequenceDeclarationStatement
	(SOM_DeclarationStatement declStmt) :
{
  SOM_SequenceDeclarationStatement sequenceDeclStmt = 
	  new SOM_SequenceDeclarationStatement();
  Token tok;
  SOM_Expression typeExpr;
  SOM_Expression valueExpr;
  SOM_TypeDeclaration typeDecl = null;
  SOM_SequenceType sequenceType;
  SOM_SequenceDeclaration sequenceDecl;
  SOM_Identifier id = declStmt.getIdentifier();
  SOM_InterfaceListing interList = declStmt.getInterfaceListing();
  SOM_IdentifierListing idList = declStmt.getIdentifierListing();
  SOM_FunctionDeclaration fncDecl = null;
  boolean isFunction = false;
}
{
   tok = <SEQUENCE>
  <LPAREN>
  typeExpr = expression()
  {
    sequenceDeclStmt.setType(typeExpr);
#ifdef SEMCHECK
   try {
	 typeExpr.resolveFinalType();
	 typeDecl = typeExpr.getResolvedType();
   } catch (SYM_UnresolvableException except) {
	 System.err.println(getLine() + except);
   } catch (SYM_TypeInconsistencyException except) {
	 System.err.println(getLine() + except);
   } catch (NullPointerException except) {
     System.err.println(getLine() + except +
	 " In sequenceDeclarationStatement (type) - to be ignored in the future");
   }
	sequenceType = new SOM_SequenceType(manager.getSequenceType());
	sequenceType.setMemberType(typeDecl);

	// The following is for populating the symbol table for declarations 
	// of variables or constants of type set
	if (interList == null) {
	  // Add first variable to the symboltable		  
	  try {
        manager.addDeclaration(new SOM_SequenceDeclaration(id, sequenceType));
	  } catch (SYM_IllegalRedeclarationException except) {
	    System.err.println(getLine() + except);
	  }
	  // Add rest of variables to the symboltable
	  if (idList != null) {
		  ListEnumeration varEnum = idList.identifiers();
		  SOM_Identifier otherId;
		  while (varEnum.hasMoreElements()) {
			otherId = (SOM_Identifier)varEnum.nextElement();
			try {
				manager.addDeclaration
					(new SOM_SequenceDeclaration(otherId, sequenceType));
			} catch (SYM_IllegalRedeclarationException except) {
			  System.err.println(getLine() + except);
			}
		  }
		}
	} else {
	  // Declaration of functions
	  isFunction = true;
	  manager.openScope();
	  processInterfaceListing(interList, SOM_Mode.IN);
	  SOM_FunctionType fncType = 
	  	new SOM_FunctionType(manager.getBunchType());
	  ListEnumeration interfaceEnum = interList.interfaces();
	  SOM_InterfaceItem currItem;
	  while (interfaceEnum.hasMoreElements()){
	    currItem = (SOM_InterfaceItem)interfaceEnum.nextElement();
	    fncType.addToDomainList(currItem.getType());
	  }
	  fncType.setRange(sequenceType);
	  fncDecl = new SOM_FunctionDeclaration(id, fncType); 
	}
#endif
  }
  <RPAREN>
  [
    <IS>
    valueExpr = expression()
    {
      sequenceDeclStmt.setValue(valueExpr);
#ifdef SEMCHECK
	   try {
		   valueExpr.resolveFinalType();
	   } catch (SYM_UnresolvableException except) {
		   // This means that item resolved to multiple types - or none!
		   System.err.println(getLine() + except);
	   } catch (SYM_TypeInconsistencyException except) {
		   // This means that it reached an assignment expression where the
		   // right operand is not of matching type for the left operand
		   System.err.println(getLine() + except);
	   } catch (NullPointerException except) {
		   // Safely ignore since a NoDeclarationException has
		   // been printed out already
		   System.err.println(getLine() + except +
		     " In sequenceDeclarationStatement - to be ignored in the future");
	   }

	   try {
		   if (!(valueExpr.getResolvedType() instanceof SOM_SequenceType))
		     throw new SYM_TypeInconsistencyException 
		       ("Expecting sequence type ...");
		   if (!SOM_TypeDeclaration.resolveFinalType
			  (((SOM_SequenceType)valueExpr.getResolvedType()).getMemberType()
			   , typeDecl)) 
			 throw new SYM_TypeInconsistencyException
			   ("Incorrect type for member of sequence ... ");
		} catch (SYM_TypeInconsistencyException except) {
		  System.err.println(getLine() + except);
		} 
#endif
    }
  ]
#ifdef SEMCHECK
  {
	if (isFunction) {
	  manager.closeScope();
	  manager.addFuncDeclaration(fncDecl);
	}
  }
#endif
  {
    return sequenceDeclStmt;
  }
}

// added by Cindy, June 29, 2000
// this production handles declaration of arrays
SOM_ArrayDeclarationStatement arrayDeclarationStatement
	(SOM_DeclarationStatement declStmt) :
{
  SOM_ArrayDeclarationStatement arrayDeclStmt = 
	  new SOM_ArrayDeclarationStatement();
  Token tok;
  SOM_Expression typeExpr;
  SOM_TypeDeclaration typeDecl = null;
  SOM_ArrayDeclaration arrayDecl;
  SOM_ArrayType arrayType;
  SOM_Expression typeValue;
  SOM_Identifier id = declStmt.getIdentifier();
  SOM_InterfaceListing interList = declStmt.getInterfaceListing();
  SOM_IdentifierListing idList = declStmt.getIdentifierListing();
  SOM_FunctionDeclaration fncDecl = null;
  boolean isFunction = false;
}
{
   tok = <ARRAY>
  <LPAREN>
  typeExpr = expression()
  {
    arrayDeclStmt.setType(typeExpr);
#ifdef SEMCHECK
  try {
	typeExpr.resolveFinalType();
	typeDecl = typeExpr.getResolvedType();
   } catch (SYM_UnresolvableException except) {
	 System.err.println(getLine() + except);
   } catch (SYM_TypeInconsistencyException except) {
	 System.err.println(getLine() + except);
   } catch (NullPointerException except) {
     System.err.println(getLine() + except +
	 " In arrayDeclarationStatement (typeExpr) - to be ignored in the future");
   }
	arrayType = new SOM_ArrayType(manager.getArrayType());
	arrayType.setMemberType(typeDecl);
	// The following is for populating the symbol table for declarations 
	// of variables or constants of type array
	if (interList == null) {
	  // Add first variable to the symboltable		  
	  try {
        manager.addDeclaration(new SOM_ArrayDeclaration(id, arrayType));
	  } catch (SYM_IllegalRedeclarationException except) {
	    System.err.println(getLine() + except);
	  }
	  // Add rest of variables to the symboltable
		if (idList != null) {
		  ListEnumeration varEnum = idList.identifiers();
		  SOM_Identifier otherId;
		  while (varEnum.hasMoreElements()) {
			otherId = (SOM_Identifier)varEnum.nextElement();
			try {
				manager.addDeclaration
					(new SOM_ArrayDeclaration(otherId, arrayType));
			} catch (SYM_IllegalRedeclarationException except) {
			  System.err.println(getLine() + except);
			}
		  }
		}
	} else {
	  // Declaration of functions
	  isFunction = true;
	  manager.openScope();
	  processInterfaceListing(interList, SOM_Mode.IN);
	  SOM_FunctionType fncType = 
	  	new SOM_FunctionType(manager.getBunchType());
	  ListEnumeration interfaceEnum = interList.interfaces();
	  SOM_InterfaceItem currItem;
	  while (interfaceEnum.hasMoreElements()){
	    currItem = (SOM_InterfaceItem)interfaceEnum.nextElement();
	    fncType.addToDomainList(currItem.getType());
	  }
	  fncType.setRange(arrayType);
	  fncDecl = new SOM_FunctionDeclaration(id, fncType); 
	}

#endif
  }
  <RPAREN>
  [
    <IS>
    typeValue = expression()
    {
      arrayDeclStmt.setValue(typeValue);
    }
  ]
#ifdef SEMCHECK
  {
	if (isFunction) {
	  manager.closeScope();
	  manager.addFuncDeclaration(fncDecl);
	}
  }
#endif
  {
    return arrayDeclStmt;
  }
}

// added by Cindy, June 20, 2000
// this production handles the declarations and usage of records 
SOM_RecordListing recordListing() :
{
  SOM_RecordListing recordList = new SOM_RecordListing();
  SOM_Identifier id;
  SOM_Expression expr;
  SOM_TypeDeclaration exprType = null;
  SOM_RecordType recordType = null;
} 
{ 
  <RECORD> 
  <LSQUARE>
  id = identifier()
  (
    <MEMBER>
	expr = expression()
    {
	  recordList.isDeclaration();
	  recordList.addToItemList(id, expr); 
#ifdef SEMCHECK
	try {
	  expr.resolveFinalType();
	  exprType = expr.getResolvedType();
      recordType = new SOM_RecordType(manager.getRecordType());
	  recordType.setFieldType(id, exprType);
    } catch (SYM_UnresolvableException except) {
		System.err.println(getLine() + except);
	} catch (SYM_TypeInconsistencyException except) {
		System.err.println(getLine() + except);
	} catch (NullPointerException except) {
		System.err.println(getLine() + except +
		" In recordListing (type)- to be ignored in the future");
	}	  
#endif
	}
	(
	 <PIPE>
	 id = identifier()
	 <MEMBER>
	 expr = expression()
	 {
	   recordList.addToItemList(id, expr);
#ifdef SEMCHECK
	try {
	  expr.resolveFinalType();
	  exprType = expr.getResolvedType();
	  recordType.setFieldType(id, exprType);
    } catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except);
	} catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
	} catch (NullPointerException except) {
      System.err.println(getLine() + except +
	  " In recordlisting (types) - to be ignored in the future");
	} 
#endif
	 }
	)*
	|
	<IS>
    expr = expression()
	{
	  recordList.addToItemList(id, expr);
#ifdef SEMCHECK
	try {
	  expr.resolveFinalType();
	  exprType = expr.getResolvedType();
      recordType = new SOM_RecordType(manager.getRecordType());
	  recordType.setFieldType(id, exprType); 
    } catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except);
    } catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
    } catch (NullPointerException except) {
      System.err.println(getLine() + except +
	  " In recordListing (values) - to be ignored in the future");
	}	  
#endif
	}
	(
     <PIPE>
	 id = identifier()
	 <IS>
	 expr = expression()
	 {
	   recordList.addToItemList(id, expr);
#ifdef SEMCHECK
	try {
	  expr.resolveFinalType();
	  exprType = expr.getResolvedType();
      recordType.setFieldType(id, exprType); 
	} catch (SYM_UnresolvableException except) {
	  System.err.println(getLine() + except);
    } catch (SYM_TypeInconsistencyException except) {
	  System.err.println(getLine() + except);
    } catch (NullPointerException except) {
      System.err.println(getLine() + except +
	 " In recordListing (values+) - to be ignored in the future");
	}
#endif
	 }
    )*
  )
  <RSQUARE>
#ifdef SEMCHECK
  {
	recordList.setTypeDeclaration(recordType);
  }
#endif
  {
	return recordList;
  }
}

// added by Cindy, June 20, 2000
// this production handles the declarations and usage of tuples
SOM_TupleListing tupleListing() :
{
  SOM_TupleListing tupleList = new SOM_TupleListing();
  SOM_Expression expr;
} 
{ 
  <TUPLE> 
  <LSQUARE>
   expr = expression()
   {
	 tupleList.addToExpressionList(expr);
   }
  (
   <PIPE>
   expr = expression()
   {
	 tupleList.addToExpressionList(expr);
   }
  )*
  <RSQUARE>
  {
	return tupleList;
  }


}

