//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: Link.java,v 1.3 2000/03/03 18:38:32 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Link.java,v $
//  Revision 1.3  2000/03/03 18:38:32  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.2  2000/02/10 03:51:12  cvs
//  Updated the Copyright information in some of the files which were omitted in the previous commit
//  - krishna
//
//  Revision 1.1  1999/10/27 20:43:59  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:52:48  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.1  1999/07/13 19:29:54  rkamath
//  The new util directory with the basic data structures
//
//
//=============================================================================
package util;

import java.lang.Object;
import java.io.*;

/** This represents a <B>Link</B> object that stores a Object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class Link
extends Object 
implements Serializable {

	private Object data;
	private Link next;
	
	/** The First Constructor. Use of this causes next to be null.
	 * @param dataIn The object to be stored at this link
	 * @return Not Applicable
	 */
	public Link(Object dataIn) {
		super();
		data = dataIn;
		next = null;
	}

	/** The Second Constructor. This is to be used only for appending
	 * the newly created Link to an already existing List! 
	 * @param dataIn The object to be stored at this link 
	 * @param prevIn The prev Link object in the List. Will throw a
	 * NullPointer Exception if not correctly initialized
	 * @return Not Applicable
	 * @see util.List
	 */
	public Link(Object dataIn, Link prevIn) {
		this(dataIn);
		prevIn.setNext(this);
	}


	/** The Third Constructor. 
	 * @param dataIn The object to be stored at this link
	 * @param nextIn The next Link object stored in the List
	 * @param prevIn The prev Link object in the List. Will throw a
	 * NullPointerException if not correctly intialized
	 * @return Not Applicable
	 * @see util.List
	 */
	public Link(Object dataIn, Link nextIn, Link prevIn) {
		data = dataIn;
		next = nextIn;
		prevIn.setNext(this);
	}

	/** To set the next Link object
	 * @param nextIn The next Link object in the List
	 * @ return void
	 */
	public void setNext(Link nextIn) {
		next = nextIn;		
	}

	/** Retreives the object stored here
	 * @return The object stored in this class
	 */
	public Object getData() {
		return data;
	}

	/** Retreives the next Link object
	 * @return The next Link object
	 */
	public Link getNext() {
		return next;
	}
}
